// cpptrain.cpp : Defines the entry point for the console application.
//


#include "stdafx.h"
#include <vector> 
#include "boxdatav2.h"
#include "box_serializer.h"
#include "svmproxy.h"
#include <svm.h>
#include "geoobj.h"

#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#include <boost/serialization/version.hpp>
#include <boost/serialization/vector.hpp>     

#pragma comment(lib,"libsvm.lib")
#pragma comment(lib , "libboost_thread-vc100-mt-gd-1_49.lib" ) 
#pragma comment(lib , "libboost_serialization-vc100-mt-gd-1_49.lib" ) 

class Tdata{

public:

	// Allow serialization to access non-public data members.    
	friend class boost::serialization::access;    
	 
	std::vector<int> list;
	std::vector<double> oo;

	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{ 
		//if(version > 0)
		//	ar & driver_name;
		ar & list;
		ar & oo;
	}

};

BOOST_CLASS_VERSION(Tdata, 1)


void testSL2(){


	Tdata t1;
	Tdata t2;

	for( int i=0;i<10;i++){
		t1.list.push_back(i);
		t1.oo.push_back(i+10.0);
	}

	t1.oo.push_back(2);
	t1.oo.push_back(4);


	// create and open a character archive for output
	std::fstream fout("tbo.bin",std::fstream::out |std::fstream::binary );
	boost::archive::binary_oarchive oa(fout);
	// write class instance to archive
	oa << t1;
	oa << t1;

	fout.close();

	// test load

	Tdata t1_re;
	Tdata t2_re;
	std::fstream fin("tbo.bin",std::fstream::in |std::fstream::binary );
	boost::archive::binary_iarchive ia(fin);
	// write class instance to archive
	ia >> t1_re;
	ia >> t2_re;

	fin.close();

}



std::vector<int> generateLabels(int labelsSize)
{
	std::vector<int> labels;
	for (int i=0; i<labelsSize; ++i)
	{
		labels.push_back(i%2+1);
	}
	return labels;
}

std::vector<std::vector<double> > generateData(int problemSize, int featureNum)
{
	std::vector<std::vector<double> > data;
	for (int i=0; i<problemSize; ++i)
	{
		std::vector<double> featureSet;
		for (int j=0; j<featureNum; ++j)
		{
			//std::cout<<"feature pushed"<<std::endl;
			featureSet.push_back(j);
		}
		data.push_back(featureSet);
	}
	return data;
}

void testSVM(){

	// prepare data
	int sizeOfProblem = 30; //number of lines with labels
	int elements = 4; //number of features for each data vector

	std::vector<std::vector<double> > data = generateData(sizeOfProblem,elements);
	std::vector<int> labels = generateLabels(sizeOfProblem);

	double * output_label = new double [sizeOfProblem];
	for (int i=0; i < labels.size(); ++i)
	{
		output_label[i] = labels[i]; 
	} 

	svm_problem prob; 
	prob.l= sizeOfProblem;
	prob.y = output_label; 
	prob.x = makeSparse(data);

	int feature_size = elements; 

	svm_parameter param = makeParm(feature_size);

	const char* checkRes = svm_check_parameter(&prob,&param);
	if(checkRes) {
		printf("ERROR in validating conversion: %s\n", checkRes);
	} 

	svm_model* model_ptr = svm_train(&prob,&param);

	double testf[4]={1,2,1,1};

	double *res_p=new double[model_ptr->nr_class];

	Eigen::Map<Eigen::MatrixXd> emap(testf,1,4);
	svm_node ** testData =makeSparse(emap);

	svm_predict_probability(model_ptr,testData[0],res_p);


	svm_save_model("",model_ptr);
	svm_model* reloadModel = svm_load_model("");


	//free
	deleteSparse(prob.x,1);
	deleteSparse(testData,1);
	delete [] res_p;
}

void testSL()
{

	const char *fp="K:\\Paper_project\\memorability_UCL\\MY_CODE\\vs12\\gui_v3\\testData\\561x427\\processed\\scenebin\\img_1030.bin";

	NaiveSceneCTX nnctx;
	PlaneData floor_ini;
	PlaneData floor_fr;
	std::vector<PlaneData> walls_fr;
	std::vector<BoxDataV2> b2d;

	loadSceneANDBoxes( 
		fp,
		nnctx,
		floor_ini,
		floor_fr,
		walls_fr,
		b2d );


}


void testLoadBin()
{
	using namespace std;

	typedef int Int32;
	typedef double Real64;
	std::cout<<"INT32 ="<<sizeof(Int32)<<"B"<<std::endl;
	std::cout<<"real64 ="<<sizeof(Real64)<<"B"<<std::endl;

	//myassert(sizeof(Int32)==4,"INT32!=32B");
	//myassert(sizeof(Real64)==8,"real64!=64B");

	const char* bpath ="K:\\Paper_project\\memorability_UCL\\MY_CODE\\results\\05_03\\fout.bin";

	std::fstream fin(bpath,std::fstream::in | std::fstream::binary);

	// read 2d matrix size
	Int32 s_row=0;
	Int32 s_col=0;
	fin.read((char*)&s_row,4);
	fin.read((char*)&s_col,4);

	Real64 *matData = new Real64[s_row*s_col];

	fin.read((char*)matData,8*s_col*s_row);

	fin.close();

	// Matlab is column major
	for (int ir = 0; ir < s_row; ir++){

		std::cout<<"["<<ir<<"] ";
		for( int ic=0; ic < s_col ; ic++ ){
			std::cout<<matData[ic*s_row+ir]<<"\t";
		}
		std::cout<<'\n';
	}


}

void testPred(){
	const char* modelFPath = "K:\\Paper_project\\memorability_UCL\\MY_CODE\\vs12\\gui_v3\\testData\\model\\m0504.model";

	svm_model* model=nullptr;
	model=svm_load_model(modelFPath);

	Eigen::MatrixXd fv;
	fv.resize(1,2);
	fv<<1,0.5;

	std::vector<double> resLids;

	Eigen::MatrixXd resProbs;

	svmPred(model,fv,resLids,resProbs);

	std::cout<<resLids[0]<<"\n\n"<<resProbs<<std::endl;
}


void loaderRes(){
	std::string root("K:\\Paper_project\\memorability_UCL\\MY_CODE\\vs12\\gui_v3\\testData\\561x427\\processed\\annoresult");
	std::vector<std::string> flist=getFileList(root.c_str(),"*.anno");

	for (int i = 0; i < flist.size() ; i++)
	{
		if( flist[i].size()<1 ){
			continue;
		}
		std::cout<<flist[i]<<std::endl;

		NaiveSceneCTX nctx;
		PlaneData floor_ini;
		PlaneData floor_fr;
		std::vector<PlaneData> walls;
		std::vector<NaiveAnnoBox> naBoxes;

		std::string ff=root+std::string("\\")+flist[i] ;

		loadAnnoBinData( ff.c_str(),nctx,floor_ini,floor_fr,walls,naBoxes,true);

	}

}

void test_svmtrain(){

	const char* bpath ="K:\\Paper_project\\memorability_UCL\\MY_CODE\\results\\05_03\\mdata_bedroom.bin";



	std::vector<double> lids;
	std::vector<std::vector<double> >fmat;

	readMatBinData(bpath,lids,fmat);


	svm_model* model=nullptr;
	svm_problem * prob=nullptr;

	svmTrain(lids,fmat,&model,&prob);
}


void trainModelANDSave(const char * DPATH , const char * SPATH)
{
	//const char* bpath ="K:\\Paper_project\\memorability_UCL\\MY_CODE\\results\\05_03\\mdata_bedroom.bin";

	std::vector<double> lids;
	std::vector<std::vector<double> >fmat;

	readMatBinData(DPATH,lids,fmat);

	svm_model* model=nullptr;
	svm_problem * prob=nullptr;

	svmTrain(lids,fmat,&model,&prob);

	//const char* modelFPath = "K:\\Paper_project\\memorability_UCL\\MY_CODE\\vs12\\gui_v3\\testData\\model\\m0504.model";
	svmSave(SPATH,model);

	svmFree(model,prob);
}

void testTrain_svm()
{
	//loaderRes();
	const char * root ="K:/Paper_project/memorability_UCL/MY_CODE/vs12/gui_v3/testData/user_study/trial_naive_v00";
	const char * allDName="";
	const char * savName="";

	char modelPath[2048]={0};
	char savPATH[2048]={0};


	allDName="start_all_svmf_h_xz.bin";
	savName="all_h_xz.svmmodel";

	memset(modelPath,0,sizeof(modelPath));
	memset(savPATH,0,sizeof(savPATH));
	sprintf(modelPath,"%s/%s",root,allDName);
	sprintf(savPATH,"%s/%s",root,savName);

	trainModelANDSave(modelPath,savPATH);

	//-------------------------------------------------------------------------

	allDName="start_floor_svmf_h_xz.bin";
	savName="floor_xz.svmmodel";

	memset(modelPath,0,sizeof(modelPath));
	memset(savPATH,0,sizeof(savPATH));
	sprintf(modelPath,"%s/%s",root,allDName);
	sprintf(savPATH,"%s/%s",root,savName);

	trainModelANDSave(modelPath,savPATH);

	//-------------------------------------------------------------------------
	allDName="start_obj_svmf_h_xz.bin";
	savName="obj_xz.svmmodel";


	memset(modelPath,0,sizeof(modelPath));
	memset(savPATH,0,sizeof(savPATH));
	sprintf(modelPath,"%s/%s",root,allDName);
	sprintf(savPATH,"%s/%s",root,savName);

	trainModelANDSave(modelPath,savPATH);
	//-------------------------------------------------------------------------

	allDName="start_wall_svmf_h_xz.bin"; 
	savName="wall_xz.svmmodel";

	memset(modelPath,0,sizeof(modelPath));
	memset(savPATH,0,sizeof(savPATH));
	sprintf(modelPath,"%s/%s",root,allDName);
	sprintf(savPATH,"%s/%s",root,savName);

	trainModelANDSave(modelPath,savPATH);

}

int main(int argc, char* argv[])
{
	testSL2();

	return 0;
}
 


