/*
MatlabWrapper

Andrej Karpathy
August 27, 2012
BSD Licence
*/

#ifndef MATLAB_WRAPPER_H__
#define MATLAB_WRAPPER_H__

#include <string>
#include <Eigen/Eigen>
#include "engine.h"

#define MATLAB_BUF_SIZE 4096

using namespace std;

/*
Lets you use Matlab from C++ effortlessly
Example usage:

Eigen::MatrixXf X = Eigen::MatrixXf::Random(3,4);
cout << X << endl;

// create wrapper and visualize tiny 3x4 random matrix
MatlabWrapper mw;
mw.sendMat("X", X);
mw.exec("imshow(X, [])", true); //true indicates that we want matlab to pause for keypress

// enter interactive session! Enter 'qq' to return 
mw.interact();

// receive matrix X back and print it to make sure it's the same
Eigen::MatrixXf mat;
mw.receiveMat("X", mat);
cout << mat << endl;
*/

class MatlabWrapper {
  
  public:
    MatlabWrapper();
    ~MatlabWrapper();
    
    // start the engine. Returns false on error.
    bool init(); 
    
    // send a matrix to matlab engine
    void sendMat(const string name, const Eigen::MatrixXf &mat);  
	void sendMat(const string name, const Eigen::MatrixXd &mat);
    
    // receive a matrix back from matlab engine
    bool receiveMat(const string name, Eigen::MatrixXf &mat) const;  
	bool receiveMat(const string name, Eigen::MatrixXd &mat) const;
	bool receiveMat(const string name, Eigen::MatrixXi &mat) const;

    // execute a command in matlab
    void exec(const string cmd, const bool waitForKey = false) const;
    
    // starts interactive mode where you can type anything and get response back from MATLAB  
    // enter qq to exit interactive mode
    void interact(); 
    
    bool verbose_;
    
	inline 
	bool isInitialed(){return initted_;}

  protected:
    
    Engine *ep_;
    bool initted_;
    char buffer_[MATLAB_BUF_SIZE]; // output buffer for matlab to talk back to us
};

#endif
