#include "stdafx.h"
#include "box_serializer.h"



void saveSceneANDBoxes( const char* FNAME , 
		NaiveSceneCTX * SCTX ,
		PlaneData const & FL_INI ,PlaneData const & FL_FR ,
		std::vector<PlaneData> const & WALLS ,
		std::vector<BoxDataV2> const & ALLBOXES )
{
	NaiveBox * nboxes= conv_BoxDataV2_to_NBox(ALLBOXES);

	// save all naive box to the file
	std::fstream fout( FNAME ,std::fstream::out | std::fstream::binary );

	//-------------------------------------------------------------------------
	// write scene

	// write flag 
	int typeFlag = STypes::SCENE;
	fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;

	// write sce
	fout.write( reinterpret_cast<char*>(SCTX) , sizeof(NaiveSceneCTX) ) ;
	 
	//-------------------------------------------------------------------------
	// write FLOOR_INI
	
	// write flag 
	typeFlag = STypes::FLOOR_INI;
	fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;

	NaivePlane nfl_ini;
	convPlData2NPL(FL_INI,nfl_ini); 

	// write floor
	fout.write( reinterpret_cast<char*>(&nfl_ini) , sizeof(NaivePlane) ) ;

	//-------------------------------------------------------------------------
	// write FLOOR_FR

	// write flag 
	typeFlag = STypes::FLOOR_FR;
	fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;

	NaivePlane nfl_fr;
	convPlData2NPL(FL_FR,nfl_fr); 

	// write floor
	fout.write( reinterpret_cast<char*>(&nfl_fr) , sizeof(NaivePlane) ) ;

	//------------------------------------------------------------------------
	// write WALLS
	if( WALLS.size() ){ 
		// write flag 
		typeFlag = STypes::WALLS_FR;
		fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;

		// write number
		const int WNUM =WALLS.size();
		fout.write( (char*)&WNUM , sizeof(WNUM) ) ; 

		NaivePlane * nwalls = new NaivePlane[WNUM];

		for( int iw=0; iw<WNUM ; iw++ ){
			convPlData2NPL(WALLS[iw],nwalls[iw]);
		}

		// write walls
		fout.write( reinterpret_cast<char*>(nwalls) , sizeof(NaivePlane)*WNUM ) ;

		delete [] nwalls;
	}
	 
	//-------------------------------------------------------------------------
	// write BOXES

	// write flag 
	typeFlag = STypes::BOX;
	fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;

	// write number
	int bnum = ALLBOXES.size();
	fout.write( (char*)&bnum , sizeof(bnum) ) ;

	// write all box data 
	fout.write( reinterpret_cast<char*>(nboxes) , sizeof(NaiveBox)*bnum ) ;
	//-------------------------------------------------------------------------

	// write END FLAG
	typeFlag = STypes::END;
	fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;
	fout.close();
	//-------------------------------------------------------------------------

	std::cout<<"SAVE DONE- at ["<<FNAME<<"]\n";

	delete [] nboxes; 
}

void loadSceneANDBoxes( const char* FNAME , 
		   NaiveSceneCTX & sctx , 
		   PlaneData & floor_ini , PlaneData & floor_fr,
		   std::vector<PlaneData> & walls,
		   std::vector<BoxDataV2> & allboxes  , bool reportMode/*=false */)
{ 

	allboxes.clear();
	 
	std::fstream fin( FNAME ,std::fstream::in | std::fstream::binary );

	if (!fin.is_open())
	{
		std::cout<<"OPEN FAILED at:"<<FNAME <<std::endl;
		throw "OPEN FAILED";
		return;
	}

	while (!fin.eof())
	{

		// read type flag
		int tflag=0;
		if( fin.read((char*)&tflag, sizeof(tflag)) ){

			if( tflag == STypes::BOX ){
				// read number
				int boxNum =0;
				fin.read((char*)&boxNum, sizeof(boxNum));

				// read all boxes
				NaiveBox * nboxes=new NaiveBox[boxNum];

				fin.read(reinterpret_cast<char*>(nboxes), sizeof(NaiveBox)*boxNum);

				//[CONV NBox 2 BoxDataV2] ...
				allboxes.clear();
				allboxes.resize(boxNum);

				for( int ib=0 ; ib < boxNum ; ib++ ){
					conv_NBox_to_BoxDataV2( nboxes[ib] , allboxes[ib]);					
				}

				delete [] nboxes;

				if( reportMode ){
					std::cout<<"[Load]:"<<"BOX :"<<"x"<<boxNum<<std::endl; 
				}

			}else if( tflag == STypes::FLOOR_INI ){
				
				NaivePlane npl;
				fin.read(reinterpret_cast<char*>(&npl), sizeof(NaivePlane));

				convNPlane2PlaneData(npl,floor_ini);

				if( reportMode ){
					std::cout<<"[Load]:"<<"FLOOR :"<<std::endl; 
				}

			}else if( tflag == STypes::FLOOR_FR ){

				NaivePlane npl;
				fin.read(reinterpret_cast<char*>(&npl), sizeof(NaivePlane));

				convNPlane2PlaneData(npl,floor_fr);

				if( reportMode ){
					std::cout<<"[Load]:"<<"FLOOR :"<<std::endl; 
				}

			}else if( tflag == STypes::WALLS_FR ){
				// read number
				int wnum =0;
				fin.read((char*)&wnum, sizeof(wnum));

				// read all boxes
				NaivePlane * npl=new NaivePlane[wnum];

				fin.read(reinterpret_cast<char*>(npl), sizeof(NaivePlane)*wnum);

				//[CONV NaivePlane 2 Plane] ...
				walls.clear();
				walls.resize(wnum);

				for( int iw=0 ; iw < wnum ; iw++ ){
					convNPlane2PlaneData( npl[iw] , walls[iw]);					
				}

				delete [] npl;

				if( reportMode ){
					std::cout<<"[Load]:"<<"WALLS :"<<"x"<<wnum<<std::endl; 
				}
			}else if( tflag == STypes::SCENE ){

				fin.read(reinterpret_cast<char*>(&sctx), sizeof(NaiveSceneCTX));

				if( reportMode ){
					std::cout<<"[Load]:"<<"SCE :"<<std::endl; 
				}
				

			}else if( tflag == STypes::END ){
				std::cout<<"Load done." <<std::endl;
				break;
			}else{ 
				std::cout<<"UNDEFINED STYPE :" << tflag <<std::endl;
				throw "UNDEFINED STYPE";
				break;
			}
		}else{
			std::cout<<"READING FAILED at:"<<FNAME <<std::endl;
			throw "READING FAILED";
			break;

		}
	}


}

void loadNaiveDatas( const char* FNAME , 
					   NaiveSceneCTX & sctx , 
					   PlaneData & floor_ini , PlaneData & floor_fr,
					   std::vector<PlaneData> & walls,
					   std::vector<BoxDataV2> & allboxes  , bool reportMode/*=false */)
{ 

	allboxes.clear();

	std::fstream fin( FNAME ,std::fstream::in | std::fstream::binary );

	if (!fin.is_open())
	{
		std::cout<<"OPEN FAILED at:"<<FNAME <<std::endl;
		throw "OPEN FAILED";
		return;
	}

	while (!fin.eof())
	{
		// read type flag
		int tflag=0;
		if( fin.read((char*)&tflag, sizeof(tflag)) ){

			if( tflag == STypes::BOX ){
				// read number
				int boxNum =0;
				fin.read((char*)&boxNum, sizeof(boxNum));

				// read all boxes
				NaiveBox * nboxes=new NaiveBox[boxNum];

				fin.read(reinterpret_cast<char*>(nboxes), sizeof(NaiveBox)*boxNum);

				//[CONV NBox 2 BoxDataV2] ...
				allboxes.clear();
				allboxes.resize(boxNum);

				for( int ib=0 ; ib < boxNum ; ib++ ){
					conv_NBox_to_BoxDataV2( nboxes[ib] , allboxes[ib]);					
				}

				delete [] nboxes;

				if( reportMode ){
					std::cout<<"[Load]:"<<"BOX :"<<"x"<<boxNum<<std::endl; 
				}

			}else if( tflag == STypes::FLOOR_INI ){

				NaivePlane npl;
				fin.read(reinterpret_cast<char*>(&npl), sizeof(NaivePlane));

				convNPlane2PlaneData(npl,floor_ini);

				if( reportMode ){
					std::cout<<"[Load]:"<<"FLOOR :"<<std::endl; 
				}

			}else if( tflag == STypes::FLOOR_FR ){

				NaivePlane npl;
				fin.read(reinterpret_cast<char*>(&npl), sizeof(NaivePlane));

				convNPlane2PlaneData(npl,floor_fr);

				if( reportMode ){
					std::cout<<"[Load]:"<<"FLOOR :"<<std::endl; 
				}

			}else if( tflag == STypes::WALLS_FR ){
				// read number
				int wnum =0;
				fin.read((char*)&wnum, sizeof(wnum));

				// read all boxes
				NaivePlane * npl=new NaivePlane[wnum];

				fin.read(reinterpret_cast<char*>(npl), sizeof(NaivePlane)*wnum);

				//[CONV NaivePlane 2 Plane] ...
				walls.clear();
				walls.resize(wnum);

				for( int iw=0 ; iw < wnum ; iw++ ){
					convNPlane2PlaneData( npl[iw] , walls[iw]);					
				}

				delete [] npl;

				if( reportMode ){
					std::cout<<"[Load]:"<<"WALLS :"<<"x"<<wnum<<std::endl; 
				}
			}else if( tflag == STypes::SCENE ){

				fin.read(reinterpret_cast<char*>(&sctx), sizeof(NaiveSceneCTX));

				if( reportMode ){
					std::cout<<"[Load]:"<<"SCE :"<<std::endl; 
				}


			}else if( tflag == STypes::END ){
				std::cout<<"Load done." <<std::endl;
				break;
			}else{ 
				std::cout<<"UNDEFINED STYPE :" << tflag <<std::endl;
				throw "UNDEFINED STYPE";
				break;
			}
		}else{
			std::cout<<"READING FAILED at:"<<FNAME <<std::endl;
			throw "READING FAILED";
			break;

		}
	}


}


#pragma region CONV_GVEC

void convCLM2EMat( float * pmat , Eigen::MatrixXf & mat , const int ROW_SIZE, const int COL_SIZE )
{
	// use COLUMN MAJOR
	
	Eigen::Map<Eigen::MatrixXf> emap(pmat,ROW_SIZE,COL_SIZE);

	mat = emap;
 
}

void copyMatrix_CLM( Eigen::MatrixXf mat , float * pmat , const int PMAT_SIZE )
{
	// use COLUMN MAJOR

	if( PMAT_SIZE==mat.size() ){

		const int RSIZE = mat.rows();
		const int CSIZE = mat.cols();

		for( int ic=0; ic < mat.cols() ; ic++ ){
			for( int ir=0 ; ir < mat.rows() ; ir++ ){
				pmat[ic*RSIZE+ir] = mat(ir,ic);
			}
		}

	}else{
		throw "EXCP .copyMatrix_CLM";
	}
}

void copyVector_CLM( Eigen::VectorXf vec , float * pmat , const int PMAT_SIZE )
{
	if( vec.cols()*vec.rows()  == PMAT_SIZE ){

		for( int i=0; i < PMAT_SIZE ; i++ ){
			pmat[i]=vec(i);
		} 

	}else{
		throw "EXCP .copyMatrix_CLM";
	}
}

void convNPlane2PlaneData( NaivePlane const & NPL , PlaneData & pldata )
{
	pldata.model.resize(4,1);
	pldata.model<<NPL.model[0],
		NPL.model[1],
		NPL.model[2],
		NPL.model[3];

	for (int iv=0 ; iv < PlaneData::CSS_VNUM ;iv++)
	{
		pldata.pts[iv]<< NPL.pts[iv*3+0],NPL.pts[iv*3+1],NPL.pts[iv*3+2];
	}
	 
}

void convPlData2NPL( PlaneData const & PLDATA , NaivePlane & NPL )
{
	for( int iv=0 ; iv < 3 ; iv++ ){
		NPL.pts[iv*3+0]=PLDATA.pts[iv](0);
		NPL.pts[iv*3+1]=PLDATA.pts[iv](1);
		NPL.pts[iv*3+2]=PLDATA.pts[iv](2);
	}


	for( int iv=0 ; iv < 4 ; iv++ ){
		NPL.model[iv]=PLDATA.model(iv);
	}
}


#pragma endregion CONV_GVEC
//-------------------------------------------------------------------------

void saveAnnoToBin( const std::string & FNAME , NaiveSceneCTX * SCTX , PlaneData const & FL_INI , PlaneData const & FL_FR , std::vector<PlaneData> const & WALLS , std::vector<BoxDataV2> const & ALLBOXES )
{
	NaiveAnnoBox * nboxes= conv_BoxDataV2_to_AnooBox(ALLBOXES);

	// save all naive box to the file
	std::fstream fout( FNAME ,std::fstream::out | std::fstream::binary );

	//-------------------------------------------------------------------------
	// write scene

	// write flag 
	int typeFlag = STypes::SCENE;
	fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;

	// write sce
	fout.write( reinterpret_cast<char*>(SCTX) , sizeof(NaiveSceneCTX) ) ;

	//-------------------------------------------------------------------------
	// write FLOOR_INI

	// write flag 
	typeFlag = STypes::FLOOR_INI;
	fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;

	NaivePlane nfl_ini;
	convPlData2NPL(FL_INI,nfl_ini); 

	// write floor
	fout.write( reinterpret_cast<char*>(&nfl_ini) , sizeof(NaivePlane) ) ;

	//-------------------------------------------------------------------------
	// write FLOOR_FR

	// write flag 
	typeFlag = STypes::FLOOR_FR;
	fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;

	NaivePlane nfl_fr;
	convPlData2NPL(FL_FR,nfl_fr); 

	// write floor
	fout.write( reinterpret_cast<char*>(&nfl_fr) , sizeof(NaivePlane) ) ;

	//------------------------------------------------------------------------
	// write WALLS
	if( WALLS.size() ){ 
		// write flag 
		typeFlag = STypes::WALLS_FR;
		fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;

		// write number
		const int WNUM =WALLS.size();
		fout.write( (char*)&WNUM , sizeof(WNUM) ) ; 

		NaivePlane * nwalls = new NaivePlane[WNUM];

		for( int iw=0; iw<WNUM ; iw++ ){
			convPlData2NPL(WALLS[iw],nwalls[iw]);
		}

		// write walls
		fout.write( reinterpret_cast<char*>(nwalls) , sizeof(NaivePlane)*WNUM ) ;

		delete [] nwalls;
	}

	//-------------------------------------------------------------------------
	// write BOXES

	// write flag 
	typeFlag = STypes::BOX;
	fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;

	// write number
	int bnum = ALLBOXES.size();
	fout.write( (char*)&bnum , sizeof(bnum) ) ;

	// write all box data 
	fout.write( reinterpret_cast<char*>(nboxes) , sizeof(NaiveAnnoBox)*bnum ) ;
	//-------------------------------------------------------------------------
	// write END FLAG
	typeFlag = STypes::END;
	fout.write( (char*)&typeFlag , sizeof(typeFlag) ) ;
	fout.close();
	//-------------------------------------------------------------------------

	std::cout<<"SAVE DONE- at ["<<FNAME<<"[\n";

	delete [] nboxes; 
} 

void loadAnnoBinData(
	const char* FNAME , 
	NaiveSceneCTX & sctx , 
	PlaneData & floor_ini , PlaneData & floor_fr,
	std::vector<PlaneData> & walls,
	std::vector<NaiveAnnoBox> & allboxes  , bool reportMode/*=false */)
{ 
	allboxes.clear();

	std::fstream fin( FNAME ,std::fstream::in | std::fstream::binary );

	if (!fin.is_open())
	{
		std::cout<<"OPEN FAILED at:"<<FNAME <<std::endl;
		throw "OPEN FAILED";
		return;
	}

	while (!fin.eof())
	{
		// read type flag
		int tflag=0;
		if( fin.read((char*)&tflag, sizeof(tflag)) ){

			if( tflag == STypes::BOX ){
				// read number
				int boxNum =0;
				fin.read((char*)&boxNum, sizeof(boxNum));

				// read all boxes
				NaiveAnnoBox * nboxes=new NaiveAnnoBox[boxNum];

				fin.read(reinterpret_cast<char*>(nboxes), sizeof(NaiveAnnoBox)*boxNum);

				//[CONV NBox 2 BoxDataV2] ...
				allboxes.clear();
				allboxes.resize(boxNum);

				for( int ib=0 ; ib < boxNum ; ib++ ){
					allboxes[ib]=nboxes[ib];				
				}

				delete [] nboxes;

				if( reportMode ){
					std::cout<<"[Load]:"<<"BOX :"<<"x"<<boxNum<<std::endl; 
				}

			}else if( tflag == STypes::FLOOR_INI ){

				NaivePlane npl;
				fin.read(reinterpret_cast<char*>(&npl), sizeof(NaivePlane));

				convNPlane2PlaneData(npl,floor_ini);

				if( reportMode ){
					std::cout<<"[Load]:"<<"FLOOR :"<<std::endl; 
				}

			}else if( tflag == STypes::FLOOR_FR ){

				NaivePlane npl;
				fin.read(reinterpret_cast<char*>(&npl), sizeof(NaivePlane));

				convNPlane2PlaneData(npl,floor_fr);

				if( reportMode ){
					std::cout<<"[Load]:"<<"FLOOR :"<<std::endl; 
				}

			}else if( tflag == STypes::WALLS_FR ){
				// read number
				int wnum =0;
				fin.read((char*)&wnum, sizeof(wnum));

				// read all boxes
				NaivePlane * npl=new NaivePlane[wnum];

				fin.read(reinterpret_cast<char*>(npl), sizeof(NaivePlane)*wnum);

				//[CONV NaivePlane 2 Plane] ...
				walls.clear();
				walls.resize(wnum);

				for( int iw=0 ; iw < wnum ; iw++ ){
					convNPlane2PlaneData( npl[iw] , walls[iw]);					
				}

				delete [] npl;

				if( reportMode ){
					std::cout<<"[Load]:"<<"WALLS :"<<"x"<<wnum<<std::endl; 
				}
			}else if( tflag == STypes::SCENE ){

				fin.read(reinterpret_cast<char*>(&sctx), sizeof(NaiveSceneCTX));

				if( reportMode ){
					std::cout<<"[Load]:"<<"SCE :"<<std::endl; 
				}


			}else if( tflag == STypes::END ){
				std::cout<<"Load done." <<std::endl;
				break;
			}else{ 
				std::cout<<"UNDEFINED STYPE :" << tflag <<std::endl;
				throw "UNDEFINED STYPE";
				break;
			}
		}else{
			std::cout<<"READING FAILED at:"<<FNAME <<std::endl;
			throw "READING FAILED";
			break;

		}
	}


}

//-------------------------------------------------------------------------
#pragma region CONV

// [Naive]
NaiveBox * conv_BoxDataV2_to_NBox(std::vector<BoxDataV2> const & ALLBOXES)
{
	NaiveBox * nboxes=new NaiveBox[ALLBOXES.size()];

	// parse box to naive box
	for( int i=0; i < ALLBOXES.size() ; i++ ){

		BoxDataV2::ConstVertex *frpts=ALLBOXES[i].getBox3d_fr();
		BoxDataV2::ConstVertex *worldpts=ALLBOXES[i].getBox3d_world(); 
		BoxDataV2::ConstVertex *aabbpts=ALLBOXES[i].getBox3d_AABB(); 

		BoxDataV2::ConstVertex *obb_fr2aabb=ALLBOXES[i].getFrOBB2AABB();
		BoxDataV2::ConstVertex *aabb2obb_fr=ALLBOXES[i].getAABB2FrOBB();

		for( int iv=0 ; iv < 24 ; iv++ ){
			nboxes[i].box3d_world[iv]=worldpts[iv];
			nboxes[i].box3d_fr[iv]=frpts[iv];
			nboxes[i].box3d_aabb[iv]=aabbpts[iv]; 
		}

		for( int iv=0 ; iv < 9 ; iv++ ){
			nboxes[i].obb_fr2aabb[iv]=obb_fr2aabb[iv];
			nboxes[i].obb_aabb2fr[iv]=aabb2obb_fr[iv];
		}

		nboxes[i].uid = ALLBOXES[i].getUID();
		nboxes[i].supType = ALLBOXES[i].getSupType();
		// local index
		nboxes[i].supLIDX = ALLBOXES[i].getSupporterLIDX();
		
		// label lid & label name
		
		if(  ALLBOXES[i].getPredTagLIDs().size() > 0){
			// lid
			nboxes[i].labelLid =ALLBOXES[i].getPredTagLIDs()[0];

			//lname
			memset(nboxes[i].labelName,0,sizeof(char)*NaiveBox::LNAME_LEN); 
			std::strcpy(nboxes[i].labelName , ALLBOXES[i].getPredTags()[0].toStdString().c_str() );
		
		}else{
			//lid
			nboxes[i].labelLid =-1;
			//lname
			memset(nboxes[i].labelName,0,sizeof(char)*NaiveBox::LNAME_LEN);
		}
				
		
		nboxes[i].closetWall_lidx = ALLBOXES[i].getClosedWallLIdx();
		nboxes[i].supByWallLidx = ALLBOXES[i].getSupWallLIdx();
		
	}

	return nboxes;
}


// [Naive]
void conv_NBox_to_BoxDataV2( NaiveBox const & NBOX , BoxDataV2 & boxv )
{
	boxv.setBox3d_aabb(NBOX.box3d_aabb);
	boxv.setBox3d_fr(NBOX.box3d_fr);
	boxv.setBox3d_world(NBOX.box3d_world);
	boxv.setObb_aabb2Fr(NBOX.obb_aabb2fr);
	boxv.setObb_fr2aabb(NBOX.obb_fr2aabb); 

	boxv.setUID(NBOX.uid);

	boxv.setSupToIni();
	boxv.setSupType(NBOX.supType);

	boxv.setSupporterLIDX(NBOX.supLIDX); 

	boxv.setSupWallLIdx(NBOX.supByWallLidx);
	boxv.setClosedWallLIdx(NBOX.closetWall_lidx);

	QList<int> lidlist;
	lidlist.push_back(NBOX.labelLid);
	boxv.setPredTagLIDs(lidlist);

	QStringList taglist;	
	taglist.push_back(QString(NBOX.labelName));

	boxv.setPredTags(taglist);

}


// [NaiveAnno]
NaiveAnnoBox * conv_BoxDataV2_to_AnooBox(std::vector<BoxDataV2> const & ALLBOXES)
{
	if( ALLBOXES.size()==0 ){
		return nullptr;
	}
	NaiveAnnoBox * nboxes=new NaiveAnnoBox[ALLBOXES.size()];

	// parse box to naive box
	for( int i=0; i < ALLBOXES.size() ; i++ ){

		BoxDataV2::ConstVertex *frpts=ALLBOXES[i].getBox3d_fr();
		BoxDataV2::ConstVertex *worldpts=ALLBOXES[i].getBox3d_world(); 
		BoxDataV2::ConstVertex *aabbpts=ALLBOXES[i].getBox3d_AABB(); 

		BoxDataV2::ConstVertex *obb_fr2aabb=ALLBOXES[i].getFrOBB2AABB();
		BoxDataV2::ConstVertex *aabb2obb_fr=ALLBOXES[i].getAABB2FrOBB();

		for( int iv=0 ; iv < 24 ; iv++ ){
			nboxes[i].box3d_world[iv]=worldpts[iv];
			nboxes[i].box3d_fr[iv]=frpts[iv];
			nboxes[i].box3d_aabb[iv]=aabbpts[iv]; 
		}

		for( int iv=0 ; iv < 9 ; iv++ ){
			nboxes[i].obb_fr2aabb[iv]=obb_fr2aabb[iv];
			nboxes[i].obb_aabb2fr[iv]=aabb2obb_fr[iv];
		}

		QList<int> const & MENU =ALLBOXES[i].getPredTagLIDs();
		
		//pre set
		for( int im=0 ; im < NaiveAnnoBox::MAX_MENU_NUM ; im++ ){
			nboxes[i].pred_menu[im]=-1;
		}
		// eff num
		int rec_nn = std::min(NaiveAnnoBox::MAX_MENU_NUM,MENU.size());
		// real set
		for( int k=0 ; k < rec_nn ; k++ ){
			nboxes[i].pred_menu[k]=MENU[k];
		}

		nboxes[i].uid = ALLBOXES[i].getUID();
		nboxes[i].supType = ALLBOXES[i].getSupType();

		// local index
		nboxes[i].supLIDX = ALLBOXES[i].getSupporterLIDX();
		

		nboxes[i].closetWall_lidx = ALLBOXES[i].getClosedWallLIdx();
		nboxes[i].supByWallLidx = ALLBOXES[i].getSupWallLIdx();

		// support relationship modified by user
		nboxes[i].box_sup_modified = ALLBOXES[i].getSupModeified();
		
		// geometry modified by user
		nboxes[i].box_geo_modified = ALLBOXES[i].getBoxModified();


		// label lid
		if(  ALLBOXES[i].getPredTagLIDs().size() > 0){
			nboxes[i].labelLid =ALLBOXES[i].getPredTagLIDs()[0];
		}else{
			nboxes[i].labelLid =-1;
		}
	}

	return nboxes;
}

#pragma endregion CONV

//-------------------------------------------------------------------------

void export_sce2matlab( std::string const&FNAME , std::vector<NaiveAnnoBox> & boxList , 
					   std::vector<PlaneData> allwall , 
					   AnnoRes & annoRes )
{
	//-------------------------------------------------------------------------
	// parsing action log
	// 
	// N=annoRes.action_log.size();
	// N x BoxNUM x ( 1 + 1 ), boxState,predRes


	//-------------------------------------------------------------------------
	// parsing the boxes in this scene
	// prepare:
	//   parentLID
	//   box_lid
	//   box_supType

	double * box_lid = new double[boxList.size()];
	double * parent_lid = new double[boxList.size()];
	double * parent_idx = new double[boxList.size()];
	double * box_supType = new double[boxList.size()];
	double * box_gtUID = new double[boxList.size()]; 
	double * box_sup_modified = new double[boxList.size()];
	double * box_geo_modified = new double[boxList.size()];

	double time_total=-1;
	double time_label=-1;
	double time_support=-1;
	double time_box=-1;

	for( int i=0 ; i < boxList.size() ; i++ ){
		box_lid[i]=boxList[i].labelLid;
		box_gtUID[i]=boxList[i].uid;
	}

	for( int i=0 ; i < boxList.size() ; i++ ){
		if( boxList[i].isSupByObj() ){
			// replace it by label id
			parent_lid[i]=box_lid[boxList[i].supLIDX];
			
			// Matlab indexing is start from 1
			parent_idx[i]=boxList[i].supLIDX;

		}else if( boxList[i].isSupByWall() ){
			parent_idx[i]=boxList[i].supByWallLidx;
			parent_lid[i]=-1;
		}else{
			parent_lid[i]=-1;
			parent_idx[i]=-1;
		}

		box_supType[i]=boxList[i].supType;

		box_sup_modified[i]=boxList[i].box_sup_modified;

		box_geo_modified[i]=boxList[i].box_geo_modified;
	}
	

	time_total = annoRes.total_time;
	time_label = annoRes.time_label;
	time_box	= annoRes.time_box;
	time_support = annoRes.time_support;
	 
	//-------------------------------------------------------------------------
	// collect box3d_fr info
	Eigen::MatrixXd allB3dFr;

	// each col = a box
	const int BOX_ESIZE=8*3;
	allB3dFr.resize( boxList.size() , BOX_ESIZE );

	for( int i=0 ; i< boxList.size() ;i++ ){
		allB3dFr.row(i) = Eigen::Map<Eigen::MatrixXf>(boxList[i].box3d_fr,1,BOX_ESIZE).cast<double>();
	}

	//-------------------------------------------------------------------------
	// collect box3d_world info
	Eigen::MatrixXd allB3dWorld;

	// each col = a box 
	allB3dWorld.resize( boxList.size() , BOX_ESIZE );

	for( int i=0 ; i< boxList.size() ;i++ ){
		allB3dWorld.row(i) = Eigen::Map<Eigen::MatrixXf>(boxList[i].box3d_world,1,BOX_ESIZE).cast<double>();
	}

	//-------------------------------------------------------------------------
	// collect AABB info
	Eigen::MatrixXd allAABB;

	// each col = a box 
	allAABB.resize( boxList.size() , BOX_ESIZE );

	for( int i=0 ; i< boxList.size() ;i++ ){
		allAABB.row(i) = Eigen::Map<Eigen::MatrixXf>(boxList[i].box3d_aabb,1,BOX_ESIZE).cast<double>();
	}
	//-------------------------------------------------------------------------
	// collect prediction menu 
	Eigen::MatrixXd allPMenu_NxPM;

	// each col = a box 
	allPMenu_NxPM.resize( boxList.size() , NaiveAnnoBox::MAX_MENU_NUM );

	for( int i=0 ; i< boxList.size() ;i++ ){
		allPMenu_NxPM.row(i) = Eigen::Map<Eigen::MatrixXi>(boxList[i].pred_menu,1,NaiveAnnoBox::MAX_MENU_NUM).cast<double>();
	}

	//-------------------------------------------------------------------------
	// collect all wall info
	Eigen::MatrixXd allWallModel;

	Eigen::MatrixXd allwall_idx;

	// each col = a box 
	allWallModel.resize( allwall.size() , 4 );
	allwall_idx.resize(boxList.size(),1);

	for( int i=0 ; i< allwall.size() ;i++ ){
		allWallModel.row(i) = allwall[i].model.cast<double>();
	}

	for( int i=0 ; i< boxList.size() ;i++ ){
		// matlab indexing is start from 1
		allwall_idx(i) = boxList[i].closetWall_lidx;
	} 

	//-------------------------------------------------------------------------
	// write out data
	std::fstream fout =std::fstream(FNAME,std::fstream::binary|std::fstream::out);

	// [ NOTE  field # ]
	// write # of field
	int num_of_field = 18;

	fout.write((char*)&num_of_field,sizeof(int));

	// 1 write content 
	writeMatrix(fout,"parentLID",parent_lid,1,boxList.size());

	// 2 write content ( prediction result )
	writeMatrix(fout,"box_lid",box_lid,1,boxList.size());

	// 3 write content 
	writeMatrix(fout,"box_supType",box_supType,1,boxList.size());

	// 4 write content - box3d_fr
	writeMatrix(fout,"all_b3d_fr",allB3dFr.data(), allB3dFr.rows(),allB3dFr.cols());

	// 5 write content - box3d_fr
	writeMatrix(fout,"all_b3d_world",allB3dWorld.data(), allB3dWorld.rows(),allB3dWorld.cols());

	// 6 write content - box3d_fr
	writeMatrix(fout,"all_b3d_aabb",allAABB.data(), allAABB.rows(),allAABB.cols());

	// 7 write content - imid
	double sce_id = annoRes.sce_id;
	writeMatrix(fout,"sce_id",&sce_id,1,1);

	// 8 write content - imid 
	writeMatrix(fout,"box_gtUID",box_gtUID,1,boxList.size());


	// 9 write content - allPMenu_NxPM 
	writeMatrix(fout,"allPMenu_NxPM",allPMenu_NxPM.data(),allPMenu_NxPM.rows(),allPMenu_NxPM.cols());

	// 10 write content - allPMenu_NxPM 
	writeMatrix(fout,"allWallModel",allWallModel.data() , allWallModel.rows() , allWallModel.cols());

	// 11  allwall_idx
	writeMatrix(fout,"allwall_idx",allwall_idx.data() , allwall_idx.rows() , allwall_idx.cols());

	// 12 parent_idx
	writeMatrix(fout,"sup_parent_idx", parent_idx ,1,boxList.size());
	 
	// 13 box_sup_modified
	writeMatrix(fout,"box_sup_modified", box_sup_modified ,1,boxList.size());

	// 14 box_geo_modified
	writeMatrix(fout,"box_geo_modified", box_geo_modified ,1,boxList.size());

	// 15 time_total
	writeMatrix(fout,"time_total", &time_total ,1,1);

	// 16 time_label
	writeMatrix(fout,"time_label", &time_label ,1,1);

	// 17 time_support
	writeMatrix(fout,"time_support", &time_support ,1,1);

	// 18 time_box
	writeMatrix(fout,"time_box", &time_box ,1,1);

	fout.close();

	std::cout<<"Save at ["<<FNAME<<"]\n";

}

void writeMatrix( std::fstream & fout,const char* field_name, double * mat , const int RSIZE , const int CSIZE )
{
	// write field name
	int fnameSize = strlen(field_name);
	fout.write( reinterpret_cast<const char*>(&fnameSize) , sizeof(int) );
	fout.write( field_name , sizeof(char)*fnameSize );

	// write mat data
	fout.write( reinterpret_cast<const char*>(&RSIZE) , sizeof(int) );
	fout.write( reinterpret_cast<const char*>(&CSIZE) , sizeof(int) );
	fout.write( reinterpret_cast<char*>(mat) , RSIZE*CSIZE*sizeof(double) );
}



