#include "stdafx.h"
#include "boxdata.h"



BoxData::BoxData(QObject * parent/* = NULL*/) : QObject(parent)
{
	//* for debug
	std::cout<<"set up test box\n";
	m_boxCx=200;
	m_boxCy=200;

	m_predTags.push_back(QString::fromLocal8Bit("bed"));
	m_predTags.push_back(QString::fromLocal8Bit("sofa"));
	m_predTags.push_back(QString::fromLocal8Bit("table"));

	m_predTagLIDs.push_back(157);
	m_predTagLIDs.push_back(36);
	m_predTagLIDs.push_back(3);

	m_predCost.push_back(1);
	m_predCost.push_back(12);
	m_predCost.push_back(1.325);

	m_iniLID=157;
	GCOST_TYPE gcost;
	gcost.push_back(100);
	gcost.push_back(233);
	gcost.push_back(14400);
	m_predCost=gcost;

	m_confirmed=false;
	m_touched=false;
	m_edited=false;

}

//real constructor
BoxData::BoxData( const int CX, const int CY,
				 const std::vector<int> & LIS_LIST, 
				 GCOST_TYPE const& COST,
				 QStringList  const & STRLIST, QObject * parent /*= NULL*/ )
{
	m_boxCx=CX;
	m_boxCy=CY;

	m_predTags = STRLIST;

	for( int i=0; i<LIS_LIST.size() ; i++){
		m_predTagLIDs.push_back( LIS_LIST[i] );
	}

	m_predCost=COST;

	m_iniLID=m_predTagLIDs[0];

	m_confirmed=false;
	m_touched=false;
	m_edited=false;
}

BoxData::BoxData( const BoxData& RVAL )
{
	//
	// ALSO UPDATE manually copy in INNER_PRED
	//
	m_boxCx=RVAL.getBoxCx();
	m_boxCy=RVAL.getBoxCy();
	
	m_predTagLIDs=RVAL.getPredTagLIDs();
	m_predTags=RVAL.getPredTags();
	
	m_predCost=RVAL.getPredCost();
	m_iniLID = RVAL.getIniLID();

	m_touched=RVAL.getTouched();
	m_confirmed=RVAL.getConfirmed();

	m_edited = RVAL.getEdited();
}

BoxData & BoxData::operator=( const BoxData & RVAL)
{
	//
	// ALSO UPDATE manually copt in INNER_PRED
	//
	this->m_boxCx=RVAL.getBoxCx();
	this->m_boxCy=RVAL.getBoxCy();

	this->m_predTags=RVAL.getPredTags();
	this->m_predTagLIDs=RVAL.getPredTagLIDs();

	this->m_predCost=RVAL.getPredCost();

	this->m_iniLID = RVAL.getIniLID();

	this->m_touched = RVAL.getTouched();
	this->m_confirmed = RVAL.getConfirmed();

	this->m_edited = RVAL.getEdited();

	return *this;
}

BoxData::~BoxData()
{

}

void BoxData::setPredTags( const QStringList & newList )
{
	m_predTags=newList;
	DEBUG_PR("onPredTagChanged");

	emit onPredTagChanged();

}

void BoxData::setPredTagLIDs( const QList<int> & newList )
{
	m_predTagLIDs=newList;

	DEBUG_PR("onPredTagLIDChanged");  

	emit onPredTagLIDChanged();

}

void BoxData::setBoxCx( int cx )
{
	m_boxCx=cx;
	emit onBoxCxChanged();
}

void BoxData::setBoxCy( int cy )
{
	m_boxCy=cy;
	emit onBoxCyChanged();
}

QStringList BoxData::getPredTags() const
{
	return m_predTags;
}

QList<int> BoxData::getPredTagLIDs() const
{
	return m_predTagLIDs;
}

int BoxData::getBoxCx() const
{
	return m_boxCx;
}

int BoxData::getBoxCy() const
{
	return m_boxCy;
}

void BoxData::getMainTag( int & lid, std::string & name  ) const
{
	lid = m_predTagLIDs[0];
	name = m_predTags[0].toStdString();
}

//
// **major interface for qml UI
void BoxData::swapMajorTag( const int SRC_IDX , const int DST_IDX )
{
	if( SRC_IDX== DST_IDX){
		return;
	}

	int temp=m_predTagLIDs[SRC_IDX];
	
	m_predTagLIDs[SRC_IDX] =m_predTagLIDs[DST_IDX];
	m_predTagLIDs[DST_IDX] =temp;

	QString tempStr = m_predTags[SRC_IDX];

	m_predTags[SRC_IDX] =m_predTags[DST_IDX];
	m_predTags[DST_IDX] =tempStr;
	 
	m_touched=true; 
	//emit onPredTagLIDChanged();
	//emit onPredTagChanged();
}

void BoxData::setPredCost( GCOST_TYPE const COST  )
{
	m_predCost=COST;
}

GCOST_TYPE BoxData::getPredCost() const
{
	return m_predCost;
}

int BoxData::getNumOfTagStr()
{
	return m_predTags.size();
}

QString BoxData::getTagStr( const int IDX )
{
	return m_predTags[IDX];
}

void BoxData::setEditedTagStr( QString const & NEWLABEL )
{
	//push to head 
	m_predTags.push_front(NEWLABEL);
	m_predTagLIDs.push_front(EDITED_LABEL_LID);
	
	m_touched=true;
	m_confirmed=true;

	m_edited=true;
}
