#ifndef MY_BOXDATA
#define MY_BOXDATA


#include <QObject>
#include <QStringList>
#include <vector>

typedef  QList<float> GCOST_TYPE;

class BoxData : public QObject
{ 
	Q_OBJECT


	Q_PROPERTY( int boxCx READ getBoxCx WRITE setBoxCx NOTIFY onBoxCxChanged )
	Q_PROPERTY( int boxCy READ getBoxCy WRITE setBoxCy NOTIFY onBoxCyChanged )
	
	//Q _PROPERTY( QStringList predTags READ getPredTags   ) 
	//Q _PROPERTY( QStringList predTags READ getPredTags WRITE setPredTags  NOTIFY onPredTagChanged ) 
	//Q _PROPERTY( QList<int> predTagIDs READ getPredTagLIDs WRITE setPredTagLIDs  NOTIFY onPredTagLIDChanged  ) 
	Q_PROPERTY( GCOST_TYPE predCost READ getPredCost WRITE setPredCost ) 
	
	Q_PROPERTY( int isConfirmed READ getConfirmed WRITE setConfirmed  ) 
	Q_PROPERTY( int isTouched READ getTouched WRITE setTouched  ) 
	
	//Q _PROPERTY( int isTouched READ getConfirmList WRITE setConfirmList  ) 
	//Q _ PROPERTY( QVariantList stateStr READ getCurStateStr NOTIFY stateStrChange) 

	
public:

	static const int EDITED_LABEL_LID = -77;
	static const int NEW_LABEL_LID = -1;
	static const int FLOATING_LID = -2;

	//-------------------------------------------------------------------------

	BoxData(QObject * parent = NULL);

	BoxData(const int, const int , const std::vector<int> &,  GCOST_TYPE const&, const QStringList &,QObject * parent = NULL);
	
	BoxData(const BoxData& rval);
	
	BoxData &BoxData::operator =(const BoxData &);
	
	~BoxData();

	//-------------------------------------------------------------------------
	//setter
	void setPredTags( const QStringList & );
	void setPredTagLIDs( const QList<int> & );
	void setPredCost( GCOST_TYPE const);

	void setBoxCx(int cx);
	void setBoxCy(int cy);

	inline
	void setConfirmed(const int & clist ){
		m_confirmed=clist;
	}
	
	inline
	void setTouched( const int & clist ){
		m_touched=clist;
	}

	inline
	void setEdited(const int & VAL){
		m_edited=VAL;
	}
	//-------------------------------------------------------------------------
	//getter
	
	QStringList getPredTags()const;
	
	QList<int> getPredTagLIDs()const;
	
	GCOST_TYPE getPredCost()const;
	
	inline
	int getConfirmed()const{
		return m_confirmed;
	}
	
	inline 
	int getTouched()const{
		return m_touched;
	}

	int getBoxCx()const;
	int getBoxCy()const;

	inline 
	int getEdited()const{
		return m_edited;
	}

	inline
	int getIniLID()const{ return m_iniLID;}

	inline 
	int getCurLID()const{ return m_predTagLIDs[0];}

	void getMainTag(int & labelId , std::string & lname )const;
	//-------------------------------------------------------------------------

    signals:

	void onBoxCxChanged();
	void onBoxCyChanged();

	void onPredTagChanged();
	void onPredTagLIDChanged();

public slots:
	// **major interface for qml UI
	void swapMajorTag(const int SRC_IDX , const int DST_IDX);

	int getNumOfTagStr();
	QString getTagStr(const int IDX);

	void setEditedTagStr(QString const & NEWLABEL);

private:

	QStringList m_predTags;   // for qml
	QList<int> m_predTagLIDs; // 1d access
	GCOST_TYPE m_predCost;    // global prediction cost

	int m_confirmed;  // for UI lock
	int m_touched;    // for UI touch  
	int m_edited; // for log UI editing     

	int m_boxCx;  
	int m_boxCy;  
	
	// the first lid
	int m_iniLID; // for detect change
};

#endif



























