#include "stdafx.h"
#include "boxdatav2.h"

BoxDataV2::BoxDataV2(  QObject *parent ):QObject(parent)
{

	memset(box3d_fr,0,sizeof(Vertex)*BOXVEX_NUM_3D);
	memset(box3d_world,0,sizeof(Vertex)*BOXVEX_NUM_3D);
	memset(box3d_aabb,0,sizeof(Vertex)*BOXVEX_NUM_3D);

	memset(obb_fr2aabb,0,sizeof(Vertex)*9);
	memset(obb_aabb2fr,0,sizeof(Vertex)*9);
	
	m_confed=0;
	m_switched=0;
	m_edited=0; 
	m_confALL=0;

	isLocalRefined=0;

	int uid=0;
	int supporterLIdx=-1;
	int supTye=-1;

	int supWallLIdx=-1;
	int closedWallLIdx=-1;
	

	ui_x0=0;
	ui_y0=0;
	ui_width=0;
	ui_height=0;
	
	sup_modified = 0;
	box_modified = 0;

}

BoxDataV2::BoxDataV2( const BoxDataV2& RVAL )
{
	this->setBox3d_aabb(RVAL.getBox3d_AABB());
	this->setBox3d_fr(RVAL.getBox3d_fr());
	this->setBox3d_world(RVAL.getBox3d_world());

	this->setObb_aabb2Fr(RVAL.getAABB2FrOBB());
	this->setObb_fr2aabb(RVAL.getFrOBB2AABB());

	this->setUID(RVAL.getUID());
	this->setSupporterLIDX(RVAL.getSupporterLIDX());
	this->setSupType(RVAL.getSupType());

	this->setSupWallLIdx(RVAL.getSupWallLIdx());
	this->setClosedWallLIdx(RVAL.getClosedWallLIdx());

	this->isLocalRefined=RVAL.getIsLocalRefined();

	// UI - 2D 
	this->ui_x0=RVAL.getUI_X0();
	this->ui_x0=RVAL.getUI_Y0();
	this->ui_width=RVAL.getUI_W();
	this->ui_height=RVAL.getUI_H();


	// result log
	this->m_svmAnsLid=(RVAL.getPredANSLid()); 
	this->svm_pred_res=RVAL.getSVMINIMenu();
	this->userSelect_rank=RVAL.getUserSelectRank();


	this->setUserConfed(RVAL.getUserConfed());
	this->setSwitched(RVAL.getSwtiched());
	this->setEdited(RVAL.getEdited());
	this->setConfALL(RVAL.getConfALL()); 


	this->setBoxCx(RVAL.getBoxCx());
	this->setBoxCy(RVAL.getBoxCy());

	// --------------------------------------------
	this->box_planes=RVAL.getBoxPlane_fr();
	this->box_planes_world = RVAL.getBoxPlane_world();

	this->set3dPcdW(RVAL.getPcdW());

	this->seg_2d_mask = RVAL.getSegMask().clone();
	this->box_2d_mask = RVAL.get_box2dMask().clone();

	// --------------------------------------------
	this->sup_modified = RVAL.getSupModeified();

	this->box_modified = RVAL.getBoxModified();

}

BoxDataV2 & BoxDataV2::operator=( const BoxDataV2 & RVAL )
{
	this->setBox3d_aabb(RVAL.getBox3d_AABB());
	this->setBox3d_fr(RVAL.getBox3d_fr());
	this->setBox3d_world(RVAL.getBox3d_world());

	this->setObb_aabb2Fr(RVAL.getAABB2FrOBB());
	this->setObb_fr2aabb(RVAL.getFrOBB2AABB());

	this->setUID(RVAL.getUID());
	this->setSupporterLIDX(RVAL.getSupporterLIDX());
	this->setSupType(RVAL.getSupType());

	this->setSupWallLIdx(RVAL.getSupWallLIdx());
	this->setClosedWallLIdx(RVAL.getClosedWallLIdx());

	this->isLocalRefined=RVAL.getIsLocalRefined();

	// UI - 2D 
	this->ui_x0=RVAL.getUI_X0();
	this->ui_x0=RVAL.getUI_Y0();
	this->ui_width=RVAL.getUI_W();
	this->ui_height=RVAL.getUI_H();

	// result log
	this->m_svmAnsLid=(RVAL.getPredANSLid()); 
	this->svm_pred_res=RVAL.getSVMINIMenu();
	this->userSelect_rank=RVAL.getUserSelectRank();

	this->setUserConfed(RVAL.getUserConfed());
	this->setSwitched(RVAL.getSwtiched());
	this->setEdited(RVAL.getEdited());
	this->setConfALL(RVAL.getConfALL()); 

	this->setBoxCx(RVAL.getBoxCx());
	this->setBoxCy(RVAL.getBoxCy());

	// --------------------------------------------
	this->box_planes=RVAL.getBoxPlane_fr();
	this->box_planes_world = RVAL.getBoxPlane_world();

	this->set3dPcdW(RVAL.getPcdW());

	this->seg_2d_mask = RVAL.getSegMask().clone();
	this->box_2d_mask = RVAL.get_box2dMask().clone();

	// --------------------------------------------

	this->sup_modified = RVAL.getSupModeified();

	this->box_modified = RVAL.getBoxModified();
	 
	return *this;
}
 
BoxDataV2::~BoxDataV2()
{

}
 
void BoxDataV2::setBox2d_img( Eigen::MatrixXf const & BOX_V )
{
	 
	// Nx3
	myassert(BOX_V.rows()==8 ,"invalid box 3d");

	for( int ir=0 ; ir< BOX_V.rows() ; ir++){
		box2d_vex[ir*2]=BOX_V(ir,0);
		box2d_vex[ir*2+1]=BOX_V(ir,1); 
	} 

}

void BoxDataV2::setBox3d_world( Vertex const* vin )
{
	for( int i=0 ; i<BOXVEX_NUM_3D ; i++ ){

		box3d_world[i]=vin[i];
	}


}

void BoxDataV2::setBox3d_fr( Vertex const* vin )
{
	for( int i=0 ; i<BOXVEX_NUM_3D ; i++ ){

		box3d_fr[i]=vin[i];
	}
}

void BoxDataV2::setBox3d_aabb( Vertex const* vin)
{
	for( int i=0 ; i<BOXVEX_NUM_3D ; i++ ){

		box3d_aabb[i]=vin[i];
	}
}

void BoxDataV2::setObb_fr2aabb( Eigen::Matrix3f const & mat )
{
	for( int ir=0 ; ir<mat.rows() ; ir++ ){
		for( int ic=0 ; ic<mat.cols() ; ic++ ){
		   obb_fr2aabb[ir+ic*3]=mat(ir,ic);
		}
	}
}

void BoxDataV2::setObb_aabb2Fr( Eigen::Matrix3f const & mat )
{
	for( int ir=0 ; ir<mat.rows() ; ir++ ){
		for( int ic=0 ; ic<mat.cols() ; ic++ ){
			obb_aabb2fr[ir+ic*3]=mat(ir,ic);
		}
	}
}

void BoxDataV2::setObb_aabb2Fr( Vertex const* vin)
{
	for( int i=0 ; i<9 ; i++ ){

		obb_aabb2fr[i]=vin[i];
	}
}

void BoxDataV2::setObb_fr2aabb( Vertex const* vin)
{
	for( int i=0 ; i<9 ; i++ ){

		obb_fr2aabb[i]=vin[i];
	}
}

void BoxDataV2::setPredTags( const QStringList & newList )
{
	m_predTags=newList;
}

void BoxDataV2::setPredTagLIDs( const QList<int> & newList )
{
	m_predTagLIDs=newList;
	m_svmAnsLid=newList[0];
	userSelect_rank=-1;
	svm_pred_res=newList;
}

#pragma region GET
Eigen::MatrixXf BoxDataV2::getBox3d_fr_n3() const
{
	Eigen::MatrixXf ff;

	ff.resize(BOXVEX_NUM,3);

	for (int iv=0 ; iv < BOXVEX_NUM ; iv++)
	{
		ff(iv,0)=box3d_fr[iv*3+0];
		ff(iv,1)=box3d_fr[iv*3+1];
		ff(iv,2)=box3d_fr[iv*3+2];
	}
	return ff;
}

Eigen::MatrixXf BoxDataV2::getBox3d_world_n3() const
{
	Eigen::MatrixXf ff;

	ff.resize(BOXVEX_NUM,3);

	for (int iv=0 ; iv < BOXVEX_NUM ; iv++)
	{
		ff(iv,0)=box3d_world[iv*3+0];
		ff(iv,1)=box3d_world[iv*3+1];
		ff(iv,2)=box3d_world[iv*3+2];
	}


	return ff;
}

Eigen::MatrixXf BoxDataV2::getBox2d_img_n2() const
{
	Eigen::MatrixXf ff;

	ff.resize(BOXVEX_NUM, 2 );

	for (int iv=0 ; iv < BOXVEX_NUM ; iv++)
	{
		ff(iv,0)=box2d_vex[iv*2+0];
		ff(iv,1)=box2d_vex[iv*2+1];
	}


	return ff;
}


void BoxDataV2::estimateFeatures()
{
	BoxFeatures fv;

	// height
	Eigen::MatrixXf box_fr = getBox3d_fr_n3();

	fv.height=box_fr.col(2-1).maxCoeff() - box_fr.col(2-1).minCoeff();

	// xz area 
	Eigen::Vector3f e1= box_fr.row(0)-box_fr.row(1);
	Eigen::Vector3f e2= box_fr.row(1)-box_fr.row(2);

	// project to xz plane
	e1.y()=0;
	e2.y()=0; 
	fv.xz_area =e1.norm()*e2.norm();

	// save
	fvec_fr = fv;
}

QStringList const& BoxDataV2::getPredTags() const
{
	return m_predTags;
}

QList<int> const& BoxDataV2::getPredTagLIDs() const
{
	return m_predTagLIDs;
}

QStringList & BoxDataV2::getPredTags_link() 
{
	return m_predTags;
}

QList<int> & BoxDataV2::getPredTagLIDs_link() 
{
	return m_predTagLIDs;
}


QString BoxDataV2::getTagStr( const int IDX )
{
	return m_predTags[IDX];
}

int BoxDataV2::getNumOfTagStr()
{
	return m_predTags.size();
}

#pragma endregion GET

void BoxDataV2::swapMajorTag( const int SRC_IDX , const int DST_IDX )
{
	if( SRC_IDX== DST_IDX){
		// this is a confirm action
		confirmThisBox();

		return;
	}

	int temp=m_predTagLIDs[SRC_IDX];

	m_predTagLIDs[SRC_IDX] =m_predTagLIDs[DST_IDX];
	m_predTagLIDs[DST_IDX] =temp;

	QString tempStr =m_predTags[SRC_IDX];

	m_predTags[SRC_IDX] =m_predTags[DST_IDX];
	m_predTags[DST_IDX] =tempStr;

	// check LID is change or not
	if( m_svmAnsLid != m_predTagLIDs[DST_IDX]){ 

		m_switched=true;  
	}else{
		m_switched=false;
	}

	// update user select order
	userSelect_rank=-1;
	for( int i=0 ; i< svm_pred_res.size() ;i++){
		if( svm_pred_res[i]==m_predTagLIDs[DST_IDX] ){ 
			userSelect_rank=i;
			break;
		}
	}
	 
	if( userSelect_rank==-1 ){
		throw "**cannot found the location in initial lids**";
	}
	
	emit onUserModifiedChange();
}

void BoxDataV2::confirmThisBox()
{
	m_confed=true; 
}

void BoxDataV2::unconfThisBox()
{
	m_confed=false; 

}

void BoxDataV2::setEditedTagStr( QString const & NEWLABEL )
{
	//push to head 
	m_predTags.push_front(NEWLABEL);
	m_predTagLIDs.push_front(-7);

	m_edited=true;
	 
	emit onUserModifiedChange();
}

void BoxDataV2::updateBoxWCen2d(Eigen::MatrixXf const & MVP , int winHeight ,int width , int height , int tx , int ty )
{
	Eigen::MatrixXf box3dw = getBox3d_world_n3();

	Eigen::Vector4f cen = (box3dw.colwise().mean()).homogeneous();

	cen = MVP*cen;
	Eigen::Vector3f cenNDC = cen.hnormalized();

	// use top-left center
	int dy=winHeight-height;
	int fx=static_cast<int>( (cenNDC.x()+1)*width/2+tx );
	int fy=static_cast<int>( dy + height- ((cenNDC.y()+1)*height/2+ty) );

	m_boxCx2d=fx;
	m_boxCy2d=fy;

	Eigen::MatrixXf box3d_wh;
	box3d_wh.resize(8,4);
	box3d_wh.col(4-1).setOnes();
	box3d_wh.block(0,0,8,3)= box3dw;
	box3d_wh.transposeInPlace();

	// 4xN
	box3d_wh = MVP*box3d_wh;

	// 4xN
	Eigen::MatrixXf box2d_h =box3d_wh.colwise().hnormalized();
	// Nx4
	Eigen::MatrixXf box2d = box2d_h.transpose();

	float min_nx = box2d.col(0).minCoeff();
	float max_nx = box2d.col(0).maxCoeff();

	float min_ny = box2d.col(1).minCoeff();
	float max_ny = box2d.col(1).maxCoeff();


	int fx_min=static_cast<int>( (min_nx+1)*width/2+tx );
	int fx_max=static_cast<int>( (max_nx+1)*width/2+tx );
	int fy_min=static_cast<int>( dy + height- ((min_ny+1)*height/2+ty) );
	int fy_max=static_cast<int>( dy + height- ((max_ny+1)*height/2+ty) );

	// due to invert y axis use fy_max as cy
	setUIRect(fx_min,fy_max,abs(fx_max-fx_min) ,abs(fy_max-fy_min));
}

void BoxDataV2::setUIRect( int x,int y,int width,int height )
{
	ui_x0=x;
	ui_y0=y;
	ui_width=width;
	ui_height=height;
}

void BoxDataV2::setBoxPlane_fr( std::vector<PlaneData> const & PLS )
{
	box_planes=PLS;
}
void BoxDataV2::setBoxPlane_world( std::vector<PlaneData> const & PLS )
{
	box_planes_world=PLS;
}

std::vector<PlaneData> & BoxDataV2::getBoxPlane_fr()
{
	return box_planes;
}

std::vector<PlaneData> const & BoxDataV2::getBoxPlane_fr() const
{
	return box_planes;
}

std::vector<PlaneData> & BoxDataV2::getBoxPlane_world()
{
	return box_planes_world;
}

std::vector<PlaneData> const& BoxDataV2::getBoxPlane_world()const
{
	return box_planes_world;
}

void BoxDataV2::setSegMask( cv::Mat & smask )
{
	seg_2d_mask=smask.clone();
}

cv::Mat const& BoxDataV2::getSegMask() const
{
	return seg_2d_mask;
}

void BoxDataV2::set_box2dMask( cv::Mat & smask )
{
	box_2d_mask=smask.clone();
}

cv::Mat const& BoxDataV2::get_box2dMask() const
{
	return box_2d_mask;
}

bool boxQTerm_desComp( BoxQualityTerm const & L, BoxQualityTerm const & R )
{
	return L.prob > R.prob;
}

