#ifndef BOXDATAV2_H
#define BOXDATAV2_H

#include <QObject>
#include <vector>
#include <Eigen/Eigen>
#include <Eigen/Geometry>
#include <Eigen/StdVector>

#include <QList>
#include <QStringList>

#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>

#include "common_tool.h" 

class BoxFeatures
{
public:
	
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
		  
	double xz_area;
	double height;
};

struct BoxQualityTerm
{
public:
	int boxIdx;
	double prob;
};


bool boxQTerm_desComp(  BoxQualityTerm const &,  BoxQualityTerm const &);


class BoxQuality{

public:
	// large [v]
	double boxSeg2d_overlapArea;
	// large [v]
	double boxSeg2d_overlapRatio;
	// large [v]
	double box2d_inScreenArea;
	// large [v]
	double box2d_inScreen_ratio; 
	// large [v]
	double segImg_Ratio;

	// prob. belief (large)
	double menu_belif;
	
	int is_box_3doccluded;
	int num_voxel_pts;

	// large
	double num_ctxEdges;
	double richCtxRatio;

	double meanPlane2VXPcdErr;

	double vol;
};



class BoxDataV2 : public QObject
{
	Q_OBJECT

	Q_PROPERTY( int boxCx READ getBoxCx NOTIFY onBoxCxChanged )
	Q_PROPERTY( int boxCy READ getBoxCy NOTIFY onBoxCyChanged )
	Q_PROPERTY( int isLocked READ getUserConfed   ) 
	Q_PROPERTY( int isTouched READ getSwtiched  ) 

	Q_PROPERTY( int userModified READ getUserModified  NOTIFY onUserModifiedChange ) 

	Q_PROPERTY( int ui_x READ getUI_X0  )
	Q_PROPERTY( int ui_y READ getUI_Y0  )
	Q_PROPERTY( int ui_w READ getUI_W  )
	Q_PROPERTY( int ui_h READ getUI_H  )

private:

	enum SUPPORT_TYPE{
		INI=0,
		BOX=1,
		WALL=2,
		FLOOR=4,
		FLOAT=8
	};

public:  
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW

	//static const int EDITED_LABEL_LID = -77;

	typedef float Vertex;
	typedef const float ConstVertex;
	typedef Eigen::Map<Eigen::MatrixXf> EMap3d;
	typedef Eigen::Map<Eigen::MatrixXf> EMap2d;

	static const int BOXVEX_NUM=8;
	static const int BOXVEX_NUM_3D=8*3;
	static const int BOXVEX_NUM_2DIMG=8*2;
	//-------------------------------------------------------------------------

	BoxDataV2(QObject *parent=0);

	BoxDataV2(const BoxDataV2& rval);

	BoxDataV2 & BoxDataV2::operator =(const BoxDataV2 &);

	~BoxDataV2();

	// Use column major order
	void setBox3d_world(Vertex const*); 

	void setBox3d_fr(Vertex const*); 

	void setBox3d_aabb(Vertex const*); 

	void setObb_aabb2Fr(Vertex const*); 
	
	void setObb_fr2aabb(Vertex const*); 
	 
	void setBox2d_img( Eigen::MatrixXf const & BOX_V );

	Eigen::MatrixXf getBox3d_fr_n3() const; 
	Eigen::MatrixXf getBox3d_world_n3() const;
	Eigen::MatrixXf getBox2d_img_n2() const;

	//-------------------------------------------------------------------------
	void setObb_fr2aabb(Eigen::Matrix3f const & mat); 
	void setObb_aabb2Fr(Eigen::Matrix3f const & mat); 
	
	//-------------------------------------------------------------------------
	void setPredTags( const QStringList & );
	void setPredTagLIDs( const QList<int> & );
	
	//-------------------------------------------------------------------------
#pragma region INL_FUNC_SET

	inline
		bool isSupINI()const{
			return supTye == SUPPORT_TYPE::INI;
	}
	inline
		bool isSupByGround()const{
			return supTye & SUPPORT_TYPE::FLOOR;
	}
	inline
		bool isSupByWall()const{
			return supTye & SUPPORT_TYPE::WALL;
	}
	inline
		bool isSupByObj()const{
			return supTye & SUPPORT_TYPE::BOX;
	}
	inline
		bool isFloat()const{
			return supTye & SUPPORT_TYPE::FLOAT;
	}
	inline
		void setSupToIni(){
			supTye=SUPPORT_TYPE::INI;
	}
	inline
		void setSupToWall(){
			supTye=supTye|SUPPORT_TYPE::WALL;
	}
	inline
		void setSupToFloor(){
			supTye=supTye|SUPPORT_TYPE::FLOOR;
	}
	inline
		void setSupToObj(){
			supTye=supTye|SUPPORT_TYPE::BOX;
	}
	inline
		void setSupToFloat(){
			supTye=supTye|SUPPORT_TYPE::FLOAT;
	}
	
	inline
		void setSupModified(bool val){
			sup_modified = val;
	}
	//-------------------------------------------------------------------------

	
	inline
		void set3dPcdW(pcl::PointCloud<pcl::PointXYZ>::Ptr & WIDX){

			pcd_world_ptr = WIDX;  
		}
	
	inline
	void setBox3d_world(Eigen::MatrixXf const & BOX_V){
		// Nx3
		myassert(BOX_V.rows()==8,"invalid box 3d");

		for( int ir=0 ; ir< BOX_V.rows() ; ir++){
			box3d_world[ir*3]=BOX_V(ir,0);
			box3d_world[ir*3+1]=BOX_V(ir,1);
			box3d_world[ir*3+2]=BOX_V(ir,2);
		}
	} 

	inline
	void setBox3d_fr(Eigen::MatrixXf const &BOX_V){
		// Nx3
		myassert(BOX_V.rows()==8,"invalid box 3d");

		for( int ir=0 ; ir< BOX_V.rows() ; ir++){
			box3d_fr[ir*3]=BOX_V(ir,0);
			box3d_fr[ir*3+1]=BOX_V(ir,1);
			box3d_fr[ir*3+2]=BOX_V(ir,2);
		}
	}

	inline
	void setBox3d_aabb(Eigen::MatrixXf const &BOX_V){

			myassert(BOX_V.rows()==8,"invalid box 3d");
			for( int ir=0 ; ir< BOX_V.rows() ; ir++){
				box3d_aabb[ir*3]=BOX_V(ir,0);
				box3d_aabb[ir*3+1]=BOX_V(ir,1);
				box3d_aabb[ir*3+2]=BOX_V(ir,2);
			}
	}
	
	inline
		void setUserConfed(const int & clist ){
			m_confed=clist;
			emit onUserModifiedChange();
	}

	inline
		void setSwitched( const int & clist ){
			m_switched=clist;
			emit onUserModifiedChange();
	}

	inline
		void setEdited(const int & VAL){
			m_edited=VAL;
			emit onUserModifiedChange();
	}

	inline 
		void setBoxCx(int bc){
			m_boxCx2d=bc;
	}
	inline 
		void setBoxCy(int bc){
			m_boxCy2d=bc;
	}
	 
	inline
		void setIsLocalRefined(bool s){
			isLocalRefined=s;
	}

	void setBoxPlane_fr(std::vector<PlaneData> const & PLS);

	inline
	void setSupWallLIdx(int wid){
		supWallLIdx=wid;
	}

	inline
	void setClosedWallLIdx(int wid){
		closedWallLIdx=wid;
	}

	inline
	void setBoxModified(int val){
		box_modified = val;
	}

#pragma endregion INL_FUNC_SET
	
	//-------------------------------------------------------------------------

#pragma region INL_FUNC_GET

	inline
		int getSupWallLIdx( )const{
			return supWallLIdx ;
	}

	inline
		int getClosedWallLIdx( )const{
			return closedWallLIdx ;
	}
	
		std::vector<PlaneData> & getBoxPlane_fr();
	 
		std::vector<PlaneData> const & getBoxPlane_fr()const;

	//*
	inline
		pcl::PointCloud<pcl::PointXYZ>::ConstPtr getPcdWConst()const{

			return pcd_world_ptr; 
	}
	
	inline
		pcl::PointCloud<pcl::PointXYZ>::Ptr getPcdW()const{

			return pcd_world_ptr; 
	}
	//*/
	inline 
		bool getUserModified()const{
			return m_confed || m_switched || m_edited || m_confALL ;
	}

	inline
		Vertex const* getBox3d_fr()const{
			return const_cast<Vertex const*>(box3d_fr);
	}
	
	inline
		Vertex const* getBox3d_world()const{
			return const_cast<Vertex const*>(box3d_world);
	}

	inline
		Vertex const* getBox3d_AABB()const{
			return const_cast<Vertex const*>(box3d_aabb);
	}
	inline
		Vertex const* getFrOBB2AABB()const{
			return const_cast<Vertex const*>(obb_fr2aabb);
	}
	inline
		Vertex const* getAABB2FrOBB()const{
			return const_cast<Vertex const*>(obb_aabb2fr);
	}

	inline
		Vertex * getBox3d_fr(){
			return const_cast<Vertex *>(box3d_fr);
	}

	inline
		Vertex * getBox3d_world(){
			return box3d_world;
	}
	inline
	 	Vertex * getBox3d_AABB(){
			return box3d_aabb;
	}
	inline
		Vertex * getFrOBB2AABB(){
			return obb_fr2aabb;
	}
	inline
		Vertex * getAABB2FrOBB(){
			return obb_aabb2fr;
	}

	inline
		void setUID(int const  ID){
			uid=ID;
	}
	inline
		int getUID()const{
			return uid;
	}
	inline
		void setSupporterLIDX(int sup){
			supporterLIdx=sup;
	}
	inline
		int getSupporterLIDX()const{
			return supporterLIdx;
	}
	inline
		void setSupType(int s){
			//supTye=(SUPPORT_TYPE)s;
			//supTye = supTye | s;
			supTye = s;
	}
	
	inline
		int getSupType()const{
			return supTye;
	}


	inline
		int getUserConfed()const{
			return m_confed;
	}

	inline 
		int getSwtiched()const{
			return m_switched;
	} 
	inline 
		int getEdited()const{
			return m_edited;
	}

	inline 
		int getBoxCx()const{
			return m_boxCx2d;
	}
	inline 
		int getBoxCy()const{
			return m_boxCy2d;
	}

	inline 
		int getPredANSLid()const{
			return m_svmAnsLid;
	} 
	inline
		bool getIsLocalRefined()const{
			return isLocalRefined;
	}
	inline
		int getUserSelectRank()const{
			return userSelect_rank;
	}
	inline
		QList<int> getSVMINIMenu()const{
			return svm_pred_res;
	}
	inline 
		int getUI_X0()const{
			return ui_x0;
	}	
	inline 
		int getUI_Y0()const{
			return ui_y0;
	}	
	inline 
		int getUI_W()const{
			return ui_width;
	}	
	inline 
		int getUI_H()const{
			return ui_height;
	}

	inline
		int getSupModeified()const{
			return sup_modified;
	}

	inline
		int getBoxModified()const{
			return box_modified;
	}

#pragma endregion INL_FUNC_GET
	
	//-------------------------------------------------------------------------
	// setter & getter v2

	void setSegMask(cv::Mat & smask);

	cv::Mat const& getSegMask()const; 

	void set_box2dMask(cv::Mat & smask);

	cv::Mat const& get_box2dMask()const; 

	inline
	void setConfALL(bool val){
			m_confALL=val;
			emit onUserModifiedChange();
	}

	inline
	int getConfALL()const{
			return m_confALL;
	}

	void setBoxPlane_world( std::vector<PlaneData> const & PLS );

	std::vector<PlaneData> & getBoxPlane_world(); 
	
	std::vector<PlaneData> const& getBoxPlane_world()const;

	//-------------------------------------------------------------------------
	
	QStringList const & getPredTags()const;

	QList<int> const & getPredTagLIDs()const;
	
	QStringList & getPredTags_link();

	QList<int> & getPredTagLIDs_link();

	//-------------------------------------------------------------------------
	
	void estimateFeatures();
	
	void updateBoxWCen2d(Eigen::MatrixXf const & MVP , int winHeight ,int width , int height , int tx , int ty );

	//-------------------------------------------------------------------------

public: 
signals: 

	void onBoxCxChanged();
	void onBoxCyChanged();
	void onUserModifiedChange();

public slots: 
	//-------------------------------------------------------------------------
	// [USER action handler]
	void swapMajorTag(const int SRC_IDX , const int DST_IDX);

	void setEditedTagStr(QString const & NEWLABEL);

	void confirmThisBox();

	void unconfThisBox();

	//------------------------------------------------------------------------
	// For QML UI

	int getNumOfTagStr();

	QString getTagStr(const int IDX);

private:
	void setUIRect( int x,int y,int width,int height );


public:
	//=====================================
	// no copy members 
	// these members should be recalculated before they accessed.
	BoxFeatures fvec_fr;

	BoxQuality quality;
	//=====================================

private:
	//=====================================
	// these members would not be saved in naive type

	std::vector<PlaneData> box_planes;
	std::vector<PlaneData> box_planes_world;

	cv::Mat seg_2d_mask;

	cv::Mat box_2d_mask;
	
	pcl::PointCloud<pcl::PointXYZ>::Ptr pcd_world_ptr; 
	
	//===================================== 
	// no copy members

	// 2d Box in image space
	Vertex box2d_vex[BOXVEX_NUM_2DIMG]; 

	//=====================================
	// box geometry

	Vertex box3d_fr[BOXVEX_NUM_3D];  
	Vertex box3d_world[BOXVEX_NUM_3D]; 
	Vertex box3d_aabb[BOXVEX_NUM_3D]; 
	 
	Vertex obb_fr2aabb[3*3]; 
	Vertex obb_aabb2fr[3*3]; 
	
	//-------------------------------------------
	int uid;
	int supporterLIdx;
	int supTye;

	int supWallLIdx;
	int closedWallLIdx;

	bool isLocalRefined; 

	// UI - for 2D visualize 
	// set by updateBoxWCen2d
	int ui_x0;
	int ui_y0;
	int ui_width;
	int ui_height; 

	//=====================================
	// for logging switching

	QList<int> svm_pred_res; // initial in the setter of m_preTagLIDs
	int userSelect_rank; // update when switch
	
	//=====================================
	// log user changed

	int sup_modified;

	int box_modified;

	//=====================================
	// QML

	QStringList m_predTags;   
	QList<int> m_predTagLIDs;  

	int m_svmAnsLid; // initial in the setter of m_preTagLIDs //gen_boxMenu
	
	int m_confed;    // for UI lock --  
	
	int m_edited;    // for log UI editing     

	int m_switched;  // for UI touch 

	int m_confALL;   // 

	int m_boxCx2d;
	
	int m_boxCy2d;
	
};

#endif // BOXDATAV2_H
