#include "boxmaniTool.h"



void proj_world2img( Eigen::MatrixXf const & BOX3D_W , Eigen::MatrixXf const & TX_WORLD2IMG44 , Eigen::MatrixXf & out_box2dimg )
{
	int verbase=0;

	myassert(BOX3D_W.cols()==3,"Nx3 , BOX3D_W.cols()==3");

	int totalV = BOX3D_W.rows();

	Eigen::MatrixXf b4d(totalV,4);
	b4d.block(0,0,totalV,3)=BOX3D_W;
	b4d.col(4-1).setOnes();

	// Nx4
	b4d=(TX_WORLD2IMG44*(b4d.transpose())).transpose();

	// to R^3 
	//b4d=b4d.rowwise().hnormalized();

	b4d.col(0) = b4d.col(0).array() / b4d.col(3).array();
	b4d.col(1) = b4d.col(1).array() / b4d.col(3).array();
	b4d.col(2) = b4d.col(2).array() / b4d.col(3).array();

	// take first two as 2D location

	out_box2dimg.resize(totalV,2);
	out_box2dimg=b4d.block(0,0,totalV,2);

	return;
}

Eigen::Vector3f proj_fr2img( Eigen::Vector3f const & P3 , Eigen::MatrixXf const & TX_WORLD2IMG44  ,
						 Eigen::MatrixXf const & TX_FR2WORLD )
{
	int verbase=0;

	Eigen::Vector4f b4d;
	b4d<<P3,1;

	// 44 * 4x1
	b4d=(TX_WORLD2IMG44*TX_FR2WORLD*(b4d));

	// to R^3 
	//b4d=b4d.rowwise().hnormalized();

	b4d(0)=b4d(0)/b4d(4-1);
	b4d(1)=b4d(1)/b4d(4-1);
	b4d(2)=b4d(2)/b4d(4-1);


	return b4d.head(3);
}

Eigen::Vector3f proj_world2img( Eigen::Vector3f const & P3 , Eigen::MatrixXf const & TX_WORLD2IMG44  )
{
	int verbase=0;

	Eigen::Vector4f b4d;
	b4d<<P3,1;

	// 44 * 4x1
	b4d=(TX_WORLD2IMG44*(b4d));

	// to R^3 
	//b4d=b4d.rowwise().hnormalized();

	b4d(0)=b4d(0)/b4d(4-1);
	b4d(1)=b4d(1)/b4d(4-1);
	b4d(2)=b4d(2)/b4d(4-1);


	return b4d.head(3);
}


// return the intersected area (remaining area)
//
double screenSpaceCuted(
	BGPolygon const & out_box2dimg , 
	const int SWIDTH , const int SHEIGHT , const int TX, const int TY)
{ 
	 


	BGPolygon blue;

	boost::geometry::read_wkt(
		"POLYGON((4.0 -0.5 , 3.5 1.0 , 2.0 1.5 , 3.5 2.0 , 4.0 3.5 , 4.5 2.0 , 6.0 1.5 , 4.5 1.0 , 4.0 -0.5))", blue);

	// get boost geometry for BOX 2d
	BGPolygon screenPly; 
	
	//ccw 
	screenPly.outer().push_back(BGPoint(0+TX,0+TY));  
	screenPly.outer().push_back(BGPoint(SWIDTH+TX,0+TY));  
	screenPly.outer().push_back(BGPoint(SWIDTH+TX,SHEIGHT+TY));  
	screenPly.outer().push_back(BGPoint(0+TX,SHEIGHT+TY));  
	screenPly.outer().push_back(BGPoint(0+TX,0+TY));  


	std::deque<BGPolygon> output;
	bg::intersection(out_box2dimg,screenPly,output);

	if( output.size() > 0 ){

		myassert(output.size()==1,"intersection candidate > 1");

		double remainArea=0;
		// return remaining ratio
		for( std::deque<BGPolygon>::iterator iter=output.begin() ; iter != output.end() ; iter++  ){
			remainArea=remainArea+bg::area(*iter);
		}
		
		return remainArea;

	}else{

		return 0;
	}

}


bool isBoxIntersected(BoxDataV2 const & BOX_U ,BoxDataV2 const & BOX_V)
{
	// check u,v intersection 
	//
	//     |  V  |
	//		 |
	// |  U   |
	//   
	const float* uvtxs = BOX_U.getBox3d_fr();

	Eigen::MatrixXf boxU_fr = BOX_U.getBox3d_fr_n3();
	Eigen::MatrixXf boxV_fr = BOX_V.getBox3d_fr_n3();

	//[TEST] 3D intersection
	// project into XZ
	// project into XY
	// project into YZ
	//		> find CONVEX HULL
	//		> check intersection 

	BGPolygon u_fxy,u_fyz,u_fxz;
	BGPolygon v_fxy,v_fyz,v_fxz;

	for( int iv=0 ; iv<8 ; iv++ ){

		u_fxy.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,1))); 
		v_fxy.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,1)));

		u_fyz.outer().push_back(BGPoint(boxU_fr(iv,1),boxU_fr(iv,2))); 
		v_fyz.outer().push_back(BGPoint(boxV_fr(iv,1),boxV_fr(iv,2)));

		u_fxz.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,2))); 
		v_fxz.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,2)));
	}

	BGPolygon u_hxy,u_hyz,u_hxz;
	BGPolygon v_hxy,v_hyz,v_hxz;

	boost::geometry::convex_hull(u_fxy, u_hxy);
	boost::geometry::convex_hull(u_fyz, u_hyz);
	boost::geometry::convex_hull(u_fxz, u_hxz);

	boost::geometry::convex_hull(v_fxy, v_hxy);
	boost::geometry::convex_hull(v_fyz, v_hyz);
	boost::geometry::convex_hull(v_fxz, v_hxz); 

	std::deque< bg::model::polygon<BGPoint, false, true> > xyOver,yzOver,xzOver;

	bg::intersection(u_hxy,v_hxy,xyOver);
	bg::intersection(u_hyz,v_hyz,yzOver);
	bg::intersection(u_hxz,v_hxz,xzOver);

	bool xyIsST = xyOver.size()>0;
	bool yzIsST = yzOver.size()>0;
	bool xzIsST = xzOver.size()>0;

	bool uv_3dintersected = xyIsST && yzIsST && xzIsST;


	// check full contain
	//(boost::geometry::within(p, poly) 

	bool xy_contain = boost_a_containBy_b(v_hxy,u_hxy);
	bool yz_contain = boost_a_containBy_b(v_hyz,u_hyz);
	bool xz_contain = boost_a_containBy_b(v_hxz,u_hxz);

	bool full_contain=xy_contain && yz_contain && xz_contain;

	bool uv_part_intersect= xyIsST+yzIsST+xzIsST+xy_contain+yz_contain+xz_contain == 3 ;
	 
	if(uv_3dintersected){
		return true;
	}else if( full_contain  ){
		return true;
	}else if( uv_part_intersect ){
		// 2 intersection + 1 containing 
		return true;
	}else{
		// NOT INTERSECTION 
		return false;
	}


}

/**----------------------------------------------------------------------
* @return overlapping area , if return value =0 ==> U is not support V
*
/**---------------------------------------------------------------------*/
float uIsSupportV(BoxDataV2 const & BOX_U , BoxDataV2 const & BOX_V )
{
	//
	//     |  V  |
	//		 |
	// |  U    |
	//  

	const float* uvtxs = BOX_U.getBox3d_fr();

	Eigen::MatrixXf boxU_fr = BOX_U.getBox3d_fr_n3();
	Eigen::MatrixXf boxV_fr = BOX_V.getBox3d_fr_n3();

	float u__maxY = boxU_fr.col(2-1).maxCoeff();
	float v__minY = boxV_fr.col(2-1).minCoeff();

	const float CL_YTH = 0.2;

	//[TEST] Y-axis distance
	if ( abs(u__maxY-v__minY) > CL_YTH)
	{
		return 0;
	}

	//[TEST] 3D intersection
	// project into XZ
	// project into XY
	// project into YZ
	//		> find CONVEX HULL
	//		> check intersection 

	BGPolygon u_fxy,u_fyz,u_fxz;
	BGPolygon v_fxy,v_fyz,v_fxz;

	for( int iv=0 ; iv<8 ; iv++ ){

		u_fxy.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,1))); 
		v_fxy.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,1)));

		u_fyz.outer().push_back(BGPoint(boxU_fr(iv,1),boxU_fr(iv,2))); 
		v_fyz.outer().push_back(BGPoint(boxV_fr(iv,1),boxV_fr(iv,2)));

		u_fxz.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,2))); 
		v_fxz.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,2)));
	}

	BGPolygon u_hxy,u_hyz,u_hxz;
	BGPolygon v_hxy,v_hyz,v_hxz;

	boost::geometry::convex_hull(u_fxy, u_hxy);
	boost::geometry::convex_hull(u_fyz, u_hyz);
	boost::geometry::convex_hull(u_fxz, u_hxz);

	boost::geometry::convex_hull(v_fxy, v_hxy);
	boost::geometry::convex_hull(v_fyz, v_hyz);
	boost::geometry::convex_hull(v_fxz, v_hxz);


	std::deque< bg::model::polygon<BGPoint, false, true> > xyOver,yzOver,xzOver;

	bg::intersection(u_hxy,v_hxy,xyOver);
	bg::intersection(u_hyz,v_hyz,yzOver);
	bg::intersection(u_hxz,v_hxz,xzOver);
	bool xyIsST = xyOver.size()>0;
	bool yzIsST = yzOver.size()>0;
	bool xzIsST = xzOver.size()>0;


	// [TEST] XZ Overlapping -- plane proj area
	//	> check XZ overlapping area  
	if( xyIsST && yzIsST && xzIsST ){
		// [INTERSECTED]

		// NOTE that we already pass Y dist checking
		myassert(xzOver.size()>0,"xzOver deque size not > 1");

		float  xzOverlapAREA = abs( boost::geometry::area(xzOver.front()) );
		return xzOverlapAREA;

	}else{

		// check XZ overlapping
		// NOTE that we already pass Y dist checking

		if( xzOver.size() > 0  ){
			float  xzOverlapAREA = abs( boost::geometry::area(xzOver.front()) ) ; 
			return xzOverlapAREA;
		}else
		{
			return 0;
		}

	}


}


bool boost_a_containBy_b( BGPolygon & a , BGPolygon & b){

	bool full_cover=true;
	for (auto &iter = a.outer().begin() ; iter != a.outer().end() ; iter++ )
	{
		full_cover = full_cover&bg::covered_by(*iter,b);
	}

	return full_cover;
}

//
// need to return a belif to determine a best parent
// 
bool uIsSupportV_v2(BoxDataV2 const & BOX_U , BoxDataV2 const & BOX_V ,double & belif)
{
	// support full containing .

	//
	//     |  V  |
	//		 |
	// |  U   |
	//   
	const float* uvtxs = BOX_U.getBox3d_fr();

	Eigen::MatrixXf boxU_fr = BOX_U.getBox3d_fr_n3();
	Eigen::MatrixXf boxV_fr = BOX_V.getBox3d_fr_n3();

	//[TEST] 3D intersection
	// project into XZ
	// project into XY
	// project into YZ
	//		> find CONVEX HULL
	//		> check intersection 

	BGPolygon u_fxy,u_fyz,u_fxz;
	BGPolygon v_fxy,v_fyz,v_fxz;

	for( int iv=0 ; iv<8 ; iv++ ){

		u_fxy.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,1))); 
		v_fxy.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,1)));

		u_fyz.outer().push_back(BGPoint(boxU_fr(iv,1),boxU_fr(iv,2))); 
		v_fyz.outer().push_back(BGPoint(boxV_fr(iv,1),boxV_fr(iv,2)));

		u_fxz.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,2))); 
		v_fxz.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,2)));
	}

	BGPolygon u_hxy,u_hyz,u_hxz;
	BGPolygon v_hxy,v_hyz,v_hxz;

	boost::geometry::convex_hull(u_fxy, u_hxy);
	boost::geometry::convex_hull(u_fyz, u_hyz);
	boost::geometry::convex_hull(u_fxz, u_hxz);

	boost::geometry::convex_hull(v_fxy, v_hxy);
	boost::geometry::convex_hull(v_fyz, v_hyz);
	boost::geometry::convex_hull(v_fxz, v_hxz); 

	std::deque< bg::model::polygon<BGPoint, false, true> > xyOver,yzOver,xzOver;

	bg::intersection(u_hxy,v_hxy,xyOver);
	bg::intersection(u_hyz,v_hyz,yzOver);
	bg::intersection(u_hxz,v_hxz,xzOver);

	bool xyIsST = xyOver.size()>0;
	bool yzIsST = yzOver.size()>0;
	bool xzIsST = xzOver.size()>0;

	bool uv_3dintersected = xyIsST && yzIsST && xzIsST;


	// check full contain
	//(boost::geometry::within(p, poly) 

	bool xy_contain = boost_a_containBy_b(v_hxy,u_hxy);
	bool yz_contain = boost_a_containBy_b(v_hyz,u_hyz);
	bool xz_contain = boost_a_containBy_b(v_hxz,u_hxz);

	bool full_contain=xy_contain && yz_contain && xz_contain;

	bool uv_part_intersect= xyIsST+yzIsST+xzIsST+xy_contain+yz_contain+xz_contain == 3 ;


	if( full_contain){

		// not 3d intersection
		// ground check
		const float TH_GROUND = 0.10;
		float v__minY = boxV_fr.col(2-1).minCoeff();
		bool v_not_touchGround = (v__minY>TH_GROUND );

		if( full_contain && v_not_touchGround ){

			// support case
			belif = 1;
			return true;

		}else{

			// full contain but very close to ground
			// rare state 
			// fail 
			belif=0.1;
			return false;
		}

	}else if(uv_3dintersected){
		//	> check XZ overlapping area  
		// [INTERSECTED]

		// max Y of V should still > U
		//       ----v-----
		// ----u-|--      |
		//       |_ | ____|
		//          | 

		float u_h = boxU_fr.col(2-1).maxCoeff()-boxU_fr.col(2-1).minCoeff();
		float v_h = boxV_fr.col(2-1).maxCoeff()-boxV_fr.col(2-1).minCoeff();

		const float Height_RTH = 1.3;

		float uv_hratio = v_h / u_h ;
		//=============================================================================
		float u__maxY = boxU_fr.col(2-1).maxCoeff();
		float u__minY = boxU_fr.col(2-1).minCoeff();

		float v__minY = boxV_fr.col(2-1).minCoeff();
		float v__maxY = boxV_fr.col(2-1).maxCoeff();


		float height_diff = abs(v__minY-u__maxY);
		
		float height_diff_ratio = height_diff/v_h;

		float height_delta;

		if (v__minY < u__minY )
		{
			height_delta =abs( v__maxY - u__minY);
		}else{
			//v__minY > u__minY
			height_delta =abs( u__maxY - v__minY);
		}


		//=============================================================================
		const float TH_GROUND_L = 0.2;
		const float TH_GROUND_STRICT= 0.09;


		if( v__minY < TH_GROUND_STRICT ){ 
			// v are very close to ground
			belif=0.1;
			return false;
		}else if( TH_GROUND_STRICT<= v__minY && 
			v__minY < TH_GROUND_L && 
			height_diff_ratio >= 0.9  ){
				//
				belif=0;
				return false;
		}else if(  !( v__maxY - u__maxY + 0.1 > 0 ) && !(v__minY >0.2)  ){
			belif=0;
			return false; 

		}else if( u__minY > v__minY &&  v_h / u_h  > 1 && height_delta/v_h < 0.8 ){
			// u is high than v AND  v is large than u AND 

			// for floating V is under U --> 
			// max Y of V should still > U
			//       ----U-----
			// ----V-|--      |
			//       |_ | ____|
			//          |
			//          |

			belif=0;
			return false; 

		}


		// check overlapping area

		myassert(xzOver.size()>0,"xzOver deque size not > 1");

		float  xzOverlapAREA = abs( boost::geometry::area(xzOver.front())  );
		float v_xz_size = abs( bg::area(v_hxz) );

		// set belif 
		belif= xzOverlapAREA/v_xz_size;

		return belif > 0.1 ;

		

	}else if( uv_part_intersect ){
		// 2 intersection + 1 containing 


		// check ground distance 
		// and 
		// area ratio

		const float TH_GROUND = 0.10;
		float v__minY = boxV_fr.col(2-1).minCoeff();
		bool v_not_touchGround = (v__minY>TH_GROUND );

		if( v_not_touchGround && xzOver.size() > 0 ){

			//check area ratio 
			float  xzOverlapAREA = abs( boost::geometry::area(xzOver.front()) ); 
			float v_xz_size = abs( bg::area(v_hxz) );

			// set belif 
			belif= xzOverlapAREA/v_xz_size; 
			return belif > 0.35 ; 

		}else{
			// rare cases
			belif=0.1;
			return 0;
		}

	}else{
		// NOT INTERSECTION 

		// normal case
		float u__maxY = boxU_fr.col(2-1).maxCoeff();
		float v__minY = boxV_fr.col(2-1).minCoeff();

		const float CL_YTH = 0.2;

		//[TEST] Y-axis distance 
		//two boxes should not leave too far
		if ( abs(u__maxY-v__minY) > CL_YTH)
		{
			belif=0;
			return 0;
		}

		// check XZ overlapping
		// NOTE that we already pass Y dist checking 
		if( xzOver.size() > 0  ){
			float  xzOverlapAREA = abs(boost::geometry::area(xzOver.front())) ; 
			float v_xz_size = abs(bg::area(v_hxz));

			// set belif 
			belif= xzOverlapAREA/v_xz_size;

			return belif > 0.35 ; 
		}else{

			belif=0;
			return 0;
		}

	}

	//=============================================================================


}




void box_3D_distance( BoxDataV2 const & BOX_U , BoxDataV2 const & BOX_V , 
					  double & xz_dist , double & xz_overAreaRatio , double & y_dist , int & good_y_cen )
{
	// support full containing 

	//
	//    |   V   |
	//   	 |
	// |  U    |
	//    
	Eigen::MatrixXf boxU_fr = BOX_U.getBox3d_fr_n3();
	Eigen::MatrixXf boxV_fr = BOX_V.getBox3d_fr_n3();

	//[TEST] 3D intersection
	// project into XZ
	// project into XY
	// project into YZ
	//		> find CONVEX HULL
	//		> check intersection 

	BGPolygon u_fxy,u_fyz,u_fxz;
	BGPolygon v_fxy,v_fyz,v_fxz;

	for( int iv=0 ; iv<8 ; iv++ ){

		u_fxy.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,1))); 
		v_fxy.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,1)));

		u_fyz.outer().push_back(BGPoint(boxU_fr(iv,1),boxU_fr(iv,2))); 
		v_fyz.outer().push_back(BGPoint(boxV_fr(iv,1),boxV_fr(iv,2)));

		u_fxz.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,2))); 
		v_fxz.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,2)));
	}

	BGPolygon u_hxy,u_hyz,u_hxz;
	BGPolygon v_hxy,v_hyz,v_hxz;

	boost::geometry::convex_hull(u_fxy, u_hxy);
	boost::geometry::convex_hull(u_fyz, u_hyz);
	boost::geometry::convex_hull(u_fxz, u_hxz);

	boost::geometry::convex_hull(v_fxy, v_hxy);
	boost::geometry::convex_hull(v_fyz, v_hyz);
	boost::geometry::convex_hull(v_fxz, v_hxz); 

	std::deque< bg::model::polygon<BGPoint, false, true> > xyOver,yzOver,xzOver;

	bg::intersection(u_hxy,v_hxy,xyOver);
	bg::intersection(u_hyz,v_hyz,yzOver);
	bg::intersection(u_hxz,v_hxz,xzOver);

	bool xyIsST = xyOver.size()>0;
	bool yzIsST = yzOver.size()>0;
	bool xzIsST = xzOver.size()>0;

	bool uv_3dintersected = xyIsST && yzIsST && xzIsST;
	 
	// check full contain
	//(boost::geometry::within(p, poly) 

	bool xy_contain = boost_a_containBy_b(v_hxy,u_hxy);
	bool yz_contain = boost_a_containBy_b(v_hyz,u_hyz);
	bool xz_contain = boost_a_containBy_b(v_hxz,u_hxz);

	bool full_contain=xy_contain && yz_contain && xz_contain;

	bool uv_part_intersect= xyIsST+yzIsST+xzIsST+xy_contain+yz_contain+xz_contain == 3 ;

	//=============================================================================
	float u__maxY = boxU_fr.col(2-1).maxCoeff();
	float v__minY = boxV_fr.col(2-1).minCoeff();
	float v__maxY = boxV_fr.col(2-1).maxCoeff();

	float v_height = v__maxY - v__minY ;

	float height_diff = abs(v__minY-u__maxY);
	float height_diff_ratio = height_diff/v_height;

	Eigen::Vector3f u_cen=boxU_fr.colwise().mean();
	Eigen::Vector3f v_cen=boxV_fr.colwise().mean();

	//=============================================================================
	good_y_cen = u_cen.y() < v_cen.y();
	y_dist = height_diff;

	if( xz_contain || xzIsST ){
		
		xz_dist=0;
		if(  bg::area(v_hxz)>0 ){

			float  xzOverlapAREA = abs( boost::geometry::area(xzOver.front())  ); 
			xz_overAreaRatio = xzOverlapAREA / bg::area(v_hxz);
		}else{
			xz_overAreaRatio =0;
		}

	}else{
		// no xz intersected 

		bg::model::linestring<BGPoint> box_u_lines;

		for( int iu=0 ; iu < 4 ;iu++ ){
			Eigen::Vector3f upts =boxU_fr.row(iu);
			box_u_lines.push_back(BGPoint(upts.x(),upts.z()));
		}

		// push first one to close
		Eigen::Vector3f upts =boxU_fr.row(0);
		
		box_u_lines.push_back(BGPoint(upts.x(),upts.z()));

		//-------------------------------------------------------------
		
		float minDist = std::numeric_limits<float>::infinity();
		float tmpDist = std::numeric_limits<float>::infinity();
		Eigen::Vector3f v_pts;

		for( int iv=0 ; iv < 4 ; iv++ ){
			v_pts=boxV_fr.row(iv);

			// v pts 
			BGPoint vpts_2d(v_pts.x(),v_pts.z());

			tmpDist=bg::distance(vpts_2d,box_u_lines);
			
			if( tmpDist < minDist ){
				minDist=tmpDist;
			}
		}

		// also check v_cen to box_u
		Eigen::Vector3f v_center = boxV_fr.colwise().mean();  
		BGPoint vpts_2d(v_center.x(),v_center.z()); 

		tmpDist=bg::distance(vpts_2d,box_u_lines);
		
		if( tmpDist < minDist ){
			minDist=tmpDist;
		}

		xz_dist = minDist;
		xz_overAreaRatio = 0;
	}


	 

	//=============================================================================


}

float box_fr_y_distance( BoxDataV2 const & BOX_U , BoxDataV2 const & BOX_V )
{
	///                                parent ,                   child 

	// check intersection 
	//

	//-------------------------------------------------------------------------
	Eigen::MatrixXf boxU_fr = BOX_U.getBox3d_fr_n3();
	Eigen::MatrixXf boxV_fr = BOX_V.getBox3d_fr_n3();

	//[TEST] 3D intersection
	// project into XZ
	// project into XY
	// project into YZ
	//		> find CONVEX HULL
	//		> check intersection 

	BGPolygon u_fxy,u_fyz,u_fxz;
	BGPolygon v_fxy,v_fyz,v_fxz;

	for( int iv=0 ; iv<8 ; iv++ ){

		u_fxy.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,1))); 
		v_fxy.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,1)));

		u_fyz.outer().push_back(BGPoint(boxU_fr(iv,1),boxU_fr(iv,2))); 
		v_fyz.outer().push_back(BGPoint(boxV_fr(iv,1),boxV_fr(iv,2)));

		u_fxz.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,2))); 
		v_fxz.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,2)));
	}

	BGPolygon u_hxy,u_hyz,u_hxz;
	BGPolygon v_hxy,v_hyz,v_hxz;

	boost::geometry::convex_hull(u_fxy, u_hxy);
	boost::geometry::convex_hull(u_fyz, u_hyz);
	boost::geometry::convex_hull(u_fxz, u_hxz);

	boost::geometry::convex_hull(v_fxy, v_hxy);
	boost::geometry::convex_hull(v_fyz, v_hyz);
	boost::geometry::convex_hull(v_fxz, v_hxz); 

	std::deque< bg::model::polygon<BGPoint, false, true> > xyOver,yzOver,xzOver;

	bg::intersection(u_hxy,v_hxy,xyOver);
	bg::intersection(u_hyz,v_hyz,yzOver);
	bg::intersection(u_hxz,v_hxz,xzOver);

	bool xyIsST = xyOver.size()>0;
	bool yzIsST = yzOver.size()>0;
	bool xzIsST = xzOver.size()>0;

	bool uv_3dintersected = xyIsST && yzIsST && xzIsST;

	// check full contain
	//(boost::geometry::within(p, poly) 

	bool xy_contain = boost_a_containBy_b(v_hxy,u_hxy);
	bool yz_contain = boost_a_containBy_b(v_hyz,u_hyz);
	bool xz_contain = boost_a_containBy_b(v_hxz,u_hxz);

	bool full_contain=xy_contain && yz_contain && xz_contain;

	bool uv_part_intersect= xyIsST+yzIsST+xzIsST+xy_contain+yz_contain+xz_contain == 3 ;
	 

	//-------------------------------------------------------------------------
	
	if( uv_3dintersected ){

		return 0;
	}else{

		Eigen::MatrixXf boxU_fr = BOX_U.getBox3d_fr_n3();
		Eigen::MatrixXf boxV_fr = BOX_V.getBox3d_fr_n3();


		float u__maxY = boxU_fr.col(2-1).maxCoeff();
		float u__minY = boxU_fr.col(2-1).minCoeff();

		float v__minY = boxV_fr.col(2-1).minCoeff();
		float v__maxY = boxV_fr.col(2-1).maxCoeff();


		float height_delta;

		if (v__minY < u__minY )
		{
			height_delta =abs( v__maxY - u__minY);
		}else{
			//v__minY > u__minY
			height_delta =abs( u__maxY - v__minY);
		}

		return height_delta;

	}

}

void box_3D_distance_resolve_wall_obj( BoxDataV2 const & BOX_U , BoxDataV2 const & BOX_V , bool & useObjLabel )
{
	///                                                 parent ,                   child 
	// support full containing 

	//
	//    |   V   |
	//   	 |
	// |  U    |
	//    
	Eigen::MatrixXf boxU_fr = BOX_U.getBox3d_fr_n3();
	Eigen::MatrixXf boxV_fr = BOX_V.getBox3d_fr_n3();

	//[TEST] 3D intersection
	// project into XZ
	// project into XY
	// project into YZ
	//		> find CONVEX HULL
	//		> check intersection 

	BGPolygon u_fxy,u_fyz,u_fxz;
	BGPolygon v_fxy,v_fyz,v_fxz;

	for( int iv=0 ; iv<8 ; iv++ ){

		u_fxy.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,1))); 
		v_fxy.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,1)));

		u_fyz.outer().push_back(BGPoint(boxU_fr(iv,1),boxU_fr(iv,2))); 
		v_fyz.outer().push_back(BGPoint(boxV_fr(iv,1),boxV_fr(iv,2)));

		u_fxz.outer().push_back(BGPoint(boxU_fr(iv,0),boxU_fr(iv,2))); 
		v_fxz.outer().push_back(BGPoint(boxV_fr(iv,0),boxV_fr(iv,2)));
	}

	BGPolygon u_hxy,u_hyz,u_hxz;
	BGPolygon v_hxy,v_hyz,v_hxz;

	boost::geometry::convex_hull(u_fxy, u_hxy);
	boost::geometry::convex_hull(u_fyz, u_hyz);
	boost::geometry::convex_hull(u_fxz, u_hxz);

	boost::geometry::convex_hull(v_fxy, v_hxy);
	boost::geometry::convex_hull(v_fyz, v_hyz);
	boost::geometry::convex_hull(v_fxz, v_hxz); 

	std::deque< bg::model::polygon<BGPoint, false, true> > xyOver,yzOver,xzOver;

	bg::intersection(u_hxy,v_hxy,xyOver);
	bg::intersection(u_hyz,v_hyz,yzOver);
	bg::intersection(u_hxz,v_hxz,xzOver);

	bool xyIsST = xyOver.size()>0;
	bool yzIsST = yzOver.size()>0;
	bool xzIsST = xzOver.size()>0;

	bool uv_3dintersected = xyIsST && yzIsST && xzIsST;
	// check full contain
	//(boost::geometry::within(p, poly) 

	bool xy_contain = boost_a_containBy_b(v_hxy,u_hxy);
	bool yz_contain = boost_a_containBy_b(v_hyz,u_hyz);
	bool xz_contain = boost_a_containBy_b(v_hxz,u_hxz);

	bool full_contain=xy_contain && yz_contain && xz_contain;

	bool uv_part_intersect= xyIsST+yzIsST+xzIsST+xy_contain+yz_contain+xz_contain == 3 ;

	//=============================================================================
	float u__maxY = boxU_fr.col(2-1).maxCoeff();
	float v__minY = boxV_fr.col(2-1).minCoeff();
	float v__maxY = boxV_fr.col(2-1).maxCoeff();

	float v_height = v__maxY - v__minY ;

	float height_diff = abs(v__minY-u__maxY);
	float height_diff_ratio = height_diff/v_height;

	Eigen::Vector3f u_cen=boxU_fr.colwise().mean();
	Eigen::Vector3f v_cen=boxV_fr.colwise().mean();

	//=============================================================================
	
	useObjLabel=false;


	if( uv_3dintersected ){

		useObjLabel=1;

	}else if(full_contain){

		useObjLabel=1;

	}else {
		float xz_overAreaRatio;
		if(  bg::area(v_hxz)>0 ){

			float  xzOverlapAREA = abs( boost::geometry::area(xzOver.front())  ); 
			xz_overAreaRatio = xzOverlapAREA / bg::area(v_hxz);
		}else{
			xz_overAreaRatio =0;
		}

		useObjLabel = height_diff  < 0.1  && xz_overAreaRatio > 0.3;

	}
	//=============================================================================


}

void printALLBoxGeometryInfo( BoxDataV2 const & BOX )
{

	Eigen::MatrixXf aabb= Eigen::Map<const Eigen::MatrixXf>( BOX.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map<const Eigen::MatrixXf>( BOX.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<const Eigen::MatrixXf>( BOX.getBox3d_world() ,3,8);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();

	Eigen::Map<const Eigen::MatrixXf> obbFr2AABB( BOX.getFrOBB2AABB() ,3,3);
	Eigen::Map<const Eigen::MatrixXf> aabb2ObbFr( BOX.getAABB2FrOBB() ,3,3);
	std::cout<<"=============================\n";

	std::cout<<"box3d aabb"<<"\n"<<aabb<<"\n";
	std::cout<<"------------------------------\n";

	std::cout<<"box3d fr"<<"\n"<<BOX.getBox3d_fr_n3()<<"\n";
	std::cout<<"------------------------------\n";

	std::cout<<"box3d world"<<"\n"<<BOX.getBox3d_world_n3()<<"\n";
	std::cout<<"------------------------------\n";

	std::cout<<"boxCen world"<<"\n"<< (BOX.getBox3d_world_n3().colwise().mean()) <<"\n";
	std::cout<<"------------------------------\n";

	std::cout<<"obb_fr to aabb"<<"\n"<<obbFr2AABB<<"\n";
	std::cout<<"------------------------------\n";

	std::cout<<"aabb to obb_fr"<<"\n"<<aabb2ObbFr<<"\n";
	std::cout<<"------------------------------\n";
 
	std::cout<<"=============================\n"; 

}

Eigen::Vector3f constrDepthSensorV3( Eigen::VectorXf const & PLANE, int mx , int my , Eigen::MatrixXf const & oglMVP, 
									int panelWidth , int panelHeight )
{
	
	float pl_a = PLANE.x();
	float pl_b = PLANE.y();
	float pl_c = PLANE.z();
	float pl_d = PLANE.w();

	Eigen::Matrix3f A; 
	Eigen::Vector3f B;

	double term_x,term_y;

	term_x = (2.0*mx/(1.0*panelWidth)  - 1);
	term_y = (2.0*my/(1.0*panelHeight) - 1);

	A << oglMVP(1-1,1-1),oglMVP(1-1,2-1),oglMVP(1-1,3-1)-term_x,
		oglMVP(2-1,1-1),oglMVP(2-1,2-1),oglMVP(2-1,3-1)-term_y,
		pl_a,pl_b,pl_c;


	B << -1*oglMVP(1-1,4-1),-1*oglMVP(2-1,4-1),-1*pl_d;

	Eigen::Vector3f sol = A.fullPivHouseholderQr().solve(B);

	return sol;
}
