#include "stdafx.h"
#include "common_tool.h"


void printOPENCVType(cv::Mat & img){

	// find type
	int imgTypeInt = img.type()%8;
	std::string imgTypeString;

	switch (imgTypeInt)
	{
	case 0:
		imgTypeString = "8U";
		break;
	case 1:
		imgTypeString = "8S";
		break;
	case 2:
		imgTypeString = "16U";
		break;
	case 3:
		imgTypeString = "16S";
		break;
	case 4:
		imgTypeString = "32S";
		break;
	case 5:
		imgTypeString = "32F";
		break;
	case 6:
		imgTypeString = "64F";
		break;
	default:
		break;
	}

	// find channel
	int channel = (img.type()/8) + 1;

	std::stringstream type;
	type<<"CV_"<<imgTypeString<<"C"<<channel;

	std::cout<<type.str()<<std::endl;
}


void debugPrint( const char* msg, const int errlevel/*=0 */ )
{
	 
	if( errlevel==0 ){
		std::cout<<msg<<std::endl;
	}else{
		std::cout<<msg<<std::endl; 
	}
}

void mylog( const char*msg/*="" */, int logLevel/*=0*/ )
{
	std::cout<<"[LOG]:"<<msg<<std::endl;
}

void myassert( bool expr , const char* msg/*=""*/, int checkLevel/*=0*/ )
{
	if( !expr ){
		std::cout<<"[ASSERT FAILED]:"<<msg<<std::endl; 
		throw "***my assert fail***";
	}
}

void findCVMAP_State(cv::Mat const & ID_IMG , std::vector<int> &uidList )
{
	// only check effective index ( idx >= 1 )
	const int IMTYPE = ID_IMG.type();
	const int CHANNEL = ID_IMG.channels();

	uidList.clear();

	const int MAX_LID = 2048;
	int conatinFlag[MAX_LID]={0};

	const int NUM_PIX = ID_IMG.total();

	for(int k=0 ; k < NUM_PIX; k++){

		if( IMTYPE==CV_8UC1 ){

			uchar uid = ID_IMG.at<uchar>(k);
			if( uid > MAX_LID ){
				throw "uid > MAX_LID";
			}
			conatinFlag[uid]=1;

		}else if( IMTYPE==CV_8UC3   ){

			std::cout<<"unsupported DEPTH format "<<IMTYPE<<std::endl;

		}else if( IMTYPE==CV_16U || IMTYPE == CV_16UC1  ){
			
			int uid = ID_IMG.at<unsigned short>(k);
			
			if( uid > MAX_LID ){
				throw "uid > MAX_LID";
			}
			conatinFlag[uid]=1; 

		}else if( IMTYPE == CV_16SC1){
			
			int uid = ID_IMG.at<int>(k);

			if( uid > MAX_LID ){
				throw "uid > MAX_LID";
			} 
			conatinFlag[uid]=1; 

		}else if( IMTYPE==CV_32SC1  ){
			
			int uid = ID_IMG.at<int>(k);
			
			if( uid > MAX_LID ){
				throw "uid > MAX_LID";
			} 
			conatinFlag[uid]=1; 
			 
		}else{
			std::cout<<"unsupported DEPTH format "<<IMTYPE<<std::endl;
		}

	}

	// finalize
	for( int i=1 ; i < MAX_LID ; i++ ){
		if( conatinFlag[i] ){
			uidList.push_back(i);
		}
	}

	return;
}

void findCVIMGIdx( const int TARID, cv::Mat const & ID_IMG , std::vector<int> &idlist )
{
	// OPENCV is ROW MAJOR indexing
	// OPENCV is ROW MAJOR indexing 

	const int IMTYPE = ID_IMG.type();
	const int CHANNEL = ID_IMG.channels();

	idlist.clear();

	const int NUM_PIX = ID_IMG.total();
	for(int k=0 ; k < NUM_PIX; k++){

		if( IMTYPE==CV_8UC1 ){
			uchar uid = ID_IMG.at<uchar>(k);

			if( TARID == (int)uid ){
				idlist.push_back(k);
			}

		}else if( IMTYPE==CV_8UC3   ){
			std::cout<<"unsupported DEPTH format "<<IMTYPE<<std::endl;

		}else if( IMTYPE==CV_16U || IMTYPE == CV_16UC1  ){
			int uid = ID_IMG.at<unsigned short>(k);

			if( TARID == uid ){
				idlist.push_back(k);
			}

		}else if( IMTYPE == CV_16SC1){
			int uid = ID_IMG.at<int>(k);

			if( TARID == uid ){
				idlist.push_back(k);
			}

		}else if( IMTYPE==CV_32SC1  ){
			int uid = ID_IMG.at<int>(k);

			if( TARID == uid ){
				idlist.push_back(k);
			}
		}else{
			std::cout<<"unsupported DEPTH format "<<IMTYPE<<std::endl;
		}


	}
}

const std::string getTimestamp()
{
	static char mstr[100];
	std::memset(mstr,0,sizeof(mstr));
	sprintf(mstr,"%d",std::time(NULL));
	return std::string(mstr);
}

// F_PARTS="*.xxx"
std::vector<std::string> getFileList( const char* DIR_PATH , const char * F_PARTS/*=""*/)
{
	static char mstr[4096];
	std::memset(mstr,0,sizeof(mstr));
	if( std::strlen(DIR_PATH)>4096 ){
		std::cout<<"path too long:"<<std::strlen(DIR_PATH)<<std::endl;
		std::cerr<<"path too long:"<<std::strlen(DIR_PATH)<<std::endl;
		return std::vector<std::string>();
	}
	std::string tmpName= getTimestamp();

	sprintf(mstr,"dir \"%s\\%s\" /b  > %s",DIR_PATH , F_PARTS , tmpName.c_str() );
	//std::cout<<"SYS: "<<mstr<<std::endl;
	system(mstr);

	std::fstream fin;
	fin.open(tmpName.c_str() , std::ios::in );

	std::vector<std::string>  dirlist;
	std::string astr;
	while( fin ){
		std::getline(fin,astr);
		if( astr.size()>0 ){
			dirlist.push_back(astr);
		}
		astr.clear();
	}
	fin.close();
	std::memset(mstr,0,sizeof(mstr));
	sprintf(mstr,"del %s", tmpName.c_str() ); 
	system(mstr);

	return dirlist;
}



bool fileisExisted( const char* FPATH )
{
	std::fstream fin(FPATH , std::fstream::in | std::fstream::binary);

	return fin.is_open();

}

bool fileisExisted( const std::string SPATH )
{
	std::fstream fin(SPATH , std::fstream::in | std::fstream::binary);

	return fin.is_open();

}

Eigen::Vector3f prjPts2Plane( PlaneData const & PLANE , Eigen::Vector3f px )
{
	// check px is already on the plane or not
	if ( abs(PLANE.model.head(3).dot(px)+PLANE.model.w()) < 1e-12 )
	{
		return px;
	}
	 
	Eigen::Vector3f nv =  PLANE.model.head(3).normalized();

	Eigen::Vector3f pxp0 = px - PLANE.pts[0];

	float cosLen =nv.dot( pxp0 );

	if( pxp0.normalized().dot(nv) < 0 ){
	// if two vector are not in same direction.
		return (px - cosLen*nv);

	}else{ 
		return (px - cosLen*nv); 
	}

}

Eigen::Vector3f prjPts2Plane( Eigen::Vector3f px, Eigen::VectorXf const & MODEL , Eigen::Vector3f const & P0 )
{
	// check px is already on the plane or not
	if ( abs(MODEL.head(3).dot(px)+MODEL.w()) < 1e-8 )
	{
		return px;
	}

	if ( (px-P0).norm() < 1e-8 )
	{
		std::cout<<"prjPts2Plane: px=p0;\n";
		return px;
	}

	Eigen::Vector3f nv =  MODEL.head(3).normalized();

	float cosLen =nv.dot( px - P0 );

	return (px - cosLen*nv);
}

OGLPlaneData makePlane( PlaneData const & PLANE , float scale , Eigen::Vector3f center ,
					   Eigen::Matrix4f const & TX_MAT )
{
	bool verbase=0;

	Eigen::Vector3f px = PLANE.pts[0].array() + 0.2;

	Eigen::Vector3f p2 = prjPts2Plane(PLANE,px);

	Eigen::Vector3f nv = PLANE.model.head(3).normalized();
	Eigen::Vector3f tangent=(p2-PLANE.pts[0]).normalized();

	Eigen::Vector3f bitangent=nv.cross(tangent).normalized();

	center=prjPts2Plane(PLANE,center);
	 
	Eigen::Vector4f fr_v1;
	Eigen::Vector4f fr_v2;
	Eigen::Vector4f fr_v3;
	Eigen::Vector4f fr_v4;

	fr_v1 << prjPts2Plane(PLANE, center - (tangent * scale) - (bitangent * scale)) ,1 ;
	fr_v2 << prjPts2Plane(PLANE, center + (tangent * scale) - (bitangent * scale)) ,1 ;
	fr_v3 << prjPts2Plane(PLANE, center + (tangent * scale) + (bitangent * scale)) ,1 ;
	fr_v4 << prjPts2Plane(PLANE, center - (tangent * scale) + (bitangent * scale)) ,1 ;

	// TX 2 world spaces
	// 

	Eigen::Vector4f v1;
	Eigen::Vector4f v2;
	Eigen::Vector4f v3;
	Eigen::Vector4f v4;

	v1=TX_MAT*fr_v1;
	v2=TX_MAT*fr_v2;
	v3=TX_MAT*fr_v3;
	v4=TX_MAT*fr_v4;

	v1=v1.array()/v1.w();
	v2=v2.array()/v2.w();
	v3=v3.array()/v3.w();
	v4=v4.array()/v4.w();

	// calculate Center
	Eigen::Vector4f p0_h = PLANE.pts[0].homogeneous();
	Eigen::Vector3f p0_world = (TX_MAT*(p0_h)).hnormalized();

	
	// push data
	OGLPlaneData oglPlane;

	Eigen::Vector4f *vertices[]={&v1,&v2,&v3,&v4};

	int iv=0;

	for (int iv=0 ; iv < OGLPlaneData::NUM_TVEX ; iv++)
	{
		oglPlane.planeTriangles[iv*3+0]=vertices[iv]->x();
		oglPlane.planeTriangles[iv*3+1]=vertices[iv]->y();
		oglPlane.planeTriangles[iv*3+2]=vertices[iv]->z();
	} 

	const int IDX[6] ={ 0,1,2,0,2,3};

	for (int iv=0 ; iv< OGLPlaneData::NUM_TRI_IDX ; iv++ )
	{
		oglPlane.planeIndex[iv]=IDX[iv];
	}

	Eigen::Vector3f nv_p2 = p0_world.array()+nv.array()*scale;
	Eigen::Vector3f dir_p3 =v1.head(3);

	Eigen::Vector3f *line_vex[]={&p0_world,&nv_p2 , &p0_world, &dir_p3};

	// center
	for (int iv=0 ; iv<OGLPlaneData::NUM_LVEX ; iv++)
	{
		oglPlane.planeNV[iv*3+0]=line_vex[iv]->x();
		oglPlane.planeNV[iv*3+1]=line_vex[iv]->y();
		oglPlane.planeNV[iv*3+2]=line_vex[iv]->z();

	}

	oglPlane.planeNVIdx[0]=0;
	oglPlane.planeNVIdx[1]=1;
	oglPlane.planeNVIdx[2]=2;
	oglPlane.planeNVIdx[3]=3;



	if (verbase)
	{
		std::cout<<"-----------------------------\n";
		std::cout<<"fr pts:\n"
				 <<fr_v1.transpose()<<"\n"
				 <<fr_v2.transpose()<<"\n"
				 <<fr_v3.transpose()<<"\n"
				 <<fr_v4.transpose()<<"\n";
		std::cout<<"-----------------------------\n";
	}

	return oglPlane;
}


Eigen::MatrixXd readMatBinData(const char * FPATH ){

	bool verbase =0;

	typedef int Int32;
	typedef double Real64;

	if( !fileisExisted(FPATH) ){
		std::cout<<"model file:"<<FPATH<<std::endl;
		throw "model file not existed.";
	}
	if( verbase ){
		std::cout<<"INT32 ="<<sizeof(Int32)<<"B"<<std::endl;
		std::cout<<"real64 ="<<sizeof(Real64)<<"B"<<std::endl;
	}

	assert(sizeof(Int32)==4);
	assert(sizeof(Real64)==8);

	std::fstream fin = std::fstream(FPATH,std::fstream::in | std::fstream::binary );

	// read 2d matrix size
	Int32 s_row=0;
	Int32 s_col=0;
	fin.read((char*)&s_row,4); 
	fin.read((char*)&s_col,4); 

	Real64 *matData = new Real64[s_row*s_col]; 

	memset(matData,0,sizeof(Real64)*s_row*s_col );
	 

	fin.read((char*)matData,sizeof(Real64)*s_col*s_row);

	if (fin){
		if( verbase ){
			std::cout << "All data read successfully.\n";
		}
	}else{
		std::cout << "ERRORs: only " << fin.gcount() << " could be read\n";
		if( fin.eof() ){
			std::cout<<"FILE reach EOF\n";
		} 
	}

	fin.close();

	return Eigen::Map<Eigen::MatrixXd>(matData , s_row , s_col );

}

int getFSize(const char* BPATH)
{

	std::fstream file(BPATH,std::fstream::in| std::fstream::binary | std::fstream::ate);
	return file.tellg();
}

// load data generate by matlab
void readMatBinData(const char * FPATH , std::vector<double> & lids ,
					std::vector< std::vector<double> > & fdata )
{
	bool verbase= 0;

	lids.clear();
	fdata.clear();

	typedef int Int32;
	typedef double Real64;

	if( verbase ){
		std::cout<<"INT32 ="<<sizeof(Int32)<<"B"<<std::endl;
		std::cout<<"real64 ="<<sizeof(Real64)<<"B"<<std::endl;	
		// get length of file: 
		std::cout<<"File len :"<<getFSize(FPATH)<<std::endl;

	}

	assert(sizeof(Int32)==4);
	assert(sizeof(Real64)==8);


	std::fstream fin = std::fstream(FPATH,std::fstream::in | std::fstream::binary );

	// read 2d matrix size
	Int32 s_row=0;
	Int32 s_col=0;
	fin.read((char*)&s_row,4); 
	fin.read((char*)&s_col,4); 

	Real64 *matData = new Real64[s_row*s_col+2]; 

	memset(matData,0,sizeof(Real64)*s_row*s_col+2);

	
	fin.read((char*)matData,sizeof(Real64)*s_col*s_row);

	if (fin){
		if(verbase){ 
			std::cout<<"Try to read :"<<sizeof(Real64)*s_col*s_row<<std::endl;
			std::cout << "All data read successfully.\n";
		}
	}else{
		std::cout << "ERRORs: only " << fin.gcount() << " could be read\n";
		if( fin.eof() ){
			std::cout<<"FILE reach EOF\n";
		} 
	}


	fin.close();

	// re-parsing

	std::vector<double> vrow;

	// Matlab is column major
	for (int ir = 0; ir < s_row; ir++){


		// the first column is LIDs
		lids.push_back(matData[0*s_row+ir]);

		if(verbase && ir<10){
			std::cout<<"["<<ir<<"] - "<<std::setw(7)<<lids.back()<<"  ";
		}

		// temp storage
		vrow.clear();

		for( int ic=1; ic < s_col ; ic++ ){ 

			vrow.push_back(matData[ic*s_row+ir]); 

			if( verbase && ir<10){
				std::cout<<std::setw(10)<<matData[ic*s_row+ir]<<"  \t";
			}

		}
		//save
		fdata.push_back(vrow);

		if(verbase&& ir<10){
			std::cout<<'\n';
		}
	}

	if(verbase){
		std::cout<<"data loaded: "<<fdata.size()<<" x "<<fdata[0].size()<<std::endl;
	}

	delete [] matData;
}

float intersected_t0( Eigen::Hyperplane<float,3> const & PLANE , Eigen::ParametrizedLine<float,3> const & LINE )
{
	// So if I'm correct this is the t at which line intersect the last plane "focalPlane"    
	float t0 = LINE.intersection( PLANE ) ;

	// p0 to p1

	return t0 ;
}

Eigen::Vector3f intersectedPoint( Eigen::Hyperplane<float,3> const & PLANE , Eigen::ParametrizedLine<float,3> const & LINE )
{
	// So if I'm correct this is the t at which line intersect the last plane "focalPlane"    
	float t0 = LINE.intersection( PLANE ) ;

	// p0 to p1

	Eigen::Vector3f p0 = t0*(LINE.direction().normalized()) + LINE.origin() ;
	return p0;
}

void drawArrow( cv::Mat& img, cv::Point pStart, cv::Point pEnd, int len, cv::Scalar& color, int thickness, int lineType , int alpha_deg )
{
	const double PI = 3.1415926;    
	cv::Point arrow;    

	double angle = atan2((double)(pStart.y - pEnd.y), (double)(pStart.x - pEnd.x));  
	line(img, pStart, pEnd, color, thickness, lineType);   


	arrow.x = pEnd.x + len * cos(angle + PI * alpha_deg / 180);     
	arrow.y = pEnd.y + len * sin(angle + PI * alpha_deg / 180);  
	line(img, pEnd, arrow, color, thickness, lineType);   

	arrow.x = pEnd.x + len * cos(angle - PI * alpha_deg / 180);     
	arrow.y = pEnd.y + len * sin(angle - PI * alpha_deg / 180);    
	line(img, pEnd, arrow, color, thickness, lineType);
}

void colorCube( int index , float * rgb , const int MAX_NUM/*=70*/ )
{
	if (index > 69)
	{
		std::cout<<"reach max limit of object\n";
		index = index % 69;
	}

	static float tcolor[]={

		0.666667,0.333333,0.666667,
		0.000000,1.000000,1.000000,
		0.000000,0.000000,0.350000,
		1.000000,0.000000,1.000000,
		1.000000,0.333333,0.666667,
		0.666667,1.000000,0.000000,
		1.000000,0.333333,0.000000,
		0.333333,0.333333,1.000000,
		0.000000,0.333333,1.000000,
		1.000000,0.000000,0.000000,
		0.333333,1.000000,0.000000,
		0.250000,0.000000,0.000000,
		0.666667,0.000000,0.333333,
		1.000000,0.666667,0.000000,
		0.833333,0.233333,0.833333,
		0.000000,0.666667,0.333333,
		0.750000,0.000000,0.000000,
		0.333333,1.000000,0.666667,
		0.666667,0.000000,1.000000,
		1.000000,0.000000,0.666667,
		0.000000,0.000000,0.250000,
		1.000000,0.666667,0.333333,
		0.000000,0.000000,0.750000,
		1.000000,1.000000,1.000000,
		1.000000,0.666667,1.000000,
		0.666667,1.000000,0.666667,
		0.666667,0.666667,0.666667,
		0.000000,0.750000,0.000000,
		0.333333,0.333333,1.000000,
		0.333333,0.333333,0.666667,
		0.666667,0.333333,0.333333,
		0.333333,0.000000,1.000000,
		0.666667,0.666667,1.000000,
		0.333333,0.666667,0.333333,
		0.666667,0.333333,1.000000,
		0.000000,0.250000,0.000000,
		0.000000,0.000000,1.000000,
		1.000000,0.666667,0.000000,
		0.500000,0.500000,0.500000,
		0.000000,1.000000,0.000000,
		0.666667,0.666667,0.000000,
		0.666667,1.000000,1.000000,
		0.500000,0.000000,0.000000,
		0.333333,0.666667,0.666667,
		1.000000,0.333333,0.333333,
		0.000000,0.666667,0.666667,
		0.250000,0.000000,0.000000,
		1.000000,1.000000,0.666667,
		0.000000,0.000000,0.000000,
		0.666667,0.000000,0.666667,
		1.000000,0.000000,1.000000,
		0.333333,0.000000,0.666667,
		0.000000,0.000000,0.500000,
		1.000000,0.333333,1.000000,
		0.000000,1.000000,0.666667,
		0.333333,0.666667,1.000000,
		0.666667,1.000000,0.333333,
		0.333333,1.000000,1.000000,
		0.333333,1.000000,0.333333,
		0.333333,0.333333,0.000000,
		1.000000,0.666667,0.666667,
		1.000000,1.000000,0.333333,
		0.166667,0.166667,0.166667,
		0.333333,0.666667,0.000000,
		0.666667,0.333333,0.000000,
		0.666667,0.666667,0.333333,
		0.000000,0.333333,0.666667,
		0.000000,0.666667,1.000000,
		0.333333,0.000000,0.333333,
		1.000000,1.000000,0.000000,
		0.333333,0.333333,0.333333,
		1.000000,0.000000,0.333333,
		0.000000,0.500000,0.000000,
		0.000000,0.333333,0.333333,
		0.000000,1.000000,0.333333,
	};

	//-------------------------------------------------------------------------

	rgb[0]=tcolor[index*3];
	rgb[1]=tcolor[index*3+1];
	rgb[2]=tcolor[index*3+2];
}





void MyTimer::recordPeriod( clock_t time_now, TimerMode preMode , TimerMode nextMode )
{
	double pre_stage = time_now - period_start;

	switch (preMode)
	{ 
	case MyTimer::T_INI:
		//pass
		std::cout<<"Timer start at ini mode\n";
		break;
	case MyTimer::T_LABEL:
		total_label = total_label + pre_stage;
		break;
	case MyTimer::T_SUP:
		total_support = total_support + pre_stage;
		break;
	case MyTimer::T_BOX:
		total_box = total_box + pre_stage;
		break;

	default:
		throw "impossible";
		break;
	}

	// update state

	// re-set start
	period_start=time_now;

	curMode = nextMode;
}
