#pragma once
 
 
#include <vector>
#include <fstream>
#include <iostream>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <geoobj.h>
#include <Eigen/Eigen>
#include <ctime>
#include <QImage>

#define LOG_MSG_L0(X,...) printf(X, ##__VA_ARGS__);


class MyTimer
{
public:

	enum TimerMode
	{
		T_INI=0,
		T_LABEL,
		T_SUP,
		T_BOX
	};


	inline 
	void reset(){
		
		begin=0;
		end=0;
		
		totalTime=0;
		period_start=0;

		total_support=0;
		total_label=0;
		total_box=0;

		curMode=T_INI;
	}

	inline 
	void start(){
		reset();
		begin=std::clock();
		period_start=begin;
	}
	inline
	void startAtMode(TimerMode begMode){
		start();
		curMode=begMode;
	}
	 
	inline 
	void done(){
		end=std::clock();
		totalTime=end-begin;

		// record last stage
		recordPeriod( end , curMode,curMode) ;
	}

	inline 
	double getTotalSecond(){
		if( totalTime != 0  ){
			return totalTime/ CLOCKS_PER_SEC;
		}else{
			return totalTime/ CLOCKS_PER_SEC;

			std::cout<< "totalTime=0\n";
		}
	}

	inline 
		double getTotalLabelSec(){ 
		return total_label/ CLOCKS_PER_SEC;
	}
	inline 
		double getTotalSupSec(){ 
			return total_support/ CLOCKS_PER_SEC;
	}
	inline 
		double getTotalBoxSec(){ 
			return total_box/ CLOCKS_PER_SEC;
	}
	//-------------------------------------------------------------------------
	
	inline
	void switch2SupMode(){

		clock_t time_now = std::clock(); 
		recordPeriod(time_now,curMode , T_SUP);
	}

	inline
	void switch2BoxMode(){

		clock_t time_now = std::clock(); 
		recordPeriod(time_now,curMode,T_BOX);
	}

	inline
	void switch2LabelMode(){

		clock_t time_now = std::clock(); 
		recordPeriod(time_now,curMode,T_LABEL);
	}

private:

	void recordPeriod(clock_t time_now, TimerMode preMode , TimerMode nextMode);

private:

	clock_t begin;
	clock_t end;

	clock_t period_start;

	double totalTime;
	
	double total_support;
	double total_label;
	double total_box;

	TimerMode curMode;
};

void colorCube( int index , float * rgb , const int MAX_NUM=70 );



void printOPENCVType(cv::Mat & img);
 
void debugPrint(const char* msg, const int errlevel=0 );
  
void mylog(const char*msg="" , int logLevel=0);

void findCVIMGIdx( const int TARID, cv::Mat const & ID_IMG , std::vector<int> &idlist);

void myassert(bool expr , const char* msg="", int checkLevel=0);

void findCVMAP_State(cv::Mat const & ID_IMG , std::vector<int> &uidList );

bool fileisExisted(const char* PATH);

bool fileisExisted( const std::string SPATH );

void drawArrow(cv::Mat& img, cv::Point pStart, cv::Point pEnd, int arrlen,       
			   cv::Scalar& color, int thickness, int lineType , int alpha_deg=30 );

Eigen::Vector3f prjPts2Plane( PlaneData const & PLANE , Eigen::Vector3f px );

Eigen::Vector3f prjPts2Plane( Eigen::Vector3f px, Eigen::VectorXf const & MODEL , Eigen::Vector3f const & P0 );

OGLPlaneData makePlane( PlaneData const & PLANE , float scale , Eigen::Vector3f center ,  Eigen::Matrix4f const & TX_MAT  );

const std::string getTimestamp();

std::vector<std::string> getFileList( const char* DIR_PATH , const char * F_PARTS/*=""*/);

 
void readMatBinData(const char * FPATH , std::vector<double> & lids , std::vector< std::vector<double> > & fdata );

Eigen::MatrixXd readMatBinData(const char * FPATH );


int getFSize(const char* BPATH);


template<typename T>
void printSTD(std::vector<T> const &VEC ,const char*ss="")
{
	std::cout<<ss<<":";

	for( size_t i=0 ; i< VEC.size() ; i++){
		std::cout<<VEC[i]<<",";
	}
	std::cout<<"\n";

}


float intersected_t0( Eigen::Hyperplane<float,3> const & PLANE , 
				    Eigen::ParametrizedLine<float,3> const & LINE );

Eigen::Vector3f intersectedPoint( Eigen::Hyperplane<float,3> const & PLANE , 
				   Eigen::ParametrizedLine<float,3> const & LINE );

/*
mygluUnProject(GLdouble winx, GLdouble winy, GLdouble winz,
			 Eigen::MatrixXf mvp,
			 const GLint viewport[4],
			 GLdouble *objx, GLdouble *objy, GLdouble *objz)
{
	double finalMatrix[16]; // mvp
	double in[4];
	double out[4];

	//__gluMultMatricesd(modelMatrix, projMatrix, finalMatrix);
	//if (!__gluInvertMatrixd(finalMatrix, finalMatrix)) return(GL_FALSE);

	in[0]=winx;
	in[1]=winy;
	in[2]=winz;
	in[3]=1.0;

	/* Map x and y from window coordinates * /
	in[0] = (in[0] - viewport[0]) / viewport[2];
	in[1] = (in[1] - viewport[1]) / viewport[3];

	/* Map to range -1 to 1 * /
	in[0] = in[0] * 2 - 1;
	in[1] = in[1] * 2 - 1;
	in[2] = in[2] * 2 - 1;

	__gluMultMatrixVecd(finalMatrix, in, out);
	if (out[3] == 0.0) return(GL_FALSE);
	out[0] /= out[3];
	out[1] /= out[3];
	out[2] /= out[3];
	*objx = out[0];
	*objy = out[1];
	*objz = out[2];
	return(GL_TRUE);
}
*/