#ifndef FCPP_GL_PANEL
#define FCPP_GL_PANEL


#pragma warning( disable : 4819  )

#include <QObject> 
#include <QImage>
#include <QGLFunctions>
#include <QPixmap>
#include <QPainter> 
#include <QQuickPaintedItem>
#include <QtGui/QOpenGLShaderProgram>
#include <QList>
#include <QString>
#include <QStringList>
#include<QOpenGLFunctions>

#include <memory>
#include <Eigen/Eigen>
#include <Eigen/Geometry>
#include <functional>
#include <vector> 

#include "mytrackball.h"
#include "fcstatemag.h" 
#include "loader.h"
#include "common_tool.h"
#include "pcd_tools.h"
#include "box_serializer.h"
#include "geoobj.h"
#include "textloader.h"
#include "data_collecter.h"
#include "boxmaniTool.h"
#include "pcd_boxquality_tool.h"
#include "pcabox.h"


#ifndef GL_MULTISAMPLE
#define GL_MULTISAMPLE  0x809D
#endif


namespace Tirth{

	class FCGLPanel :public QQuickPaintedItem  
	{

	Q_OBJECT
		//Q_PROPERTY(int imgID READ getImgID WRITE setImgID )
		//Q_PROPERTY(int brushSize READ getBrushSize WRITE setBrushSize )
		//Q_PROPERTY( QString stateStr READ getCurStateStr NOTIFY stateStrChange )  
	Q_PROPERTY( QList<QObject*> boxData READ getBoxData NOTIFY boxDataChanged )

	Q_PROPERTY( bool showSupport READ getShowSupport WRITE setShowSupport NOTIFY showSupChange )
	
	Q_PROPERTY( bool bxmi_x_spinVal READ getBxmiXPainVal WRITE setBxmiXSpinVal NOTIFY bxmiXPainValChanged )
	//Q_PROPERTY( bool bxmi_z_spinVal READ getShowSupport WRITE setShowSupport NOTIFY showSupChange )

	// Mode switch:
	// label
	// structure 
	// box
	Q_PROPERTY( QString ini_mode READ getIniMode() WRITE setIniMode NOTIFY onIniModeChanged )


	//============================================================================
	// DATA MEMBER
	public:

		inline
		void setShowSupport(bool val){

			if( showSupport==val ){ 
				return;
			}
			showSupport=val;
			showSupChange(showSupport);
			requireUpdateContourTex=1;
			
		}

		inline
		bool getShowSupport(){
			return showSupport;
		}

		inline
		void setBxmiXSpinVal(int newVal){
			if( _bxmi_spinXVal== newVal ){
				return;
			}
			_bxmi_spinXVal=newVal;
			bxmiXPainValChanged(_bxmi_spinXVal);
		}

		inline
		int getBxmiXPainVal(){
			return _bxmi_spinXVal;
		}

		inline
			void setIniMode(QString str){
				if( _ini_mode==str ){ 
					return;
				} 
				_ini_mode=str;
				onIniModeChanged(_ini_mode);
		}
		inline
			QString getIniMode()const{
				return _ini_mode;
		}

	private:
		//-------------------------------------------------------------------------
		Loader loader;

		Tirth::FCStateMag ctx;

		//-------------------------------------------------------------------------
		std::vector<int> trail_sidList;

		int trail_cur_k; 
		int requireUpdateContourTex;
		int requireUpdateBKTex;
		int requireUpdateDrawPaperTex;

		bool anno_force_rerun;

		// UI Flags
		bool showBK;
		bool showBox;
		bool showConfAllBox;  
		bool showSG;
		bool showSupport;
		bool showSuggestion;

		bool enableInstTest;


		bool use_eraser;

		// UI BINDING
		int _bxmi_spinXVal;

		// Mode switch:
		// label
		// structure 
		// box
		QString _ini_mode;

		std::string annosavePath;
		std::string lastFileDropPath;

		TextCTX tctx ;
		 
		//-------------------------------------------------------------------------
		// Rendering 

		QOpenGLShaderProgram *m_program_3d;
		QOpenGLShaderProgram *m_program_2d;
		QOpenGLShaderProgram *m_program_supadj;
		QOpenGLShaderProgram *m_program_bxmi_vex;
		QOpenGLShaderProgram *m_program_bxmi_rot;

		QOpenGLFunctions* oglEntry;
		GLuint tid;

		//-------------------------------------------------------------------------
		// State Control
		// next_state func(void) 
		typedef std::function< FCStateMag::State(FCGLPanel&) > UpdateFunc;
		typedef std::function< void( FCGLPanel&) > TransFunc;

		// state X signals 2D table
		UpdateFunc updateTable[FCStateMag::NUM_OF_STATE][FCStateMag::NUM_OF_USIG];

	//============================================================================
	public: 
		FCGLPanel(QQuickItem  * parent=0);
		~FCGLPanel(); 
		
		virtual void paint(QPainter * painter);
		 
		QList<QObject*> getBoxData(); 

	private: 

		inline
		QString getCurStateStr(){return QString::fromLocal8Bit(Tirth::FCMODESTRS[ctx.currentState]);} 

		inline
		QPoint getLastMPos(){ return ctx.lastPos ;} 

 		// utility
		void dispatch( FCStateMag::State fromS1 , FCStateMag::UISignal usignal );
		 
		// [Logic]
		//void initial();
		void resetCtx();
		void genBoxByGT();
		TextCTX const loadUIConf(const char*);
		void loadMatlabGTBox(const char* bpath ); 
		
		// [Room Extraction]
		void exFloor();  
		void calFRPts();
		void exWalls();
		void exWalls_v2();

		void convOGLPlane();

		// [ML]
		//void calBoxSupState();	 
		void calBoxSupState_v2();
		
		void updateBoxPlanes();
		int box_checkInst(const int IDX );
		int box_checkInst(const int IDX , BoxDataV2 const & refinedBox );
		int box_checkInst(const int SKIP_IDX1 , const int SKIP_IDX2 , BoxDataV2 const & refinedBox );
		int box_checkGroundInst(const int SKIP_IDX1 , const int SKIP_IDX2 , BoxDataV2 const & refinedBox );

		void checkBoxOccuState(const int IDX );

		void re_estiBoxFeatures();
		void update_sceneSupState(); 

	 	void make_a_pred_v2( BoxStateLog & stateLog );
		void gen_boxMenu_v2( std::vector<int> &unkFlag, int i, Eigen::MatrixXd &fvecData, bool verbase, std::vector<std::string> lnames, const int NUM_MENU );
 
		void local_refine(std::vector<int> const& UNK_FLAG);
		void global_refine(  std::vector<int> const& BOX_STATES  );
		void global_refine_v2(  std::vector<int> const& BOX_STATES  );
		
		void export_ini_batch();
		void auto_rerun_gtlabel();
		void auto_rerun_gtsupport();
		void reload_recalSupport();

		// [S/L]
		void loadCase(const int IMG_ID, bool useAutoSeg  );
		void fast_reloadCase(const int IMG_ID, bool useAutoSeg );
		void fast_loadifexist(const int IMG_ID,bool autoSave);
		void saveProcessedIniCase();
		void fast_reloadCase_processed( const int IMG_ID, bool useAutoSeg );
		void fast_reloadCase_processed( const int IMG_ID, const std::string INPUT_DIR ,bool useAutoSeg   );
	
		// [Rendering]
		void iniShader2d();
		void iniShader2d_seg();
		void iniShader2d_supadj();
		void iniShaderBxmi_vex();
		void iniShaderBxmi_Rot();

		void iniShader3d(); 
		void iniBKTex(); 
		void render2d();
		void render2d_seg_contour();

		void renderBox3d_v2();
		void renderSupport();
		void renderSceneStruct();
		
		void renderManiBox_Vex();
		void renderManiBox_Rot();

		void render2d_bk_supMode();
		void render2d_ui_supMode();
		void renderBox3d_v2_supadj();
		void renderBox3d_v2_boxsel();

		void checkGLError(const char * hint="");
		
		void drawContour_supSet(int focusIdx,int sugIdx ,cv::Mat rgba , std::vector< std::vector<std::vector<cv::Point> > > & all_coutours );
		void drawContour_sug(int focusIdx,int sugIdx ,cv::Mat rgba , std::vector< std::vector<std::vector<cv::Point> > > & all_coutours );
		void drawContour(int focusIdx ,cv::Mat rgba , std::vector< std::vector<std::vector<cv::Point> > > & all_coutours );
		
		void updateCVDraw_Tex();
		void updateCVSegTex();

		void iniContour();
		void drawSupport();

		void updateBoxPoly2D();
		void updateBox2DSeg();

		void updateCVSupSelectTex();
		void updateCVSupSetTex();
		
		// [Debug]
		void paintTexturedCube();
		void setupTestBox(); 

		void forceReRunTCF();
		void ini_svm(TextCTX const & TCTX);
		void convert2segLoc(int & wx , int & wy);

		void cal2DPixelOverlapp(const int IDX, int & num_seg , int & num_box, int & num_over , float & overRatio );
		void cal2DPixelOverlapp(Eigen::MatrixXf const& BOX3Dn3_WORLD , cv::Mat const & BOX_SEG_MASK, int & num_seg , int & num_box , int & num_over, float & overRatio );
	
		void calculate_box_conf();
		void makeNextBoxSuggestion(BoxStateLog const & BS_LOG);

		void resolveSupportLabel(int i );
		void resolveSupportLabel_v2( int const IDX );

		void testRefine();
		//-------------------------------------------------------------------------
		// [ EVENT Press Handler ]
		FCStateMag::State _runBatch();
		FCStateMag::State _run_tctf_mode();

		//
		void auto_fillAnswer();
		void auto_fillSupport();

		FCStateMag::State _anno_initialPred(); 
		FCStateMag::State _anno_rePred();
		FCStateMag::State _anno_allConf();
		FCStateMag::State _anno_allConf_v2();
		
		//
		FCStateMag::State _pauseTimer();
		FCStateMag::State _resumeTimer();

		//
		void _saveAnnotate();
		FCStateMag::State _test_refBox();

		//
		FCStateMag::State _actionDoneBack2Idle();
		FCStateMag::State _predSegMPressHandler();	
		FCStateMag::State _predMenuCloseHandler();

		FCStateMag::State _initial_entry();
		
		// annotation serials
		FCStateMag::State _anno_initial();
		FCStateMag::State _mode_sup_initial();
		FCStateMag::State _anno_load_aCase(); 
		FCStateMag::State _anno_save();
		FCStateMag::State _anno_move2Next();
		FCStateMag::State _anno_move2Prev();
		FCStateMag::State _anno_reset(); 
		FCStateMag::State _anno_alldone(); 

		Tirth::FCStateMag::State _mode_sup_start();
		Tirth::FCStateMag::State _mode_sup_save();
		Tirth::FCStateMag::State _mode_sup_reset();
		Tirth::FCStateMag::State _mode_sup_move2Prev();
		Tirth::FCStateMag::State _mode_sup_move2Next();

		bool Tirth::FCGLPanel::_initialLoad_without_supesti();
		void Tirth::FCGLPanel::_load_preprocessed_case(const int IMG_ID);
		void Tirth::FCGLPanel::_load_preprocessed_case(const int IMG_ID, const std::string LOAD_BIN_DIR);

		Tirth::FCStateMag::State _mode_box_initial();
		Tirth::FCStateMag::State _mode_box_start();
		Tirth::FCStateMag::State _mode_box_save();
		Tirth::FCStateMag::State _mode_box_reset();
		Tirth::FCStateMag::State _mode_box_move2Prev();
		Tirth::FCStateMag::State _mode_box_move2Next();

		//-------------------------------------------------------------------------
		//box manipulate
		FCStateMag::State _bxmi_enterBoxManiMode();	
		FCStateMag::State _bxmi_mouseMove();
		FCStateMag::State _bxmi_mouseMove_vexEnt();

		void _bxmi_handle_y( int cv_x, int cv_y );
		void _bxmi_handle_xz( int mx, int my  );

		FCStateMag::State _bxmi_mousePressed();
		FCStateMag::State _bxmi_mouseReleased();
		FCStateMag::State _bxmi_keyPressed();
		FCStateMag::State _bxsel_actionDone();
		FCStateMag::State _bxsel_entermode();
		
		FCStateMag::State _bxsel_mouseMove();
		FCStateMag::State _bxsel_mousePressed();
		FCStateMag::State _bxsel_mouseReleased();
		FCStateMag::State _bxsel_keyPressed();

		FCStateMag::State _supadj_enterMode();
		FCStateMag::State _supadj_mouseMove();
		FCStateMag::State _supadj_mousePressed();
		FCStateMag::State _supadj_mouseReleased();
		FCStateMag::State _supadj_keyPressed();

		FCStateMag::State _supSel_mouseMove();
		FCStateMag::State _supSel_mousePressed();
		FCStateMag::State _supSel_mouseReleased();
		FCStateMag::State _supSel_keyPressed();
		FCStateMag::State _supSel_actionDone();

		Tirth::FCStateMag::State _drawobj_mouseMove();
		Tirth::FCStateMag::State _drawobj_addObjPressed();
		Tirth::FCStateMag::State _drawobj_mousePressed();
		Tirth::FCStateMag::State _drawobj_mouseReleased();
		Tirth::FCStateMag::State _drawobj_keyPressed();
		Tirth::FCStateMag::State _drawobj_actionDone();

		Tirth::FCStateMag::State _draw_initial();
		Tirth::FCStateMag::State _draw_load_aCase();
		Tirth::FCStateMag::State _draw_next();
		Tirth::FCStateMag::State _draw_previous();

		Tirth::FCStateMag::State _menuOpen_MPressedHandler();
		Tirth::FCStateMag::State _predMouseMove();
		Tirth::FCStateMag::State _load_config();
		//-------------------------------------------------------------------------
		Tirth::FCStateMag::State _anno_initialPred__forexport();

	//============================================================================
	signals:   
		void stateStrChange(QString);
		void tagIsReady();
		void boxDataChanged();
		
		void segClick(int,int);
		void segMMoveFocusChange(int);

		void showSupChange(bool);
		void bxmiXPainValChanged(int);
		void onIniModeChanged(QString);

	public slots :
		//-------------------------------------------------------------------------
		// signal forwarding

		//------------------------------------------
		// mouse Events
		void mousePressEvent(int x, int y, int btu);
		void mouseReleaseEvent(int x, int y);
		void mouseMoveEvent(int x, int y); 
		void mouseEnter();
		void mouseLeave();

		void keyPressEvent(QKeyEvent *event);

		//------------------------------------------
		void textLoadBtuPress();
		void inipredModeBtuPressed();
		void confALLBtuPressed();
		void pauseBtuPressed();
		void actionDoneBtuPressed();
		void saveLabelBtuPressed();

		void qmlRequestRePred();
		void qmlMenuCloseRaise();

		void file_drop_slot(QString );

		void anno_iniListPressed();
		void anno_startPressed();
		void anno_resetPressed();
		void anno_allDonePressed();
		void anno_nextCasePressed();
		void anno_prevCasePressed();
		void anno_saveResPressed();

		void bxmi_enterBoxMiModePressed();
		void supadj_enterSupAdjModePressed();
		 
		void handle_bxmiXPainValChanged();

		void addObjSegSlot();
		//-------------------------------------------------------------------------
		// BYPASS State machine
		void directExportSlot();
		void de_rerun_refineSlot();
		
		// eval
		void collectDataSlot();

		void debugSlot();
		void fastOpenSlot(int imid);

		void toggleInstTest();
		void toggleShowWall();

		void paintBox(bool enable); 
		void enableSuggest(bool enable);

		void paintSeg(bool enable);
		void paintBK(bool enable);

		void setBoxManiMode(QString modestr);

		void setSupMode_showBox(bool enable);
		
		QString getDisplayInfo();
		QString getAnnoInfo();

		void useEraser(bool enable);

	//============================================================================
	//[system slot]
	private slots:

		void handleWindowChanged(QQuickWindow *win);
		
		// 3d rendering state sync 
		void stateSYNC_3drendering();

		void paint_ogl_bef();
		void paint_ogl_aft();
		void ogl_cleanup();

		void debugCheck();

	private:
		void conv_CTX_to_NaiveSCTX(NaiveSceneCTX & scectx);
		void conv_NaiveSCTX_to_CTX(NaiveSceneCTX & SCTX);
		void box2dCenUpdate();

		void optimalBoxSize(const int IDX);
		void testBoxOpt();
		void optimalBoxSize_inn(const int BOX_IDX,const int IDX_V1);
		int cal2DPixelOverlap_sim(const int IDX );
		int cal2DPixelOverlap_sim(Eigen::MatrixXf const& BOX3Dn3_WORLD , cv::Mat const & BOX_SEG_MASK );
	
		void iniBoxVoxelDS();
		void calculateBox3Error();
		void iniboxAfterLoading();
		void gen_boxMenu_v3( std::vector<int> &unkFlag, int ib, Eigen::MatrixXd &fvecData, bool verbase, std::vector<std::string> lnames, const int NUM_MENU );
		void gen_boxMenu_v2_5_allsvm( std::vector<int> &unkFlag, int ib, Eigen::MatrixXd &fvecData, bool verbase, std::vector<std::string> lnames, const int NUM_MENU );
		
		
		void draw_on_finerSeg( std::vector<int> const & SPTS, const int OBJ_ID );
		void ini_afterObjseg();
		void loadCase_uDrawMode( const int IMG_ID);
		void load_finerSeg(const int IMG_ID);

		void render2d_draw_objseg();
	
};

}




#endif

