#pragma once

#include <vector>
#include <string>
#include <Eigen/Eigen>
#include <map>
#include <opencv2/core/core.hpp>

#include <QList>

#include "boxdatav2.h"
#include "pcd_tools.h"
#include "common_tool.h"
#include "geoobj.h"
#include "textloader.h"
#include "svmproxy.h"
#include "freqModel.h"


namespace Tirth{

	// for box manipulate
	class BxmiCtx{
	public:
		
		enum ManiType
		{
			M_INI=0,
			M_ROT_Y=1,
			M_VEX_XZ,
			M_VEX_Y
		};		
		
		int bxmi_focus_boxIdx;
		int bxmi_sel_vidx;

		bool bxmi_pressed;
		bool drag_isgood;
		int press_x;
		int press_y;

		float drag_pts[3];
		float rot_theta;

		ManiType maniMode;

		BoxDataV2 box;

		BxmiCtx(){
			resetALL();
		}

		void resetControl(){

			bxmi_focus_boxIdx=-1;
			bxmi_sel_vidx=-1;
			
			bxmi_pressed=false;
			drag_isgood=0;
			press_x=-1;
			press_y=-1;
			rot_theta=0;

			memset(drag_pts,0,sizeof(drag_pts));
		}

		void resetALL(){
			
			resetControl();

			maniMode=M_INI;

		}
	};


	// for assign support relationship
	class SetSupCtx{
	public:

		static const int SUPSET_PT_OBJ=1;
		static const int SUPSET_PT_WALL=2;
		static const int SUPSET_PT_FLOOR=3;
		static const int SUPSET_PT_FLOAT=4;

		int supLocalIdx;
		int supParentType;

		int drag_mx;
		int drag_my;

		//-------------------------------------------------------------------------
		
		SetSupCtx(){
			resetALL();
		}

		void resetControl(){

			supLocalIdx=-1;
			supParentType=-1; 
			drag_mx=-1;
			drag_my=-1;
		}

		

		void resetALL(){

			resetControl();
			 
		}

	};

	class DrawSegCtx{


	public:
		int pressed;

		int objectCount;
		int objID;

		std::vector<int> strokePts;

		DrawSegCtx(){
			resetALL();
		}

		void reiniControl(){ 
			pressed=0;
			strokePts.clear();
		}

		void resetALL(){ 
			reiniControl(); 
			objectCount=0;
			objID=1;
		}

	};

	// A simple structure to hold the state variables
	// NO METHODS
	struct FCStateMag
	{ 
	public:

		EIGEN_MAKE_ALIGNED_OPERATOR_NEW

		//-----------------------------------
#pragma region STATIC_REG

		
		// NOTE that also update STATE string in cpp
		static const int NUM_OF_STATE = 18+4+2+1+1;
		enum State{
			S_NULL=0,
			S_PRE_INI,
			S_INI,

			S_IDLE,
			S_CUTS_DRAWLINE,
			S_SELECT,
			S_ROOM,   

			S_CUTSP,
			S_EXBOX,
			S_PREDICT,
			S_PRE_PAUSE,
			S_PRE_MENU_FOLD,

			S_BOX_SEL,
			S_BOXMANI,
			S_SUP_EDITIDLE,
			S_SUP_ASSIG,
			S_FINAL, //17

			S_SUPM_IDLE,
			S_SUPM_FINAL,
			S_BOXM_IDLE,	
			S_BOXM_FINAL,	

			S_DRAW_IDLE,
			S_DRAW_OBJSEG,
			S_AFTDRAW_IDLE,
			S_DRAW_FINAL,

			S_END_GUARD
		};
		//-----------------------------------
		static const int NUM_OF_USIG = 26+1+1;
		enum UISignal{
			U_MPress=0,		 //1
			U_MMove,
			U_MRelease,
			U_BtuOpenImg,
			U_BtuSelectTool, //mouse
			 
			U_BtuLoadBtu,
			U_BtuIniPredBtu,
			U_BtuRePredBtu,
			U_BtuPauseTimeBtu,
			U_BtuConfAllBtu,

			U_ReqRepredictBtu,
			U_BtuActionDoneBtu, 
			U_BtuSaveLabelBtu,
			U_QML_MenuCloseSignal, // 14

			U_AnnoIniListBtu, //15
			U_AnnoStartBtu,
			U_AnnoResetBtu,
			U_AnnoNextBtu,
			U_AnnoPrevBtu,
			U_AnnoSaveBtu,
			U_AnnoALLDoneBtu, //21

			U_DropFileSignal, //22
			U_EnterBoxManiMode,
			U_EnterSupAdjMode,

			U_KeyPress, //24
			U_BoxSpinValueChange,

			U_AddObjPressed ,
			U_END_GUARD
		};

		struct LineDrawContext{
			bool line_isPressed; 
			QPoint line_LeftStartPos;
			QPoint line_MovePos;
			QPoint line_EndPos;
			std::vector<QPoint> line_allLines;
		};

		//------------------------------
#pragma endregion STATIC_REG

	public: 
		static const int MAX_UID=1024;

		typedef Eigen::MatrixXf EMatX;
		typedef Eigen::Matrix4f EMat44;
		typedef Eigen::Matrix3f EMat33;
		typedef Eigen::Vector3f EVec3;
		typedef Eigen::Vector4f EVec4;
		typedef Eigen::RowVector3f ERowVec3;
		typedef Eigen::RowVector4f ERowVec4;

		//----------------------------------
		//State 
		State currentState;	

		int imid; 

		int suggestBOX_LID;

		//----------------------------------
		std::string annosavePath;
		 
		//----------------------------------
		// Control Flag
		bool isReady;

		int showStructure;
		
		//-------------------------------------------------------------------------
		SVMProxy all_svmProxy;
		SVMProxy wall_svmProxy;
		SVMProxy floor_svmProxy;
		SVMProxy obj_svmProxy;

		MyTimer timer;

		std::vector<BoxStateLog> predLog;

		FreqModel freModel;

		//------------------------------
		//BOX data
		QList<QObject*> qmlBoxDatas;
		
		std::vector<BoxDataV2> boxDatas; 

		//------------------------------
		// scene tx data
		EMatX tx_world2fr;
		EMatX tx_fr2world;

		//------------------------------
		// 3d data 
		// world to ndc
		QMatrix4x4 oglMVP;  
		EMatX oglMVP_e;

		EMatX TX_world2img;

		int viewport_tx;
		int viewport_ty;

		int panelWidth;
		int panelHeight;
		
		PointCloudXYZ::Ptr fr_xyzPcd;
		PointCloudXYZ::Ptr fr_nvPcd;
		PointCloudXYZ::Ptr ini_xyzPcd;
		 

		Eigen::Vector3f worldCen;

		//----------------------------------

		PlaneData floor_ini;
		PlaneData floor_fr;

		Eigen::Vector3f cam_fr;

		std::vector<PlaneData> walls_fr;

		OGLPlaneData oglFloor;
		std::vector<OGLPlaneData> oglWallPls;
		//------------------------------ 
		// seg draw

		DrawSegCtx segDrawCtx;
		cv::Mat segDraw_paper;
		cv::Mat segDraw_idmap;
		cv::Mat segDraw_finerseg;

		QImage segDraw_paperQImg;
		QOpenGLTexture* segDraw_paperTex; 

		//----------------------------------
		// 2d data
		QImage inputImg;
		QImage labelImg;

		QImage gtsegCoutourImg;

		cv::Mat gtsegCvMat_16u;
		cv::Mat gtsegCvMat_8uc4; // for rendering 


		cv::Mat roomImg;
		std::vector< std::vector<std::vector<cv::Point> > > all_contours;
		std::vector<int> contourUID;

		// use by screen clipping test
		std::vector< BGPolygon > all2dBoxPolygon; // in box order
		
		std::vector< std::vector< cv::Point > > all2dBoxCVPoly; // in box order

		//std::vector< BGPolygon > allSeg_Polygon; // in box order


		QOpenGLTexture* inputImgTex;
		QOpenGLTexture* labelImgTex;
		QOpenGLTexture* gtsegContourTex; 
		
		//------------------------------ 
		// UI
		QPoint lastPos;
		Qt::MouseButton lastBtu; 
		LineDrawContext lineDrawCtx;

		int focus_box_uid;
		int bkseg_focus_box_idx;

		int lastKeyCode;

		//------------------------------ 
		// box mani
		BxmiCtx bxmiCtx;

		int bxsel_boxselIdx;

		//------------------------------ 
		// support vis and adjust
		cv::Mat supVisMat;
		cv::Mat supVisArrIDMat;
		QImage supVisQImg;
		QOpenGLTexture* supVisTex; 

		bool supVis_showBox;

		std::vector< std::vector<std::vector<cv::Point> > > supVis_roomContour;
		std::vector< int >  supVis_roomCTRUid;

		bool supVis_requireUpdateTex;
		
		int supAdj_focus_box_idx;
				
		SetSupCtx setSupCtx;
	};

	//for debug print
	extern const char* FCMODESTRS[FCStateMag::NUM_OF_STATE];

};