#include "freqModel.h"


FreqModel::FreqModel(void)
{
}


FreqModel::~FreqModel(void)
{
}


void FreqModel::loadModel(const std::string & MODEL_ROOT,const std::string & TARGET_LID_PATH)
{
	// NOTE that the array index start from 0 
	bool verbase=0;

	if( !fileisExisted(TARGET_LID_PATH) ){
		throw "cannot open TARGET_LID FILE";
	}

	std::fstream fin(TARGET_LID_PATH,std::fstream::in);

	targetLID_map.clear();
	targetLID_map.resize(1024,0);

	int tmplid=0;
	int count=0;
	std::string bufstr;
	while (fin>>tmplid)
	{	
		// enable this one
		targetLID_map[tmplid]=1; 
		count=count+1;
		getline(fin,bufstr);
	}

	fin.close();
	//-------------------------------------------------------------------------
	
	char path[2048]={0};

	memset(path,0,sizeof(path));
	sprintf(path,"%s/%s",MODEL_ROOT.c_str() ,"u_sup_v_prob.bin");

	prob_USupV = readMatBinData(path);

	memset(path,0,sizeof(path));
	sprintf(path,"%s/%s",MODEL_ROOT.c_str() ,"sup_by_floor_prob.bin");

	prob_u_floor = readMatBinData(path);
	   
	memset(path,0,sizeof(path));
	sprintf(path,"%s/%s",MODEL_ROOT.c_str() ,"sup_by_wall_prob.bin");

	prob_supByWall = readMatBinData(path);
	 
	memset(path,0,sizeof(path));
	sprintf(path,"%s/%s",MODEL_ROOT.c_str() ,"align_wall_prob.bin");

	prob_u_alignWall = readMatBinData(path);

	memset(path,0,sizeof(path));
	sprintf(path,"%s/%s",MODEL_ROOT.c_str() ,"touch_wall_prob.bin");

	prob_u_touchWall = readMatBinData(path);
	 

	myassert(prob_USupV.rows()>=894,"prob_USupV.rows()>894");
	myassert(prob_USupV.cols()>=894,"prob_USupV.cols()>894");

	myassert(prob_u_floor.cols()==1,"prob_USupV.cols()==1");
	myassert(prob_supByWall.cols()==1,"prob_USupV.cols()==1");
	myassert(prob_u_alignWall.cols()==1,"prob_USupV.cols()==1");
	myassert(prob_u_touchWall.cols()==1,"prob_USupV.cols()==1");

	// [CONVERT INDEXING to 1-N ]
	// drop last row and col -- to align matlab indexing
	// 
	for( int ir=prob_USupV.rows()-1 ; ir >= 1  ; ir-- ){
		for( int ic=prob_USupV.cols()-1 ; ic >= 1  ; ic-- ){
			prob_USupV(ir,ic)=prob_USupV(ir-1,ic-1);
		}
	}

	// 1D shift
	Eigen::MatrixXd shifted_prob_supByWall;
	shifted_prob_supByWall.resize(prob_supByWall.rows()+1,prob_supByWall.cols());

	shifted_prob_supByWall.block(1,0,prob_supByWall.rows(),1)=prob_supByWall;
	prob_supByWall=shifted_prob_supByWall;

	// 1D shift
	Eigen::MatrixXd shifted_prob_u_floor;
	shifted_prob_u_floor.resize(prob_u_floor.rows()+1,prob_u_floor.cols());

	shifted_prob_u_floor.block(1,0,prob_u_floor.rows(),1)=prob_u_floor;
	prob_u_floor=shifted_prob_u_floor;


	// 1D shift
	Eigen::MatrixXd shifted_prob_u_alignWall;
	shifted_prob_u_alignWall.resize(prob_u_alignWall.rows()+1,prob_u_alignWall.cols());

	shifted_prob_u_alignWall.block(1,0,prob_u_alignWall.rows(),1)=prob_u_alignWall;
	prob_u_alignWall=shifted_prob_u_alignWall;


	// 1D shift
	Eigen::MatrixXd shifted_prob_u_touchWall;
	shifted_prob_u_touchWall.resize(prob_u_touchWall.rows()+1,prob_u_touchWall.cols());

	shifted_prob_u_touchWall.block(1,0,prob_u_touchWall.rows(),1)=prob_u_touchWall;
	prob_u_touchWall=shifted_prob_u_touchWall;



	//==========
	// convert to pre-sorted std element
	prob_USupV_std.clear();
	prob_USupV_std.resize(prob_USupV.rows());

	prob_USupV_preSorted.clear();
	prob_USupV_preSorted.resize(prob_USupV.rows());
	 

	for( int ir=1 ; ir < prob_USupV.rows() ; ir++ ){

		prob_USupV_std[ir].resize(prob_USupV.cols());

		for( int ic=1 ; ic < prob_USupV.cols() ; ic++ ){
			 
			prob_USupV_std[ir][ic].prob=prob_USupV(ir,ic);
			prob_USupV_std[ir][ic].u_label=ir;
			prob_USupV_std[ir][ic].v_label=ic;
		}
		std::vector<ProbElement> tmp_sorted=prob_USupV_std[ir];
		// pre-sorting
		std::sort(tmp_sorted.begin(),tmp_sorted.end(),DesComp());

		// reduce size
		prob_USupV_preSorted[ir].clear();
		for( int k=0 ;k <tmp_sorted.size();k++  ){

			// handle inlier selection & exclude no info
			bool isINLIER = targetLID_map[tmp_sorted[k].u_label] > 0 &&  targetLID_map[tmp_sorted[k].v_label] > 0 ;

			// use -1 to indicate no-data condition
			if( tmp_sorted[k].prob>=0 && isINLIER ){
				prob_USupV_preSorted[ir].push_back(tmp_sorted[k]);
			} 
		}
		
	}

	bool g_check=0;

	if (g_check)
	{

		// guard check
		for (int i=0 ; i<prob_u_floor.rows() ; i++)
		{
			for (int j=0 ; j<prob_u_floor.cols() ; j++)
			{
				if( prob_u_floor(i,j) < 0 && prob_u_floor(i,j)  !=-1 ){
					std::cout<<"WIRED STATE prob_u_floor:"<<i<<","<<j<<"  "<<prob_u_floor(i,j)<<std::endl;
					break;
				}
			}
		}

		for (int i=0 ; i<prob_USupV.rows() ; i++)
		{
			for (int j=0 ; j<prob_USupV.cols() ; j++)
			{
				if( prob_USupV(i,j) < 0 && prob_USupV(i,j)!=-1 ){
					std::cout<<"WIRED STATE prob_USupV:"<<i<<","<<j<<"  "<<prob_USupV(i,j)<<std::endl;
					break;
				}
			}
		}

		for (int i=0 ; i<prob_u_touchWall.rows() ; i++)
		{
			for (int j=0 ; j<prob_u_touchWall.cols() ; j++)
			{
				if( prob_u_touchWall(i,j) < 0 && prob_u_touchWall(i,j)!=-1 ){
					std::cout<<"WIRED STATE prob_u_touchWall:"<<i<<","<<j<<"  "<<prob_u_touchWall(i,j)<<std::endl;
					break;
				}
			}
		}


		for (int i=0 ; i<prob_supByWall.rows() ; i++)
		{
			for (int j=0 ; j<prob_supByWall.cols() ; j++)
			{
				if( prob_supByWall(i,j)< 0 && prob_supByWall(i,j)!=-1 ){
					std::cout<<"WIRED STATE prob_supByWall:"<<i<<","<<j<<"  "<<prob_supByWall(i,j)<<std::endl;
					break;
				}
			} 
		}


		for (int i=0 ; i<prob_u_alignWall.rows() ; i++)
		{
			for (int j=0 ; j<prob_u_alignWall.cols() ; j++)
			{
				if( prob_u_alignWall(i,j) ){
					std::cout<<"WIRED STATE prob_u_alignWall:"<<i<<","<<j<<"  "<<prob_u_alignWall(i,j)<<std::endl;
					break;
				}
			}
		}
	}

		

	if( verbase ){
		std::cout<<"prob_USupV(157,119)="<<prob_USupV(157,119)<<std::endl;
		std::cout<<"prob_u_floor(158)="<<prob_u_floor(158)<<std::endl;
		std::cout<<"prob_USupV_preSorted(157,0)="<<prob_USupV_preSorted[157][0].prob<<std::endl; 
		std::cout<<"# of selected lid = "<<count<<std::endl;
	}

}
