#pragma once


#include <Eigen/Eigen> 
#include <vector>
#include <fstream> 
#include <iostream> 
#include <svm.h> 
#include <string> 
#include <algorithm> 
#include "common_tool.h"


struct ProbElement
{
public:
	int u_label;
	int v_label; 
	double prob;
};

class FreqModel
{
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW


	FreqModel(void);
	~FreqModel(void);

	void loadModel(const std::string & MODEL_ROOT , const std::string & TARGET_LID_PATH);

	Eigen::MatrixXd prob_USupV;

	Eigen::MatrixXd prob_u_floor;
	Eigen::MatrixXd prob_supByWall;

	Eigen::MatrixXd prob_u_alignWall;
	Eigen::MatrixXd prob_u_touchWall;

	std::vector< std::vector<ProbElement> > prob_USupV_std;
	std::vector< std::vector<ProbElement> > prob_USupV_preSorted;

	std::vector< int > targetLID_map;

};


class DesComp {
public:
	inline
	bool operator() (int i,int j) { return (i>j);}
	
	inline
	bool operator() (double i,double j) { return (i>j);}
	
	inline
	bool operator() (ProbElement const & I, ProbElement const & J){
		return I.prob > J.prob ;
	}
};
 