#pragma once

#include <Eigen/Eigen>
#include <vector>
#include <QOpenGLContext>

#include <iostream>
#include <fstream>
#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#include <boost/serialization/version.hpp>
#include <boost/serialization/vector.hpp>    
 

#pragma warning( disable : 4482 )

struct NaiveBox{
public:
	typedef float Vertex;
	static const int LNAME_LEN = 2048;

	// Use column major order
	Vertex box3d_world[24];
	Vertex box3d_fr[24];
	Vertex box3d_aabb[24];

	Vertex obb_fr2aabb[3*3]; 
	Vertex obb_aabb2fr[3*3]; 

	int uid;
	int supType;
	int supLIDX; 
	int labelLid;

	char labelName[LNAME_LEN];

	int supByWallLidx;
	int closetWall_lidx;
}; 

struct NaiveAnnoBox{

private:

	enum SUPPORT_TYPE{
		INI=0,
		BOX=1,
		WALL=2,
		FLOOR=4,
		FLOAT=8
	};
public:
	// Use column major order
	typedef float Vertex;

	static const int MAX_MENU_NUM = 6;
	
	Vertex box3d_world[24];
	Vertex box3d_fr[24];
	Vertex box3d_aabb[24];

	Vertex obb_fr2aabb[3*3]; 
	Vertex obb_aabb2fr[3*3]; 

	int pred_menu[MAX_MENU_NUM];

	int uid;
	int supType;
	int supLIDX; 
	int labelLid; 

	int supByWallLidx;
	int closetWall_lidx;

	int box_sup_modified; // user modify structure
	int box_geo_modified; // user modify geometry 

	inline
		bool isSupINI()const{
			return supType ==SUPPORT_TYPE::INI;
	}
	inline
		bool isSupByGround()const{
			return supType & SUPPORT_TYPE::FLOOR;
	}
	inline
		bool isSupByWall()const{
			return supType & SUPPORT_TYPE::WALL;
	}
	inline
		bool isSupByObj()const{
			return supType & SUPPORT_TYPE::BOX;
	}
	inline
		bool isFloat()const{
			return supType & SUPPORT_TYPE::FLOAT;
	}
	inline
		void setSupToIni(){
			supType=supType| SUPPORT_TYPE::INI;
	}
	inline
		void setSupToWall(){
			supType=supType|SUPPORT_TYPE::WALL;
	}
	inline
		void setSupToFloor(){
			supType=supType|SUPPORT_TYPE::FLOOR;
	}
	inline
		void setSupToObj(){
			supType=supType|SUPPORT_TYPE::BOX;
	}
	inline
		void setSupToFloat(){
			supType=supType|SUPPORT_TYPE::FLOAT;
	}
}; 

class OGLPlaneData
{
public:
	typedef float Vertex;
	// 2 triangle 
	static const int NUM_TRI=2;
	// use 4 vertices
	static const int NUM_TVEX=4;
	static const int NUM_TRI_IDX = NUM_TRI*3;

	static const int NUM_LVEX=4;
	static const int NUM_LVEX_IDX=4;


	// 2x3x3=18
	Vertex planeTriangles[NUM_TVEX*3];

	// 2x3
	GLubyte planeIndex[NUM_TRI_IDX];
	
	Vertex planeNV[NUM_LVEX*3];
	
	GLubyte planeNVIdx[NUM_LVEX_IDX];

};
 
class PlaneData
{
public: 
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW

	static const int CSS_VNUM = 3;

	Eigen::VectorXf model;  
	Eigen::Vector3f pts[CSS_VNUM];
};

struct NaivePlane
{
public:
	typedef float Vertex;

	//**the normal dir of the plane should be checked**

	// ax+by+cz+d=0
	Vertex model[4]; 

	// point on plane
	Vertex pts[PlaneData::CSS_VNUM*3];
};

struct NaiveSceneCTX
{
public:
	// Use column major order

	typedef float Vertex;
	
	// TX data
	Vertex tx_world2fr[4*4]; 
	Vertex tx_fr2world[4*4]; 
	 
};

struct BoxStateLog
{
public:
	friend class boost::serialization::access;    

	enum BSTATE{
		BSTATE_UNK=0,
		BSTATE_SWITCHED,
		BSTATE_LOCKED,
		BSTATE_EDITED,
		BSTATE_CONF_ALL
	};
	
	std::vector<int> boxState;
	std::vector<int> predRes;
	std::vector<int> userSelectMenuIdx;

	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{ 
		//if(version > 0)
		//	ar & driver_name;
		ar & boxState;
		ar & predRes;
		ar & userSelectMenuIdx;
	}
};
BOOST_CLASS_VERSION(BoxStateLog, 1)

class AnnoRes{

public:
	friend class boost::serialization::access;    

	std::vector<BoxStateLog> action_log;
	
	double total_time;
	double time_support;
	double time_box;
	double time_label;
	
	int sce_id;

	//-----------------------------

	AnnoRes();
	 
	void printInfo(); 

	//-----------------------------

	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{ 
		
		ar & action_log;
		ar & total_time;

		if(version ==2){
			ar & time_support;
			ar & time_box;
			ar & time_label;
		}


		ar & sce_id;
	}


};

BOOST_CLASS_VERSION(AnnoRes, 2)






























