#include "pcabox.h"



void pca_box( const int TID , cv::Mat const & indImg , PointCloudXYZ::Ptr f_xyzPcd , PointCloudXYZ::Ptr f_nvPcd , Eigen::MatrixXf const& fr2worldMat, BoxDataV2 & resBox )
{
	bool verbase=1;

	// extract target points  
	std::vector<int> tarIds;
	findCVIMGIdx(TID,indImg,tarIds);

	if(verbase){
		LOG_MSG_L0("target pts #=%d\n",tarIds.size());
	}

	PointCloudXYZ::Ptr tarPcd(new PointCloudXYZ());
	// filter-out pts via 'map'
	pcl::copyPointCloud(*f_xyzPcd , tarIds , *tarPcd); 

	// filter-out pts via NN filter
	std::vector<int> filtedIdx; 
	knnFilter(tarPcd,&tarIds,filtedIdx,1.5,80);

	//[FIND OBB TX]
	Eigen::Vector3f axis0,axis1,axis2; 

	tarPcd->clear();

	if( filtedIdx.size() < 20000  ){
		
		pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);

	}else{
		// too many points
		std::vector<int> dsIdx_tmp(filtedIdx.begin(),filtedIdx.end()); 
		std::random_shuffle(dsIdx_tmp.begin(),dsIdx_tmp.end());

		std::vector<int> dsIdx(dsIdx_tmp.begin(),dsIdx_tmp.begin()+20000-1 ); 
		pcl::copyPointCloud(*f_xyzPcd , dsIdx , *tarPcd);

	}

	//-------------------------------------------------------------------------
	// measure box orientation

	// Nx3
	Eigen::MatrixXf mat=tarPcd->getMatrixXfMap(3,4,0);

	// -> 3xN
	mat.transposeInPlace();

	Eigen::MatrixXf centered = mat.rowwise() - mat.colwise().mean();
	Eigen::MatrixXf cov = centered.adjoint() * centered;

	//and then perform the eigen-decomposition:
	Eigen::SelfAdjointEigenSolver<Eigen::MatrixXf> eig(cov);

	// From eig, you have access to the sorted (increasing order) 
	// eigenvalues (eig.eigenvalues()) and respective eigenvectors (eig.eigenvectors()). For instance, eig.eigenvectors().rightCols(N) 
	// gives you the best N-dimension basis. 
	Eigen::MatrixXf evec = eig.eigenvectors();
	Eigen::MatrixXf eval = eig.eigenvalues();

	if( verbase ){

		std::cout<<"eigen_vector=\n"<<evec.rightCols(3)<<"\n\n";
		std::cout<<"eigen_val=\n"<<eval.bottomRows(3)<<"\n\n";

	}

	const int NUM_COLS=evec.cols();

	//-------------------------------------------------------------------------
	// tx points by OBB

	// re-set pts
	pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);

	Eigen::Matrix3f obbmat;
	obbmat.col(0) = evec.rightCols(NUM_COLS-1).normalized();
	obbmat.col(1) = evec.rightCols(NUM_COLS-2).normalized();
	obbmat.col(2) = (obbmat.col(0).cross(obbmat.col(1))).normalized();

	extr_obb(obbmat,tarPcd,fr2worldMat,resBox);

	resBox.setUID(TID);
}


void plane_by_pca( const int TID , cv::Mat const & indImg,
	PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0,
	PlaneData & res )
{
	//*
	bool verbase=0;

	// [FR SPACE]
	std::vector<int> tarIdx;
	findCVIMGIdx(TID,indImg,tarIdx);

	PointCloudXYZ::Ptr tarPcd(new PointCloudXYZ());
	// filter-out pts via 'map'
	pcl::copyPointCloud(*f_xyzPcd , tarIdx , *tarPcd); 
	

	std::vector<int> filtedIdx = tarIdx;

	// update pts
	tarPcd->clear();
	pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);

	
	//=====================================================================================
	// measure box orientation

	// Nx3
	Eigen::MatrixXf mat=tarPcd->getMatrixXfMap(3,4,0);

	// -> 3xN
	mat.transposeInPlace();

	Eigen::MatrixXf centered = mat.rowwise() - mat.colwise().mean();
	Eigen::MatrixXf cov = centered.adjoint() * centered;

	//and then perform the eigen-decomposition:
	Eigen::SelfAdjointEigenSolver<Eigen::MatrixXf> eig(cov);

	// From eig, you have access to the sorted (increasing order) 
	// eigenvalues (eig.eigenvalues()) and respective eigenvectors 
	// (eig.eigenvectors()). For instance, eig.eigenvectors().rightCols(N) 
	//  gives you the best N-dimension basis. 
	Eigen::MatrixXf evec = eig.eigenvectors();
	Eigen::MatrixXf eval = eig.eigenvalues();

	const int NUM_COLS=evec.cols();

	Eigen::Vector3f pl_d1 = evec.rightCols(NUM_COLS-1).normalized();
	Eigen::Vector3f fr_nv ;
	fr_nv<<0,1,0;

	Eigen::Vector3f pl_nv = pl_d1.cross(fr_nv);

	//=====================================================================================
	

	// Nx3
	Eigen::MatrixXf mat3d=tarPcd->getMatrixXfMap(3,4,0);

	Eigen::MatrixXf coef_d=-1*mat3d*pl_nv;

	float avg_d = coef_d.array().mean();

	//=====================================================================================
	//optimal coeff
	res.model.resize(4,1);
	res.model<< pl_nv , avg_d;


	Eigen::Vector3f pOnPL = findAP0(res.model);

	std::vector<int> const & VIDX = filtedIdx;

	// get 3 inlier 	
	for (int i=0; i < PlaneData::CSS_VNUM ; i++)
	{
		int idx=VIDX[i];

		res.pts[i]=tarPcd->points[idx].getVector3fMap(); 


		if( !ptsIsOnPlane(res.pts[i] , res.model )){
			// prj point 
			res.pts[i]=prjPts2Plane( res.pts[i] ,res.model, pOnPL );
		}

		// check all pts is on the plane
		myassert( ptsIsOnPlane(res.pts[i] , res.model ),"pts not on plane");

	}

	if( sameDirC0( res.model.head(3), res.pts[0]-REF_CAM0 ) ){
		res.model=-1*res.model;
	}

	//*/

}


void plane_by_lsq( const int TID , cv::Mat const & indImg,
	PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0,
	PlaneData & res )
{
	//*
	bool verbase=0;

	// [FR SPACE]
	std::vector<int> tarIdx;
	findCVIMGIdx(TID,indImg,tarIdx);

	PointCloudXYZ::Ptr tarPcd(new PointCloudXYZ());
	// filter-out pts via 'map'
	pcl::copyPointCloud(*f_xyzPcd , tarIdx , *tarPcd); 

	// project to XZ plane
	for( int i=0 ; i < tarPcd->size() ; i++ ){
		tarPcd->points[i].y=0;
	}

	// filter-out pts via NN filter
	std::vector<int> filtedIdx; 
	knnFilter(tarPcd,&tarIdx,filtedIdx,3,30);

	if( verbase ){
		std::cout<<"knn filtering ratio="<<filtedIdx.size()*1.0/tarIdx.size()<<'\n';
	}

	// update pts
	tarPcd->clear();
	pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);

	
	//=====================================================================================
	// measure box orientation

	// Nx3
	Eigen::MatrixXf mat=tarPcd->getMatrixXfMap(3,4,0);

	// -> 3xN
	mat.transposeInPlace();

	Eigen::MatrixXf centered = mat.rowwise() - mat.colwise().mean();
	Eigen::MatrixXf cov = centered.adjoint() * centered;

	//and then perform the eigen-decomposition:
	Eigen::SelfAdjointEigenSolver<Eigen::MatrixXf> eig(cov);

	// From eig, you have access to the sorted (increasing order) 
	// eigenvalues (eig.eigenvalues()) and respective eigenvectors 
	// (eig.eigenvectors()). For instance, eig.eigenvectors().rightCols(N) 
	//  gives you the best N-dimension basis. 
	Eigen::MatrixXf evec = eig.eigenvectors();
	Eigen::MatrixXf eval = eig.eigenvalues();

	const int NUM_COLS=evec.cols();

	Eigen::Vector3f pl_d1 = evec.rightCols(NUM_COLS-1).normalized();
	Eigen::Vector3f fr_nv ;
	fr_nv<<0,1,0;

	Eigen::Vector3f pl_nv = pl_d1.cross(fr_nv);

	//=====================================================================================
	

	// Nx4
	Eigen::MatrixXf mat3d=tarPcd->getMatrixXfMap(3,4,0);

	Eigen::MatrixXf coef_d=mat3d*pl_nv;

	float avg_d = coef_d.array().mean(); 

	//=====================================================================================
	//optimal coeff
	res.model.resize(4,1);
	res.model<< pl_nv , avg_d;


	Eigen::Vector3f pOnPL = findAP0(res.model);

	std::vector<int> const & VIDX = filtedIdx;

	// get 3 inlier 	
	for (int i=0; i < PlaneData::CSS_VNUM ; i++)
	{
		int idx=VIDX[i];

		res.pts[i]=tarPcd->points[idx].getVector3fMap(); 


		if( !ptsIsOnPlane(res.pts[i] , res.model )){
			// prj point 
			res.pts[i]=prjPts2Plane( res.pts[i] ,res.model, pOnPL );
		}

		// check all pts is on the plane
		myassert( ptsIsOnPlane(res.pts[i] , res.model ),"pts not on plane");

	}

	if( sameDirC0( res.model.head(3), res.pts[0]-REF_CAM0 ) ){
		res.model=-1*res.model;
	}

	//*/

}
