#include "stdafx.h"
#include "pcd_tools.h"




void copyViaIndex(std::vector<int> const & VIN , std::vector<int> const& FLAG, std::vector<int> &vout ){

	//if( &VIN == &vout ){

	//input = output
	std::vector<int> temp;
	for( int k=0 ; k < FLAG.size() ; k++ ){
		if( FLAG[k] ){
			temp.push_back(VIN[k]);
		}
	}
	vout=temp;

}

void knnFilter( pcl::PointCloud<pcl::PointXYZ>::ConstPtr PCD_IN , std::vector<int> * pcd_index, 
			   std::vector<int> & inlIdx , double medianRatio /*=3*/ ,int const NUM_K/*=30*/ )
{

	// check 
	myassert(&inlIdx != pcd_index);

	// K nearest neighbor search
	int K = NUM_K;

	pcl::KdTreeFLANN<pcl::PointXYZ> kdtree;
	kdtree.setInputCloud(PCD_IN);

	std::vector<int> pointIdxNKNSearch(K);
	std::vector<float> pointNKNSquaredDistance(K);
	std::vector<float> NNDist(PCD_IN->size());

	int res=0;
	for(int i=0 ; i < PCD_IN->size() ; i++ ){
		res=kdtree.nearestKSearch(PCD_IN->points[i], K, pointIdxNKNSearch, pointNKNSquaredDistance);
		if( res ){
			NNDist[i]=0;
			for( int k=0; k<pointNKNSquaredDistance.size() ;k++ ){
				NNDist[i]=NNDist[i]+pointNKNSquaredDistance[k];
			}
			NNDist[i]=NNDist[i]/pointNKNSquaredDistance.size();
		}else{
			NNDist[i]=std::numeric_limits<float>::infinity();

		}
	}
	// median
	std::vector<float> sortedNNDist(NNDist);
	std::sort(sortedNNDist.begin(), sortedNNDist.end());

	float threshold = sortedNNDist[ static_cast<int>(sortedNNDist.size()/2) ] *2;

	myassert( _finite(threshold) && ~_isnan(threshold) ,"knn filter -- wired median value");

	// extract inlier
	inlIdx.clear();

	if(pcd_index!=NULL){
		//also copy index
		for( int i=0; i<NNDist.size() ; i++ ){
			if( NNDist[i] < threshold ){
				inlIdx.push_back( (*pcd_index)[i] );
			}
		}

	}else{
		//simple mode 
		for( int i=0; i<NNDist.size() ; i++ ){
			if( NNDist[i] < threshold ){
				inlIdx.push_back(i);
			}
		}

	}


	//std::cout<<"filted ratio="<<inlIdx.size()<<","<<NNDist.size()<<"\t"<<inlIdx.size()*1.0/NNDist.size()<<'\n';
}

void exbox(const int TID  , cv::Mat const &  indImg ,
		   PointCloudXYZ::Ptr f_xyzPcd , PointCloudXYZ::Ptr f_nvPcd ,
		   Eigen::MatrixXf const& fr2worldMat,
		   BoxDataV2 & resBox   )
{
	// world_pts * W_2_FR --> FR pts
	// FR pts * FR_2_W --> world_pts

	//NOTE that we must use Ptr to access point_cloud which guarantee the action of shared_ptr is right.

	bool verbase=false;

	// extract target points  
	std::vector<int> tarIds;
	findCVIMGIdx(TID,indImg,tarIds);

	if(verbase){

		LOG_MSG_L0("target pts #=%d\n",tarIds.size());
	}

	PointCloudXYZ::Ptr tarPcd(new PointCloudXYZ());
	// filter-out pts via 'map'
	pcl::copyPointCloud(*f_xyzPcd , tarIds , *tarPcd); 

	// filter-out pts via NN filter
	std::vector<int> filtedIdx; 
	knnFilter(tarPcd,&tarIds,filtedIdx);

	//[FIND OBB TX]
	Eigen::Vector3f axis0,axis1,axis2; 

	// check SIZE issue
	if( tarIds.size() > 300 ){

		// many pts --> try to filting them

		// update pts
		tarPcd->clear();
		pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);

		// filter-out top face by NV
		// update NV
		// [filtedIdx]
		PointCloudXYZ::Ptr tarNvPcd(new PointCloudXYZ());
		pcl::copyPointCloud(*f_nvPcd,filtedIdx,*tarNvPcd);

		//filter by NV
		// N*3 
		//assert norm of NV ==1
		Eigen::MatrixXf nvMap=tarNvPcd->getMatrixXfMap();
		Eigen::MatrixXf floorNV(1,4);
		floorNV(0,0)=0;
		floorNV(0,1)=1;
		floorNV(0,2)=0;
		floorNV(0,3)=0;

		//dot product -> 1x4 * 4xN 
		//nvMap = nvMap.coeff() / nvMap.rowwise().norm();
		if( verbase ){
			std::cout<<floorNV.rows()<<','<<floorNV.cols()<<'\n';
			std::cout<<nvMap.rows()<<','<<nvMap.cols()<<'\n';
		}

		//[EX Top] exclude the top face
		nvMap=floorNV*nvMap;
		Eigen::ArrayXf theta=(nvMap.transpose()).array();

		//filter by NV
		// 360deg = 2PI
		float nvTh = 35.0*M_PI/180.0;
		Eigen::ArrayXi nvInlier = (theta.abs() > nvTh).cast<int>();

		// inlier Index 
		std::vector<int> nvf_index;
		for(int i =0 ; i< nvInlier.size() ; i++){ 
			if( nvInlier(i) ){
				nvf_index.push_back(filtedIdx[i]);
			}
		}

		// [CHECK we have enough pts]
		if( nvf_index.size() < 150 ){
			nvf_index=filtedIdx;
		}

		// re-extract pts ->tarPcd
		// [nvf_index]
		pcl::copyPointCloud(*f_xyzPcd , nvf_index , *tarPcd);

		// project to XZ plane
		for(int k=0; k<tarPcd->size() ; k++){
			tarPcd->points[k].y=0;
		}

		// perform 2d RANSAC  
		pcl::SampleConsensusModelLine< pcl::PointXYZ >::Ptr 
			model_line( new  pcl::SampleConsensusModelLine< pcl::PointXYZ  >( tarPcd ));

		pcl::ProgressiveSampleConsensus<pcl::PointXYZ> ransac(model_line);

		ransac.setDistanceThreshold(0.05);
		ransac.computeModel();
		std::vector<int> ransacInlierLIdx;
		ransac.getInliers(ransacInlierLIdx); 

		// get real idx
		copyViaIndex(nvf_index,ransacInlierLIdx,nvf_index);

		// get line coeff (3d)
		/*
		point_on_line.x : the X coordinate of a point on the line
		point_on_line.y : the Y coordinate of a point on the line
		point_on_line.z : the Z coordinate of a point on the line
		line_direction.x : the X coordinate of a line's direction
		line_direction.y : the Y coordinate of a line's direction
		line_direction.z : the Z coordinate of a line's direction
		*/

		Eigen::VectorXf model_coeff;
		ransac.getModelCoefficients(model_coeff);

		//std::cout<<"model\n"<<model_coeff<<std::endl;

		//-------------------------------------------------------------------------
		axis0<<0,1,0; 
		axis1<<model_coeff(0+3) , model_coeff(0+4) , model_coeff(0+5);

		//normal should point out
		if (sameDirC0(axis1,model_coeff.head<3>())  ){
			axis1=-axis1;
		}

		axis2 = axis0.cross(axis1);
		axis2.normalize();

	}else{
		//direct measure OBB
		axis1<<1,0,0; 
		axis0<<0,1,0; 
		axis2<<0,0,1; 
	}


	//-------------------------------------------------------------------------
	// OBB 

	// reset 3d target pcd 
	pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);

	// tx points by OBB
	Eigen::Matrix3f obbmat;
	obbmat.col(0)=axis1;
	obbmat.col(1)=axis0;
	obbmat.col(2)=axis2;

	extr_obb(obbmat,tarPcd,fr2worldMat,resBox);

	resBox.setUID(TID);
}

bool sameDirC0( Eigen::Vector3f v1 , Eigen::Vector3f v2 )
{
	//check direction with camera ray

	v2.normalize();
	v1.normalize();

	if( v1.dot(v2) < 0 ){
		// theta > 90 = good (diff direction)

		return false;
	}else{

		return true;
	}

}

Eigen::MatrixX3f getBoxByMaxMin( Eigen::Vector3f bmax , Eigen::Vector3f bmin )
{
	myassert(bmin.size()==3 && bmax.size()==3,"getBoxByMaxMin");

	Eigen::MatrixX3f p8;
	p8.resize(8,3);
	p8 << bmax.x() , bmax.y() , bmax.z() ,
		bmax.x() , bmax.y() , bmin.z(),
		bmin.x() , bmax.y() , bmin.z(),
		bmin.x() , bmax.y() , bmax.z() , //4

		bmax.x() , bmin.y() , bmax.z() , //5
		bmax.x() , bmin.y() , bmin.z() , //6
		bmin.x() , bmin.y() , bmin.z() , //7
		bmin.x() , bmin.y() , bmax.z() ; //8

	return p8;
}
Eigen::MatrixX3f getBoxByMaxMin_row( Eigen::RowVector3f bmax , Eigen::RowVector3f bmin )
{
	myassert(bmin.size()==3 && bmax.size()==3,"getBoxByMaxMin");

	Eigen::MatrixX3f p8;
	p8.resize(8,3);
	p8 << bmax.x() , bmax.y() , bmax.z() ,
		bmax.x() , bmax.y() , bmin.z(),
		bmin.x() , bmax.y() , bmin.z(),
		bmin.x() , bmax.y() , bmax.z() , //4

		bmax.x() , bmin.y() , bmax.z() , //5
		bmax.x() , bmin.y() , bmin.z() , //6
		bmin.x() , bmin.y() , bmin.z() , //7
		bmin.x() , bmin.y() , bmax.z() ; //8

	return p8;
}
Eigen::MatrixXf cenTx( Eigen::RowVector3f cen ,Eigen::MatrixXf const & TX , Eigen::MatrixXf pts )
{
	// pts is Nx3 

	myassert(pts.cols()==3,"pts should be Nx3");


	//Center TX
	pts = pts.rowwise() - cen; 
	//3xN
	pts.transposeInPlace();
	pts=TX * pts;	
	//Nx3
	pts.transposeInPlace();
	pts = pts.rowwise() + cen; 

	return pts;
}

Eigen::MatrixXf cenTx44( Eigen::RowVector3f cen_in ,Eigen::MatrixXf const & TX , Eigen::MatrixXf pts )
{
	// pts is Nx4 

	if(pts.cols()==4){

		Eigen::RowVector4f cen;
		cen<<cen_in,0;

		//Center TX
		pts = pts.rowwise() - cen; 
		//4xN
		pts.transposeInPlace();
		pts=TX * pts;	

		//Nx4
		pts.transposeInPlace();

		pts.col(0) = pts.col(0).array() / pts.col(3).array();
		pts.col(1) = pts.col(1).array() / pts.col(3).array();
		pts.col(2) = pts.col(2).array() / pts.col(3).array();

		pts = pts.rowwise() + cen;  
		return pts;

	}else if(pts.cols()==3){
		// Nx3 

		//Center TX
		pts = pts.rowwise() - cen_in; 

		Eigen::MatrixXf pts4;
		pts4.resize(8,4);
		pts4.block(0,0,8,3)=pts;

		pts4.col(4-1).setOnes();

		// 4xN
		pts4.transposeInPlace();
		pts4=TX * pts4;	 
		//Nx4
		pts4.transposeInPlace();

		pts4.col(0) = pts4.col(0).array() / pts4.col(3).array();
		pts4.col(1) = pts4.col(1).array() / pts4.col(3).array();
		pts4.col(2) = pts4.col(2).array() / pts4.col(3).array();

		pts=pts4.block(0,0,8,3);

		//Center TX
		pts = pts.rowwise() + cen_in;  


		////3xN
		//pts.transposeInPlace();

		//pts=TX.block(0,0,3,3) * pts;	

		////Nx3
		//pts.transposeInPlace();

		//Eigen::RowVector3f v3f = TX.block(1-1,4-1,3,1).transpose();
		////add translation
		//pts = pts.rowwise() + v3f;   

		return pts;

	}else{
		throw "Invalid pts size";
	}



}

float planeRANSAC( const int TID , cv::Mat const & indImg,
				 PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0,
				 PlaneData & res )
{
	// [WORLD SPACE]
	std::vector<int> tarIdx;
	findCVIMGIdx(TID,indImg,tarIdx);

	PointCloudXYZ::Ptr tarPcd(new PointCloudXYZ());
	// filter-out pts via 'map'
	pcl::copyPointCloud(*f_xyzPcd , tarIdx , *tarPcd); 

	// filter-out pts via NN filter
	
	std::vector<int> filtedIdx; 
	//knnFilter(tarPcd,&tarIdx,filtedIdx);

	filtedIdx=tarIdx;

	// update pts
	tarPcd->clear();
	pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);


	pcl::SampleConsensusModelPlane< pcl::PointXYZ >::Ptr 
		model_plane( new  pcl::SampleConsensusModelPlane< pcl::PointXYZ  >( tarPcd ));

	pcl::ProgressiveSampleConsensus<pcl::PointXYZ> ransac(model_plane);
	//pcl::MaximumLikelihoodSampleConsensus<pcl::PointXYZ> ransac(model_plane);

	ransac.setDistanceThreshold(0.01);
	//ransac.setEMIterations(500);
	//ransac.setDistanceThreshold(0.001);
	//ransac.setMaxIterations(1000);
	ransac.computeModel();
	std::vector<int> ransacInlierLIdx;
	ransac.getInliers(ransacInlierLIdx); 

	float inl_ratio = ransacInlierLIdx.size()*1.0/filtedIdx.size();
	std::cout<<"RANSAC inl ratio:"<<inl_ratio<<'\n';

	//get fr index
	copyViaIndex(filtedIdx,ransacInlierLIdx,filtedIdx);

	// its Hessian Normal form: [normal_x normal_y normal_z d]
	// ax+by+cz+d=0
	Eigen::VectorXf model_coeff;
	ransac.getModelCoefficients(model_coeff);

	Eigen::VectorXf resultCoeff;
	//optimal coeff
	model_plane->optimizeModelCoefficients(ransacInlierLIdx,model_coeff,resultCoeff);

	/*
	std::cout<<"Model\n"
	<<"ax+by+cz+d=0\n"<<resultCoeff<<std::endl
	<<"renormalized :\n";
	*/

	// normalize
	resultCoeff = resultCoeff.array() / resultCoeff.head(3).norm();

	res.model=resultCoeff;

	Eigen::Vector3f pOnPL = findAP0(res.model);

	// get 3 inlier 	
	for (int i=0; i < PlaneData::CSS_VNUM ; i++)
	{
		int idx=ransacInlierLIdx[i];

		res.pts[i]=tarPcd->points[idx].getVector3fMap(); 


		if( !ptsIsOnPlane(res.pts[i] , res.model )){
			// prj point 
			res.pts[i]=prjPts2Plane( res.pts[i] ,res.model, pOnPL );
		}

		// check all pts is on the plane
		myassert( ptsIsOnPlane(res.pts[i] , res.model ),"pts not on plane");

	}

	if( sameDirC0( res.model.head(3), res.pts[0]-REF_CAM0 ) ){
		res.model=-1*res.model;
	}


	return inl_ratio;
}

void planeRANSAC_floor( const int TID , cv::Mat const & indImg,
				 PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0,
				 PlaneData & res )
{
	// [WORLD SPACE]
	std::vector<int> tarIdx;
	findCVIMGIdx(TID,indImg,tarIdx);

	PointCloudXYZ::Ptr tarPcd(new PointCloudXYZ());
	// filter-out pts via 'map'
	pcl::copyPointCloud(*f_xyzPcd , tarIdx , *tarPcd); 


	bool normalMode = tarPcd->size() > 2000;


	//pcl::ProgressiveSampleConsensus<pcl::PointXYZ> ransac(model_plane);
	//pcl::MaximumLikelihoodSampleConsensus<pcl::PointXYZ> ransac(model_plane);

	//ransac.setDistanceThreshold(0.01);
	//ransac.setEMIterations(500);
	//ransac.setDistanceThreshold(0.001);
	//ransac.setMaxIterations(1000);
	//ransac.computeModel();
	//=====================================================================================
	// M-SAC

	// its Hessian Normal form: [normal_x normal_y normal_z d]
	// ax+by+cz+d=0
	Eigen::VectorXf model_coeff;
	Eigen::VectorXf resultCoeff;

	std::vector<int> ransacInlierLIdx;

	if (normalMode)
	{
		//normal mode	
		// filter-out pts via NN filter
		std::vector<int> filtedIdx; 
		knnFilter(tarPcd,&tarIdx,filtedIdx);

		// update pts
		tarPcd->clear();
		pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd); 
		pcl::SampleConsensusModelPlane< pcl::PointXYZ >::Ptr 
			model_plane( new  pcl::SampleConsensusModelPlane< pcl::PointXYZ  >( tarPcd ));

		pcl::MEstimatorSampleConsensus<pcl::PointXYZ> ransac(model_plane);
		ransac.setDistanceThreshold(0.03);
		ransac.computeModel();

		//post process

		ransac.getInliers(ransacInlierLIdx); 
		std::cout<<"RANSAC inl ratio:"<<ransacInlierLIdx.size()*1.0/tarPcd->size()<<'\n';

		//get fr index
		//copyViaIndex(filtedIdx,ransacInlierLIdx,filtedIdx);
		ransac.getModelCoefficients(model_coeff); 
		//optimal coeff
		model_plane->optimizeModelCoefficients(ransacInlierLIdx,model_coeff,resultCoeff);


	}else{
		// few points mode
		pcl::SampleConsensusModelPlane< pcl::PointXYZ >::Ptr 
			model_plane( new  pcl::SampleConsensusModelPlane< pcl::PointXYZ  >( tarPcd ));
		pcl::MEstimatorSampleConsensus<pcl::PointXYZ> ransac(model_plane);
		ransac.setDistanceThreshold(0.01);
		ransac.computeModel();

		//post process
		
		ransac.getInliers(ransacInlierLIdx); 
		std::cout<<"RANSAC inl ratio:"<<ransacInlierLIdx.size()*1.0/tarPcd->size()<<'\n';
		
		//get fr index
		//copyViaIndex(tarIdx,ransacInlierLIdx,tarIdx);

		ransac.getModelCoefficients(model_coeff); 
		//optimal coeff
		model_plane->optimizeModelCoefficients(ransacInlierLIdx,model_coeff,resultCoeff);

	}


	//=====================================================================================
	  

	/*
	std::cout<<"Model\n"
	<<"ax+by+cz+d=0\n"<<resultCoeff<<std::endl
	<<"renormalized :\n";
	*/

	// normalize
	resultCoeff = resultCoeff.array() / resultCoeff.head(3).norm();

	res.model=resultCoeff;

	Eigen::Vector3f pOnPL = findAP0(res.model);

	// get 3 inlier 	
	for (int i=0; i < PlaneData::CSS_VNUM ; i++)
	{
		int idx=ransacInlierLIdx[i];

		res.pts[i]=tarPcd->points[idx].getVector3fMap(); 
		 
		if( !ptsIsOnPlane(res.pts[i] , res.model )){
			// prj point 
			res.pts[i]=prjPts2Plane( res.pts[i] ,res.model, pOnPL );
		}

		// check all pts is on the plane
		myassert( ptsIsOnPlane(res.pts[i] , res.model ),"pts not on plane - planeRANSAC_floor");

	}

	if( sameDirC0( res.model.head(3), res.pts[0]-REF_CAM0 ) ){
		res.model=-1*res.model;
	}

}

void planeRANSAC_byXZLine( const int TID , cv::Mat const & indImg,
						  PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0,
						  PlaneData & res )
{

	bool verbase=0;

	// [FR SPACE]
	std::vector<int> tarIdx;
	findCVIMGIdx(TID,indImg,tarIdx);

	PointCloudXYZ::Ptr tarPcd(new PointCloudXYZ());
	// filter-out pts via 'map'
	pcl::copyPointCloud(*f_xyzPcd , tarIdx , *tarPcd); 

	// project to XZ plane
	for( int i=0 ; i < tarPcd->size() ; i++ ){
		tarPcd->points[i].y=0;
	}


	// filter-out pts via NN filter
	std::vector<int> filtedIdx; 
	knnFilter(tarPcd,&tarIdx,filtedIdx,3,30);

	if( verbase ){
		std::cout<<"knn filting ratio="<<filtedIdx.size()*1.0/tarIdx.size()<<'\n';
	}

	// update pts
	tarPcd->clear();
	pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);


	pcl::SampleConsensusModelLine< pcl::PointXYZ >::Ptr 
		model_line( new  pcl::SampleConsensusModelLine< pcl::PointXYZ  >( tarPcd ));

	pcl::ProgressiveSampleConsensus<pcl::PointXYZ> ransac(model_line);
	//pcl::MaximumLikelihoodSampleConsensus<pcl::PointXYZ> ransac(model_plane);

	ransac.setDistanceThreshold(0.05);
	//ransac.setEMIterations(500);
	//ransac.setDistanceThreshold(0.001);
	//ransac.setMaxIterations(1000);
	ransac.computeModel();
	std::vector<int> ransacInlierLIdx;
	ransac.getInliers(ransacInlierLIdx); 

	std::cout<<"RANSAC inl ratio:"<<ransacInlierLIdx.size()*1.0/filtedIdx.size()<<'\n';

	//get fr index
	copyViaIndex(filtedIdx,ransacInlierLIdx,filtedIdx);

	// 3D line model [px,py,pz,dx,dy,dz] 
	Eigen::VectorXf model_coeff;
	ransac.getModelCoefficients(model_coeff);

	Eigen::VectorXf resultCoeff;
	//optimal coeff
	model_line->optimizeModelCoefficients(ransacInlierLIdx,model_coeff,resultCoeff);

	if( verbase ){
		std::cout<<"Model\n"
			<<" [px,py,pz,dx,dy,dz] =\n"<<resultCoeff<<std::endl ;
	}

	// normalize
	Eigen::Vector3f dir = resultCoeff.block(3,0,3,1);
	dir.normalize();

	Eigen::Vector3f fnv(0,1,0);

	Eigen::Vector3f wallNV = dir.cross(fnv);
	wallNV.normalize();


	Eigen::Vector3f p0 = resultCoeff.head(3);

	float coeff_d = wallNV.dot(p0) * -1;

	res.model.resize(4,1);
	res.model<< wallNV , coeff_d;


	Eigen::Vector3f pOnPL = findAP0(res.model);

	// get 3 inlier 	
	for (int i=0; i < PlaneData::CSS_VNUM ; i++)
	{
		int idx=ransacInlierLIdx[i];

		res.pts[i]=tarPcd->points[idx].getVector3fMap(); 


		if( !ptsIsOnPlane(res.pts[i] , res.model )){
			// prj point 
			res.pts[i]=prjPts2Plane( res.pts[i] ,res.model, pOnPL );
		}

		// check all pts is on the plane
		myassert( ptsIsOnPlane(res.pts[i] , res.model ),"pts not on plane - planeRANSAC_byXZLine");

	}

	if( sameDirC0( res.model.head(3), res.pts[0]-REF_CAM0 ) ){
		res.model=-1*res.model;
	}

}

float planeRANSAC_LSE( const int TID , cv::Mat const & indImg,
				 PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0,
				 PlaneData & res )
{
	static int call_id=0;

	call_id=call_id+1;

	// [WORLD SPACE]
	std::vector<int> tarIdx;
	findCVIMGIdx(TID,indImg,tarIdx);

	// filter-out pts via 'map'
	PointCloudXYZ::Ptr tarPcd(new PointCloudXYZ());
	pcl::copyPointCloud(*f_xyzPcd , tarIdx , *tarPcd); 

	// project to XZ plane
	for( int i=0 ; i < tarPcd->size() ; i++ ){
		tarPcd->points[i].y=0;
	}
	//-------------------------------------------------------------------------
	/*
	char pname[1024]={0};
	memset(pname,0,sizeof(pname));
	sprintf(pname,"wall_%d.pcd",call_id);
	pcl::io::savePCDFileBinary(std::string(pname),*tarPcd);
	*/
	//-------------------------------------------------------------------------

	// filter-out pts via NN filter
	
	std::vector<int> filtedIdx=tarIdx; 
	//std::vector<int> filtedIdx; 
	//knnFilter(tarPcd,&tarIdx,filtedIdx);

	// update pts
	//tarPcd->clear();
	//pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);

	//pcl::SampleConsensusModelPlane< pcl::PointXYZ >::Ptr 
	//	model_plane( new  pcl::SampleConsensusModelPlane< pcl::PointXYZ  >( tarPcd ));


	pcl::SampleConsensusModelLine< pcl::PointXYZ >::Ptr 
		model_line( new  pcl::SampleConsensusModelLine< pcl::PointXYZ  >( tarPcd ));

	pcl::LeastMedianSquares<pcl::PointXYZ> ransac(model_line);
	ransac.setDistanceThreshold(0.5);
	ransac.computeModel();

	//-------------------------------------------------------------------------
	/*

	point_on_line.x : the X coordinate of a point on the line
	point_on_line.y : the Y coordinate of a point on the line
	point_on_line.z : the Z coordinate of a point on the line
	line_direction.x : the X coordinate of a line's direction
	line_direction.y : the Y coordinate of a line's direction
	line_direction.z : the Z coordinate of a line's direction

	*/
	Eigen::VectorXf model_coeff;
	ransac.getModelCoefficients(model_coeff);

	//-------------------------------------------------------------------------
	// make a plane
	//std::cout<<"model_coeff="<<model_coeff.transpose()<<"\n";

	Eigen::Vector3f dir;
	dir << model_coeff(3),model_coeff(4),model_coeff(5);
	dir.normalize();

	Eigen::Vector3f frnv;
	frnv<<0,1,0;
	Eigen::Vector3f pl_nv = dir.cross(frnv);
	pl_nv.normalize();

	// ax+by+cz+d = 0;
	Eigen::Vector3f p0;
	p0<< model_coeff(0),model_coeff(1),model_coeff(2);
	float pl_d = -1 * pl_nv.dot(p0);


	Eigen::VectorXf resultCoeff;
	resultCoeff.resize(4);
	resultCoeff << pl_nv,pl_d; 

	res.model=resultCoeff;

	Eigen::Vector3f pOnPL = findAP0(res.model);

	std::vector<int> const & VINL_IDX = filtedIdx;
	// get 3 inlier 	
	for (int i=0; i < PlaneData::CSS_VNUM ; i++)
	{
		int idx= (int)(i*tarPcd->points.size()/PlaneData::CSS_VNUM);

		res.pts[i]=tarPcd->points[idx].getVector3fMap(); 
		
		if( !ptsIsOnPlane(res.pts[i] , res.model )){
			// prj point 
			res.pts[i]=prjPts2Plane( res.pts[i] ,res.model, pOnPL );
		}

		// check all pts is on the plane
		// myassert( ptsIsOnPlane(res.pts[i] , res.model ),"pts not on plane - planeRANSAC_LSE");
		if( !ptsIsOnPlane(res.pts[i] , res.model )  ){
			float diff= res.model.head(3).dot(res.pts[i])+res.model.w();
			std::cout<<"planeRANSAC_LSE diff = "<<diff<<'\n';
		}

	}

	if( sameDirC0( res.model.head(3), res.pts[0]-REF_CAM0 ) ){
		res.model=-1*res.model;
	}

	//-------------------------------------------------------------------------
	// calculate error
	// 4xN
	Eigen::MatrixXf mat4 = tarPcd->getMatrixXfMap(4,4,0);
	// Nx4
	mat4.transposeInPlace();

	// 4x1
	Eigen::Vector4f model = res.model;

	Eigen::MatrixXf cof_d = mat4*model;
	float mean_error=(cof_d).array().mean();

	float err_var=(cof_d.array()-mean_error).abs().pow(2).sum() / cof_d.rows();
	
	//-------------------------------------------------------------------------
	
	std::cout<<"plane model="<<res.model.transpose()<<"\n";

	return err_var;
}



//
// use fr points 
// 
float planeRANSAC_byXZLine_room
	( const int TID , cv::Mat const & indImg,
					  PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0,
					  PlaneData & res )
{

	bool verbase=0;

	// [FR SPACE]
	std::vector<int> tarIdx;
	findCVIMGIdx(TID,indImg,tarIdx);

	PointCloudXYZ::Ptr tarPcd(new PointCloudXYZ());
	// filter-out pts via 'map'
	pcl::copyPointCloud(*f_xyzPcd , tarIdx , *tarPcd); 

	// project to XZ plane
	for( int i=0 ; i < tarPcd->size() ; i++ ){
		tarPcd->points[i].y=0;
	}


	// filter-out pts via NN filter
	std::vector<int> filtedIdx; 
	
	//*
	knnFilter(tarPcd,&tarIdx,filtedIdx,3,100);
	if( verbase ){
		std::cout<<"knn filting ratio="<<filtedIdx.size()*1.0/tarIdx.size()<<'\n';
	}
	//*/
	filtedIdx = tarIdx;


	// update pts
	tarPcd->clear();
	pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);

	pcl::SampleConsensusModelLine< pcl::PointXYZ >::Ptr 
		model_line( new  pcl::SampleConsensusModelLine< pcl::PointXYZ  >( tarPcd ));


	//=====================================================================================
	// pro sac 
	
	//pcl::ProgressiveSampleConsensus<pcl::PointXYZ> ransac(model_line);
	//ransac.setDistanceThreshold(0.05);
	//ransac.computeModel();

	//=====================================================================================
	// maximum likelihood sac
	//pcl::MaximumLikelihoodSampleConsensus<pcl::PointXYZ> ransac(model_line);	
	//ransac.setEMIterations(500);
	//ransac.setDistanceThreshold(0.005);
	//ransac.setMaxIterations(1000);

	//=====================================================================================
	// M-SAC
	//*
	pcl::MEstimatorSampleConsensus<pcl::PointXYZ> ransac(model_line);
	ransac.setDistanceThreshold(0.05);
	ransac.computeModel();

	std::vector<int> ransacInlierLIdx;
	ransac.getInliers(ransacInlierLIdx); 

	float inlRatio = ransacInlierLIdx.size()*1.0/filtedIdx.size();
	std::cout<<"RANSAC inl ratio:"<<inlRatio<<'\n';
	//*/
	
	//get fr index
	copyViaIndex(filtedIdx,ransacInlierLIdx,filtedIdx);

	//=====================================================================================
	
	//
	//pcl::LeastMedianSquares<pcl::PointXYZ> ransac(model_line);
	//ransac.computeModel();
	
	//=====================================================================================
	 
	// 3D line model [px,py,pz,dx,dy,dz] 
	Eigen::VectorXf model_coeff;
	ransac.getModelCoefficients(model_coeff);

	Eigen::VectorXf resultCoeff;
	resultCoeff=model_coeff;

	//=====================================================================================
	// calculate plane errors



	//=====================================================================================
	/*
	//optimal coeff
	model_line->optimizeModelCoefficients(ransacInlierLIdx,model_coeff,resultCoeff);
	if( verbase ){
		std::cout<<"Model\n"
			<<" [px,py,pz,dx,dy,dz] =\n"<<resultCoeff<<std::endl ;
	}
	*/
	//=====================================================================================

	// normalize
	Eigen::Vector3f dir = resultCoeff.block(3,0,3,1);
	dir.normalize();

	Eigen::Vector3f fnv(0,1,0);

	Eigen::Vector3f wallNV = dir.cross(fnv);
	wallNV.normalize();


	Eigen::Vector3f p0 = resultCoeff.head(3);

	float coeff_d = wallNV.dot(p0) * -1;

	res.model.resize(4,1);
	res.model<< wallNV , coeff_d;


	Eigen::Vector3f pOnPL = findAP0(res.model);

	// get 3 inlier 	
	for (int i=0; i < PlaneData::CSS_VNUM ; i++)
	{
		int idx=ransacInlierLIdx[i];

		res.pts[i]=tarPcd->points[idx].getVector3fMap(); 


		if( !ptsIsOnPlane(res.pts[i] , res.model )){
			// prj point 
			res.pts[i]=prjPts2Plane( res.pts[i] ,res.model, pOnPL );
		}

		// check all pts is on the plane
		myassert( ptsIsOnPlane(res.pts[i] , res.model ),"pts not on plane");

		
	}

	if( sameDirC0( res.model.head(3), res.pts[0]-REF_CAM0 ) ){
		res.model=-1*res.model;
	}


	return inlRatio;
}

bool ptsIsOnPlane(Eigen::Vector3f const &  v0 , Eigen::Vector4f const &  model)
{

	float res= model.head(3).dot(v0)+model.w();

	return abs(res) < 1e-6;

}

Eigen::Vector3f findAP0( Eigen::VectorXf const & PL_MODEL )
{

	//ax + by + cz + d=0

	if( PL_MODEL.x() != 0 ){
		// x= -1*(by+cz+d)/a = -d/a
		// [x,0,0] on plane
		float x_val = -1*PL_MODEL.w()/PL_MODEL.x();

		return Eigen::Vector3f(x_val,0,0);

	}else if( PL_MODEL.y() !=0 ){

		// y= -1*(ax+cz+d)/b = -d/b
		// [0,y,0] on plane
		float y_val = -1*PL_MODEL.w()/PL_MODEL.y();
		return Eigen::Vector3f(0,y_val,0);

	}else if(PL_MODEL.z() !=0 ){

		// z= -1*(d)/z = -d/z
		// [0,0,z] on plane
		float z_val = -1*PL_MODEL.w()/PL_MODEL.z();
		return Eigen::Vector3f(0,0,z_val);

	}else
	{
		throw "exp findAP0";
	}

}


Eigen::MatrixXf rotateV2V(Eigen::Vector3f srcVec , Eigen::Vector3f dstVec ){

	/*
	vtheta = acos(dot(fNvec,[0,1,0]));  
	size_2d=[obj.y_size,obj.x_size];
	% rotation 
	laxis = cross( fNvec,[0,1,0]);
	laxis = laxis ./ norm(laxis,2);

	fRmat = createRotation3dLineAngle( [0,0,0,laxis(1),laxis(2),laxis(3)] , vtheta );
	fRmat = fRmat(1:3,1:3);
	obj.fgrhPts3_1d = (fRmat*obj.grPts3_1d')';
	*/

	srcVec.normalize();
	dstVec.normalize();

	// theta
	float e= acos(srcVec.dot(dstVec));

	// rotation axis
	Eigen::Vector3f axis = srcVec.cross(dstVec);

	axis.normalize();

	float ux=axis.x();
	float uy=axis.y();
	float uz=axis.z();

	Eigen::MatrixXf rmat ;
	rmat.resize(3,3);

	rmat << cos(e)+ux*ux*(1-cos(e))        , ux*uy*(1-cos(e)) - uz*sin(e)  , ux*uz*(1-cos(e))+uy*sin(e),
		uy*ux*(1-cos(e)) + uz*sin(e)   , cos(e) + uy*uy*(1-cos(e))    , uy*uz*(1-cos(e))-ux*sin(e) ,
		uz*ux*(1-cos(e)) - uy*sin(e)   , uz*uy*(1-cos(e))+ux*sin(e)   , cos(e)+uz*uz*(1-cos(e)) ;

	return rmat;
}


//*
//void estimateNE( string const& INPCD_PATH ,string const& OUT_PATH , string const& N_METHOD ,
//		 float ne_maxDepthChange, string & ne_borderPolicy , float ne_normal_smoothSize  )
void estimateNE( pcl::PointCloud< pcl::PointXYZ >::Ptr cloud_in, pcl::PointCloud< pcl::PointXYZ >::Ptr cloud_out ,
				float ne_maxDepthChange, float ne_normal_smoothSize  )
{ 

	//load input point cloud
	//pcl::PointCloud< InputNodeT >::Ptr cloud_in( loadPCD<InputNodeT>(INPCD_PATH.c_str()) ); 
	//pcl::PointCloud< OutputNodeT >::Ptr cloud_out( new pcl::PointCloud< OutputNodeT >() );

	// calculating NORMAL
	typedef pcl::PointXYZ  InputNodeT ;
	typedef pcl::PointNormal OutputNodeT ;

	pcl::IntegralImageNormalEstimation< InputNodeT , OutputNodeT > neInt;
	neInt.setViewPoint(0,0,0);

	//"ignore - BORDER_POLICY_IGNORE \n"
	//"mirror - BORDER_POLICY_MIRROR \n"

	//if( ne_borderPolicy == std::string("mirror") )
	neInt.setBorderPolicy( pcl::IntegralImageNormalEstimation< InputNodeT , OutputNodeT >::BORDER_POLICY_MIRROR );
	//else
	//	neInt.setBorderPolicy( pcl::IntegralImageNormalEstimation< InputNodeT , OutputNodeT >::BORDER_POLICY_IGNORE );

	neInt.setNormalEstimationMethod (   pcl::IntegralImageNormalEstimation<InputNodeT,OutputNodeT>::COVARIANCE_MATRIX  ); 

	neInt.setMaxDepthChangeFactor(ne_maxDepthChange);
	neInt.setNormalSmoothingSize(ne_normal_smoothSize);

	neInt.setDepthDependentSmoothing(false); 
	neInt.setInputCloud(cloud_in->makeShared()); 

	//pre copy other data
	pcl::PointCloud<OutputNodeT> pcd_ne;

	pcl::copyPointCloud(*cloud_in,pcd_ne);
	cout<<"NormalEstimation START \n";
	neInt.compute(pcd_ne);
	cout<<"NormalEstimation DONE \n";

	pcl::copyPointCloud(pcd_ne,*cloud_out);

	//OUT_PATH
	//pcl::io::savePCDFileBinary( OUT_PATH.c_str() , *cloud_out );
	for (int k = 0; k < pcd_ne.size(); k++)
	{
		cloud_out->points[k].x=pcd_ne.points[k].normal_x;
		cloud_out->points[k].y=pcd_ne.points[k].normal_y;
		cloud_out->points[k].z=pcd_ne.points[k].normal_z;
	}

}

void extr_obb( Eigen::MatrixXf const BASIS , PointCloudXYZ::Ptr tarPcd, Eigen::Matrix4f const & FR2WORLD ,BoxDataV2 & resBox )
{
	bool verbase=0;

	Eigen::Matrix3f const obb_fr2local=BASIS.inverse();
	Eigen::Matrix3f const obb_local2fr=BASIS;

	// 4 x N
	Eigen::MatrixXf pts4 = tarPcd->getMatrixXfMap(); // via copy
	Eigen::MatrixXf frpts3 = pts4.block(0,0,3,pts4.cols());

	// 3xN
	myassert(frpts3.rows()==3);

	frpts3.transposeInPlace();
	// to local AABB - record center in ORIGINAL space  
	Eigen::RowVector3f	gcenter = frpts3.colwise().mean();

	// Center TX
	frpts3 = frpts3.rowwise() - gcenter; 

	// 3xN
	frpts3.transposeInPlace();
	frpts3=obb_fr2local * frpts3;	

	// Nx3
	frpts3.transposeInPlace();
	frpts3 = frpts3.rowwise() + gcenter; 


	// get the AABB
	Eigen::Vector3f maxV = frpts3.colwise().maxCoeff();
	Eigen::Vector3f minV = frpts3.colwise().minCoeff();

	Eigen::MatrixXf aabb = getBoxByMaxMin(maxV,minV);

	// ex box3d_fr
	// use Nx3
	Eigen::MatrixXf obb_fr = cenTx(gcenter,obb_local2fr,aabb);

	// update AABB by fr center 
	Eigen::RowVector3f newC0=obb_fr.colwise().mean();

	Eigen::MatrixXf aabb_new = cenTx(newC0,obb_fr2local,obb_fr);

	// ex box3d_world 
	// ex box2d 
	Eigen::MatrixXf obb_world  = cenTx44(Eigen::Vector3f(0,0,0),FR2WORLD,obb_fr);


	resBox.setBox3d_world(obb_world);
	resBox.setBox3d_fr(obb_fr);
	resBox.setBox3d_aabb(aabb_new);

	resBox.setObb_fr2aabb(obb_fr2local);
	resBox.setObb_aabb2Fr(obb_local2fr);


	// debug
	if( verbase ){
		//std::cout<<"aabb\n"<<aabb<<std::endl;
		//std::cout<<"obb_fr\n"<<obb_fr<<std::endl;
		//std::cout<<"obb_world\n"<<obb_world<<std::endl;
		

		const Eigen::MatrixXf mt_obb2aabb=Eigen::Map<const Eigen::MatrixXf>( resBox.getFrOBB2AABB() ,3,3);
		const Eigen::MatrixXf mt_aabb2obb=Eigen::Map<const Eigen::MatrixXf>( resBox.getAABB2FrOBB() ,3,3);
		
		std::cout<<"================EX=======================\n";
		std::cout<<"obb_fr2local=\n\n"<<obb_fr2local<<std::endl;
		std::cout<<"obb_local2fr=\n\n"<<obb_local2fr<<std::endl;
		std::cout<<"mx=\n"<<obb_fr2local*obb_local2fr<<std::endl;
		std::cout<<"mx re=\n"<<mt_obb2aabb*mt_aabb2obb<<std::endl;
		
		std::cout<<"check match=\n";
		std::cout<<"frobb2aabb="<<(mt_obb2aabb-obb_fr2local).array().abs().sum()<<"\n";
		std::cout<<"aabb2obbfr="<<(mt_aabb2obb-obb_local2fr).array().abs().sum()<<"\n";
		
		std::cout<<"mt_obb2aabb--------\n"<<mt_obb2aabb<<'\n'
				 <<"obb_fr2local--------\n"<<obb_fr2local<<'\n';

		std::cout<<"=======================================\n";


		//std::cout<<"fr2worldMat\n"<<FR2WORLD<<std::endl;
	}
}


void mBoxScale( BoxDataV2 & resBox, 
			   Eigen::Matrix4f const & FR2WORLD  ,  Eigen::RowVector3f  scaleVec )
{
	bool verbase=0;

	// start from aabb
	Eigen::MatrixXf aabb= Eigen::Map< Eigen::MatrixXf>( resBox.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map< Eigen::MatrixXf>( resBox.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map< Eigen::MatrixXf>( resBox.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( resBox.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( resBox.getAABB2FrOBB() ,3,3);


	if( verbase ){

		std::cout<<"fr2world=["<<FR2WORLD<<"]\n\n";

		std::cout<<"ori_aabb=[\n";
		std::cout<<aabb.transpose()<<"];\n";
		std::cout<<"obb_fr=[\n";
		std::cout<<obb_fr.transpose()<<"];\n";
		std::cout<<"obb_world=[\n";
		std::cout<<obb_world.transpose()<<"];\n";
	}

	// 3xN -> Nx3
	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();

	// FR-> local basis -> AABB -> FR -> World
	Eigen::RowVector3f cen_Fr = obb_fr.colwise().mean();
	 
	aabb=aabb.rowwise() - cen_Fr;
	aabb.col(0)=aabb.col(0)*scaleVec(0);
	aabb.col(1)=aabb.col(1)*scaleVec(1);
	aabb.col(2)=aabb.col(2)*scaleVec(2);
	aabb=aabb.rowwise() + cen_Fr;
	
	
	obb_fr=cenTx(cen_Fr,aabb2ObbFr,aabb);

	obb_world=cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD,obb_fr);

	resBox.setBox3d_aabb(aabb);
	resBox.setBox3d_fr(obb_fr);
	resBox.setBox3d_world(obb_world);
	
	if( verbase ){
		std::cout<<"aft_aabb=[\n";
		std::cout<<aabb<<"];\n---\n";
		std::cout<<"aft_fr=[\n";
		std::cout<<obb_fr<<"];\n";
		std::cout<<"aft_world=[\n";
		std::cout<<obb_world<<"];\n==============\n\n";
	}

}

//
void mBoxEntend2Plane_Side(BoxDataV2 & resBox, PlaneData const & PLANE_FR , Eigen::Matrix4f const & FR2WORLD )
{
	bool verbase=0;

	// handle X and Z axis extension

	// start from aabb
	Eigen::MatrixXf aabb= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( resBox.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( resBox.getAABB2FrOBB() ,3,3);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();

	if( verbase ){
		std::cout<<"aabb\n"<<aabb<<"\n";
		std::cout<<"obb_fr\n"<<obb_fr<<"\n";
	}

	//[FR_SPACE] calculate point to plane distance 
	// we only test top face vertices

	float minDist = std::numeric_limits<float>::max();
	int minVIdx = -1;
	float pdist=0;
	for( int  iv=0 ;iv<4 ;iv++ ){
		pdist = pointPlaneABSDist(obb_fr.row(iv),PLANE_FR);
		if ( pdist < minDist )
		{
			minDist = pdist;
			minVIdx = iv;
		}
	}


	// extend the vertex
	Eigen::Vector3f nv = PLANE_FR.model.head(3).normalized();
	Eigen::Vector3f closeVex =  obb_fr.row(minVIdx); 


	float signDist = pointPlaneSignDist(closeVex,PLANE_FR);

	Eigen::Vector3f extv = closeVex + signDist*nv;

	//guards
	double diff = abs((PLANE_FR.model.head(3)).dot(extv)+PLANE_FR.model.w()) ;

	if( diff >  1e-6 ){
		//myassert( diff < 1e-6, "extv should lie on plane. - mBoxEntend2Plane_Side");
	}

	//-------------------------------------------------------------------------
	// select the diagonal to extending the min/max pair
	// 0,1,2,3
	// 4,5,6,7
	// v2 should be the bottom face vertex
	int v2Idx = (minVIdx+2)%4+4;

	// re-extract boxes
	Eigen::RowVector3f newVmax = extv;
	Eigen::RowVector3f newVmin = obb_fr.row(v2Idx);


	if( verbase ){
		std::cout<<"closeVex \n"<<closeVex.transpose()<<"\n";
		std::cout<<"extv     \n"<<extv.transpose()<<"\n";
		std::cout<<"extv-diag\n"<<newVmin.transpose()<<"\n";
	}


	// MaxMin should only work in AABB domain

	Eigen::MatrixXf redOBBfr ;
	redOBBfr.resize(2,3);
	redOBBfr.row(0)=newVmax;
	redOBBfr.row(1)=newVmin;

	Eigen::RowVector3f newFRCen = redOBBfr.colwise().mean();
	// use new fr center
	Eigen::MatrixXf red_AABB = cenTx( newFRCen , obbFr2AABB, redOBBfr);
	Eigen::RowVector3f nmax = red_AABB.colwise().maxCoeff();
	Eigen::RowVector3f nmin = red_AABB.colwise().minCoeff();

	Eigen::MatrixXf newAABB = getBoxByMaxMin_row( nmax , nmin );
	 
	//-------------------------------------------------------------------------  
	// update boxes 
	Eigen::MatrixXf newOBBFr = cenTx( newFRCen , aabb2ObbFr, newAABB);
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBFr);

	resBox.setBox3d_aabb(newAABB);
	resBox.setBox3d_fr(newOBBFr);
	resBox.setBox3d_world(newOBBWorld);

}

// use bottom face of the vertex
void mBoxEntend2Plane_BotVex(BoxDataV2 & resBox, PlaneData const & PLANE_FR , Eigen::Matrix4f const & FR2WORLD )
{ 
	// handle X and Z axis extension

	// start from aabb
	Eigen::MatrixXf aabb= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( resBox.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( resBox.getAABB2FrOBB() ,3,3);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();

	//-------------------------------------------------------------------------
	//[FR_SPACE] calculate point to plane distance 
	// we only test BOTTOM face vertices 
	float minDist = std::numeric_limits<float>::max();
	int minVIdx = -1;
	float pdist=0;

	for( int  iv=4 ;iv<8 ;iv++ ){
		pdist = pointPlaneABSDist(obb_fr.row(iv),PLANE_FR);
		if ( pdist < minDist )
		{
			minDist = pdist;
			minVIdx = iv;
		}
	}


	// extend the vertex
	Eigen::Vector3f nv = PLANE_FR.model.head(3).normalized();
	Eigen::Vector3f minvex = obb_fr.row(minVIdx);

	float signDist = pointPlaneSignDist(minvex,PLANE_FR);

	Eigen::Vector3f extv = minvex + signDist*nv;

	double diff = abs((PLANE_FR.model.head(3)).dot(extv)+PLANE_FR.model.w()) ;
	// assert 
	myassert( abs((PLANE_FR.model.head(3)).dot(extv)+PLANE_FR.model.w()) < 1e-10 , "extv should lie on plane. - mBoxEntend2Plane_BotVex");

	//-------------------------------------------------------------------------

	// select the diagonal to extending the min/max pair
	// 0,1,2,3
	// 4,5,6,7 
	int v2Idx = (minVIdx+2)%4; // v2 should be the top face vertex

	// [BOTTOM mode]
	// re-extract boxes
	Eigen::RowVector3f newVmax = obb_fr.row(v2Idx);
	Eigen::RowVector3f newVmin = extv;
	// MaxMin should only work in AABB domain

	Eigen::MatrixXf redOBBfr ;
	redOBBfr.resize(2,3);
	redOBBfr.row(0)=newVmax;
	redOBBfr.row(1)=newVmin;

	Eigen::RowVector3f newFRCen = redOBBfr.colwise().mean();
	// use new fr center
	Eigen::MatrixXf red_AABB = cenTx( newFRCen , obbFr2AABB, redOBBfr);
	Eigen::RowVector3f nmax = red_AABB.colwise().maxCoeff();
	Eigen::RowVector3f nmin = red_AABB.colwise().minCoeff();

	Eigen::MatrixXf newAABB = getBoxByMaxMin_row( nmax , nmin );

	//-------------------------------------------------------------------------  
	// update boxes 
	Eigen::MatrixXf newOBBFr = cenTx( newFRCen , aabb2ObbFr, newAABB);
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBFr);


	resBox.setBox3d_aabb(newAABB);
	resBox.setBox3d_fr(newOBBFr);
	resBox.setBox3d_world(newOBBWorld);

}

void mBoxRotate_Y( BoxDataV2 & resBox, double y_theta , Eigen::Matrix4f const & FR2WORLD )
{
	// handle X and Z axis extension

	bool verbase=1;

	// start from aabb
	Eigen::MatrixXf aabb= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( resBox.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( resBox.getAABB2FrOBB() ,3,3);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();
	//=========================================================================================================

	Eigen::Matrix3f rot_m;
	rot_m = Eigen::AngleAxisf(y_theta,  Eigen::Vector3f::UnitY());

	// update TX
	// New_FR_OBB =  rot_m * aabb2fr * aabb
	Eigen::MatrixXf new_aabb2ObbFr = rot_m*aabb2ObbFr;
	Eigen::MatrixXf new_obbFr2AABB = new_aabb2ObbFr.inverse(); 

	//=========================================================================================================
	// update boxes  
	Eigen::RowVector3f frCen = obb_fr.colwise().mean(); 
	 
	// aabb would not be changed
	//Eigen::MatrixXf newOBBFr = cenTx( frCen , new_aabb2ObbFr, aabb);
	Eigen::MatrixXf newOBBFr = cenTx( frCen , rot_m, obb_fr);


	// world box would be updated 
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBFr);


	if(verbase)
	{
		std::cout<<"frCen:"<<frCen<<std::endl;
		std::cout<<"obb_fr:"<<obb_fr<<std::endl; 
		std::cout<<"newOBBFr:"<<newOBBFr<<std::endl;

		std::cout<<"rot_m:(theta)"<<y_theta<<"\n"<<rot_m<<std::endl; 
		/*
		std::cout<<"obbFr2AABB:\n"<<obbFr2AABB<<std::endl;
		std::cout<<"aabb2ObbFr:\n"<<aabb2ObbFr<<std::endl;

		std::cout<<"new_obbFr2AABB:\n"<<new_obbFr2AABB<<std::endl;
		std::cout<<"new_aabb2ObbFr:\n"<<new_aabb2ObbFr<<std::endl;
		*/

		std::cout<<"frCen:"<<frCen<<std::endl; 
		std::cout<<"new cen:"<<newOBBFr.colwise().mean()<<std::endl; 


		Eigen::MatrixXf de_obb = cenTx( frCen , aabb2ObbFr, aabb);

		std::cout<<"------------------\n"
			     <<"old obb:\n\t"<<obb_fr<<"\n"
				 <<"ret obb:\n\t"<<de_obb<<"\n\n"
  				 <<"old-ret OBB sum="<<(obb_fr-de_obb).array().sum()<<"\n" ;


		Eigen::RowVector3f newfrCen = newOBBFr.colwise().mean();
		Eigen::MatrixXf de_new_obb = cenTx( newfrCen , new_aabb2ObbFr, aabb); 

		std::cout<<"-------NEW OBB TEST-----------\n"
			<<"new obb:\n\t"<<newOBBFr<<"\n"
			<<"ret obb:\n\t"<<de_new_obb<<"\n\n"
			<<"new-ret OBB sum="<<(de_new_obb-newOBBFr).array().sum()<<"\n" ;


		Eigen::MatrixXf de_aabb = cenTx( frCen , obbFr2AABB, obb_fr);
		std::cout<<"------------------\n"
				 <<"old-ret aabb sum="<<(aabb-de_aabb).array().sum()<<"\n" ;


	}

	//=========================================================================================================  
	// SAVE
	// aabb is not changed

	resBox.setBox3d_fr(newOBBFr);
	resBox.setBox3d_world(newOBBWorld);
	  
	resBox.setObb_aabb2Fr(new_aabb2ObbFr);
	resBox.setObb_fr2aabb(new_obbFr2AABB); 


}

void mBoxRotate_mat( BoxDataV2 & resBox, Eigen::Matrix3f const & R , Eigen::Matrix4f const & FR2WORLD )
{
	// handle X and Z axis extension

	// start from aabb
	Eigen::MatrixXf aabb= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( resBox.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( resBox.getAABB2FrOBB() ,3,3);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();
	//=========================================================================================================

	// update TX
	//
	// New_FR_OBB =  rot_m * aabb2fr * aabb
	obbFr2AABB = R*aabb2ObbFr; 
	aabb2ObbFr = obbFr2AABB.inverse();

	//=========================================================================================================
	// update boxes  
	Eigen::RowVector3f frCen = obb_fr.colwise().mean(); 

	// aabb is not change
	Eigen::MatrixXf newOBBFr = cenTx( frCen , aabb2ObbFr, aabb);
	// world box would be updated 
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBFr);


	//=========================================================================================================  
	// SAVE
	// aabb is not changed

	resBox.setBox3d_fr(newOBBFr);
	resBox.setBox3d_world(newOBBWorld);

	resBox.setObb_aabb2Fr(aabb2ObbFr);
	resBox.setObb_fr2aabb(obbFr2AABB); 

}

// This function require box plane , set in iniBoxAfterLoading
void mBox_Align2Wall(BoxDataV2 & resBox,PlaneData const & WALL, Eigen::Matrix4f const & FR2WORLD ){

	Eigen::Vector3f wall_nv = WALL.model.head(3);
	wall_nv.y()=0;
	wall_nv.normalize(); 
	 
	//// construct box plane
	//static int FACE_VIDX[][3]={		
	//	{0,1,2},
	//	{0,4,5}, //1
	//	{1,5,6}, //2
	//	{2,6,7}, //3
	//	{3,7,4}, //4
	//	{4,5,6}
	//};	
	auto & boxPlanes=resBox.getBoxPlane_fr();

	Eigen::Vector3f nv1=boxPlanes[1].model.head(3);
	Eigen::Vector3f nv2=boxPlanes[2].model.head(3);

	nv1.normalize();
	nv2.normalize();

	double th1 = acos( wall_nv.dot(nv1) );
	double th2 = acos( wall_nv.dot(nv2) );

	if( abs(th1) < abs(th2) ){
		// th1 is min
		 
		Eigen::Vector3f rot_axis = nv1.cross(wall_nv);
		rot_axis.normalize(); 
		Eigen::Matrix3f rot_m ;
		rot_m=Eigen::AngleAxisf(abs(th1),  rot_axis );

		mBoxRotate_mat( resBox , rot_m, FR2WORLD );

	}else{
		// th2 is min

		Eigen::Vector3f rot_axis = nv2.cross(wall_nv);
		rot_axis.normalize();
		Eigen::Matrix3f rot_m ;
		rot_m=Eigen::AngleAxisf(abs(th2),  rot_axis );

		mBoxRotate_mat( resBox , rot_m, FR2WORLD );
	} 
	 
}

// use Wall's normal to retake box 
// [NOTE] you should be aware that the retaking action would cancel the floor extension refinement
void mBox_Align2Wall_retake(BoxDataV2 & resBox,PlaneData const & WALL,
							PointCloudXYZ::Ptr f_xyzPcd , PointCloudXYZ::Ptr f_nvPcd ,
							Eigen::MatrixXf const& FR2WORLD ){

	bool verbase=0;

	Eigen::Vector3f wall_nv = WALL.model.head(3);
	wall_nv.y()=0;
	wall_nv.normalize(); 

#ifdef _DEBUG
	int numofpts=cv::countNonZero(resBox.getSegMask());
#endif // _DEBUG
	
	int oriUID = resBox.getUID();

	exbox_withnv(wall_nv,255,resBox.getSegMask(),f_xyzPcd,f_nvPcd,FR2WORLD,resBox);

	resBox.setUID(oriUID);

	if (verbase)
	{
		std::cout<<"call ->[b-"<<resBox.getUID()<<"][mBox_Align2Wall_retake]\n";
		std::cout<<"after check:\n";
		checkBox(resBox,FR2WORLD);
		std::cout<<"=========================";
	}
}
 

void exbox_withnv( Eigen::Vector3f nv1,
		   const int TID  , cv::Mat const &  indImg ,
		   PointCloudXYZ::Ptr f_xyzPcd , PointCloudXYZ::Ptr f_nvPcd ,
		   Eigen::MatrixXf const& fr2worldMat,
		   BoxDataV2 & resBox   )
{
	// world_pts * W_2_FR --> FR pts
	// FR pts * FR_2_W --> world_pts

	//NOTE that we must use Ptr to access point_cloud which guarantee the action of shared_ptr is right.

	bool verbase=false;

	// extract target points  
	std::vector<int> tarIds;
	findCVIMGIdx(TID,indImg,tarIds);

	if(verbase){
		LOG_MSG_L0("target pts #=%d\n",tarIds.size());
	}

	//myassert(tarIds.size()>100);

	PointCloudXYZ::Ptr tarPcd(new PointCloudXYZ());
	// filter-out pts via 'map'
	pcl::copyPointCloud(*f_xyzPcd , tarIds , *tarPcd); 

	// filter-out pts via NN filter
	std::vector<int> filtedIdx; 
	knnFilter(tarPcd,&tarIds,filtedIdx);

	//[FIND OBB TX]
	Eigen::Vector3f axis0,axis1,axis2; 
	 
	axis0=nv1;
	axis1<<0,1,0; 

	axis2 = axis0.cross(axis1);
	axis2.normalize();  
	//-------------------------------------------------------------------------
	// OBB 

	// reset 3d target pcd 
	pcl::copyPointCloud(*f_xyzPcd , filtedIdx , *tarPcd);

	// tx points by OBB
	Eigen::Matrix3f obbmat;
	obbmat.col(0)=axis0;
	obbmat.col(1)=axis1;
	obbmat.col(2)=axis2;

	extr_obb(obbmat,tarPcd,fr2worldMat,resBox);

	resBox.setUID(TID);
}


 
void mBox_EdgeScale( BoxDataV2 const& BOX_IN , float scale , int v1_idx , int v2_idx , Eigen::Matrix4f const & FR2WORLD ,BoxDataV2 & resBox )
{
	myassert(v1_idx<4,"v1_idx<4");
	myassert(v2_idx<4,"v2_idx<4");
	myassert(v2_idx!=v1_idx,"v2_idx<4");

	if( v1_idx>v2_idx ){
		std::swap(v1_idx,v2_idx);
	}

	if( v1_idx != 0 && v2_idx !=0 ){
		myassert(v2_idx-v1_idx==1,"v2_idx-v1_idx==1");
	}else{
		myassert(v2_idx+v1_idx==3||v2_idx+v1_idx==1,"v2_idx+v1_idx==3");
	}

	// start from aabb
	Eigen::MatrixXf aabb=Eigen::Map<const  Eigen::MatrixXf>( BOX_IN.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr=Eigen::Map<const  Eigen::MatrixXf>( BOX_IN.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<const  Eigen::MatrixXf>( BOX_IN.getBox3d_world() ,3,8);

	Eigen::Map<const  Eigen::MatrixXf> obbFr2AABB( BOX_IN.getFrOBB2AABB() ,3,3);
	Eigen::Map<const  Eigen::MatrixXf> aabb2ObbFr( BOX_IN.getAABB2FrOBB() ,3,3);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();


	//=========================================================================================================
	// 0,1 - f1
	// 1,2 - f2
	// 2,3 - f3
	// 3,0 - f4

	Eigen::Vector3f v1 =obb_fr.row(v1_idx);
	Eigen::Vector3f v2 =obb_fr.row(v2_idx);

	Eigen::Vector3f v1_bot =obb_fr.row(v1_idx+4);
	Eigen::Vector3f v2_bot =obb_fr.row(v2_idx+4);

	// height
	Eigen::Vector3f e1_bot =(v1_bot-v1);
	Eigen::Vector3f e2_bot =(v2_bot-v2);

	// cal new vertices
	const int TOTAL_V=4;

	int v3_idx = ((v1_idx-1+TOTAL_V)%TOTAL_V != v2_idx) ? (v1_idx-1+TOTAL_V)%TOTAL_V : (v1_idx+1+TOTAL_V)%TOTAL_V;
	int v4_idx = ((v2_idx-1+TOTAL_V)%TOTAL_V != v1_idx) ? (v2_idx-1+TOTAL_V)%TOTAL_V : (v2_idx+1+TOTAL_V)%TOTAL_V;

	if( v3_idx <0 ){
		v3_idx=v3_idx+TOTAL_V;
	}
	if( v4_idx <0 ){
		v4_idx=v4_idx+TOTAL_V;
	}
	// get corresponding neighbor
	Eigen::Vector3f v3 =obb_fr.row(v3_idx);
	Eigen::Vector3f v4 =obb_fr.row(v4_idx);

	Eigen::Vector3f e_v31= (v1-v3)*scale;
	Eigen::Vector3f e_v42 = (v2-v4)*scale;

	//=========================================================================================================
	// new vertices
	Eigen::MatrixXf newOBBfr = obb_fr;

	newOBBfr.row(v1_idx)=v3+e_v31;
	newOBBfr.row(v2_idx)=v4+e_v42;

	// update vertices and corresponding vertices in bottom face
	// 0-4
	// 1-5
	// 2-6
	// 3-7

	// update bottom vertices
	newOBBfr.row(v1_idx+4)=v3+e_v31+e1_bot;
	newOBBfr.row(v2_idx+4)=v4+e_v42+e2_bot;

	//=========================================================================================================
	// update boxes  
	Eigen::RowVector3f newFRCen = newOBBfr.colwise().mean();

	// use new fr center
	Eigen::MatrixXf newAABB = cenTx( newFRCen , obbFr2AABB, newOBBfr); 
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBfr);

	//=========================================================================================================  
	// SAVE
	// aabb is not changed

	resBox=BOX_IN;

	resBox.setBox3d_aabb(newAABB); 
	resBox.setBox3d_fr(newOBBfr);
	resBox.setBox3d_world(newOBBWorld); 
}

//
// index start from 0
// assume use top face --> v_idx = [0~3]
void mBox_EdgeScale( BoxDataV2 & resBox , float scale , int v1_idx , int v2_idx , Eigen::Matrix4f const & FR2WORLD  )
{ 
	myassert(v1_idx<4,"v1_idx<4");
	myassert(v2_idx<4,"v2_idx<4");
	myassert(v2_idx!=v1_idx,"v2_idx<4");

	if( v1_idx>v2_idx ){
		std::swap(v1_idx,v2_idx);
	}

	if( v1_idx != 0 && v2_idx !=0 ){
		myassert(v2_idx-v1_idx==1,"v2_idx-v1_idx==1");
	}else{
		myassert(v2_idx+v1_idx==3||v2_idx+v1_idx==1,"v2_idx+v1_idx==3");
	}

	// start from aabb
	Eigen::MatrixXf aabb=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( resBox.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( resBox.getAABB2FrOBB() ,3,3);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();
	
	//=========================================================================================================
	// 0,1 - f1
	// 1,2 - f2
	// 2,3 - f3
	// 3,0 - f4
	 
	Eigen::Vector3f v1 =obb_fr.row(v1_idx);
	Eigen::Vector3f v2 =obb_fr.row(v2_idx);

	Eigen::Vector3f v1_bot =obb_fr.row(v1_idx+4);
	Eigen::Vector3f v2_bot =obb_fr.row(v2_idx+4);

	// height
	Eigen::Vector3f e1_bot =(v1_bot-v1);
	Eigen::Vector3f e2_bot =(v2_bot-v2);

	// cal new vertices
	const int TOTAL_V=4;

	int v3_idx = ((v1_idx-1+TOTAL_V)%TOTAL_V != v2_idx) ? (v1_idx-1+TOTAL_V)%TOTAL_V : (v1_idx+1+TOTAL_V)%TOTAL_V;
	int v4_idx = ((v2_idx-1+TOTAL_V)%TOTAL_V != v1_idx) ? (v2_idx-1+TOTAL_V)%TOTAL_V : (v2_idx+1+TOTAL_V)%TOTAL_V;

	if( v3_idx <0 ){
		v3_idx=v3_idx+TOTAL_V;
	}
	if( v4_idx <0 ){
		v4_idx=v4_idx+TOTAL_V;
	}
	// get corresponding neighbor
	Eigen::Vector3f v3 =obb_fr.row(v3_idx);
	Eigen::Vector3f v4 =obb_fr.row(v4_idx);

	Eigen::Vector3f e_v31= (v1-v3)*scale;
	Eigen::Vector3f e_v42 = (v2-v4)*scale;

	//=========================================================================================================
	// new vertices
	Eigen::MatrixXf newOBBfr = obb_fr;
	
	newOBBfr.row(v1_idx)=v3+e_v31;
	newOBBfr.row(v2_idx)=v4+e_v42;

	// update vertices and corresponding vertices in bottom face
	// 0-4
	// 1-5
	// 2-6
	// 3-7

	// update bottom vertices
	newOBBfr.row(v1_idx+4)=v3+e_v31+e1_bot;
	newOBBfr.row(v2_idx+4)=v4+e_v42+e2_bot;
	
	//=========================================================================================================
	// update boxes  
	Eigen::RowVector3f newFRCen = newOBBfr.colwise().mean();

	// use new fr center
	Eigen::MatrixXf newAABB = cenTx( newFRCen , obbFr2AABB, newOBBfr); 
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBfr);

	//=========================================================================================================  
	// SAVE
	// aabb is not changed

	resBox.setBox3d_aabb(newAABB); 
	resBox.setBox3d_fr(newOBBfr);
	resBox.setBox3d_world(newOBBWorld); 

}
 
// index start from 0
// assume use top face --> v_idx = [0~3]
void mBox_enforceParentSupport( BoxDataV2 & resParBox , BoxDataV2 const & tarChild , Eigen::Matrix4f const & FR2WORLD  )
{    
	Eigen::MatrixXf par_obb_fr= Eigen::Map<Eigen::MatrixXf>( resParBox.getBox3d_fr() ,3,8);
  
	par_obb_fr.transposeInPlace(); 
	//=========================================================================================================

	

	//=========================================================================================================

	// if the center of target child is already inside the parent
	// skip
	Eigen::MatrixXf ch_boxfr_n3=tarChild.getBox3d_fr_n3();
	Eigen::Vector3f ch_cen = ch_boxfr_n3.colwise().mean();

	BGPolygon u_fxz; 

	for( int iv=0 ; iv<4 ; iv++ ){
		u_fxz.outer().push_back(BGPoint(par_obb_fr(iv,0),par_obb_fr(iv,2)));  
	}

	BGPolygon u_hxz;
	boost::geometry::convex_hull(u_fxz, u_hxz);


	bool massIsInside = boost::geometry::covered_by(BGPoint(ch_cen(0),ch_cen(2)) , u_hxz );

	if( massIsInside ){
		return;
	}

	//=========================================================================================================
 
	// found a closest vertex 

	float minDist=std::numeric_limits<float>::infinity();
	int minVIdx=-1;
	for( int iv=0 ; iv<4 ; iv++ ){
		Eigen::Vector3f vex=par_obb_fr.row(iv);
		float vdist = (vex-ch_cen).norm();

		if( vdist < minDist ){
			minDist=vdist;
			minVIdx=iv;
		}
	} 

	// projection direction
	int e_v1_idx = (minVIdx+1)%4;
	int e_v2_idx = (minVIdx-1)%4;

	if( e_v2_idx <0 ){
		e_v2_idx=e_v2_idx+4;
	}

	// e_v1_idx,minVIdx,e_v2_idx
	Eigen::Vector3f v0 = par_obb_fr.row(minVIdx);
	Eigen::Vector3f v1 = par_obb_fr.row(e_v1_idx);
	Eigen::Vector3f v2 = par_obb_fr.row(e_v2_idx);

	Eigen::Vector3f tarCx = ch_cen;

	tarCx.y()=0;
	v0.y()=0;
	v1.y()=0;
	v2.y()=0;

	Eigen::Vector3f ext_vec = tarCx-v0;
	Eigen::Vector3f ext_dir = tarCx-v0;
	ext_dir.normalize();

	Eigen::Vector3f dir_v01 = v0-v1;
	Eigen::Vector3f dir_v02 = v0-v2;

	dir_v01.normalize();
	dir_v02.normalize();

	float cos_v01 = ext_dir.dot(dir_v01);
	float cos_v02 = ext_dir.dot(dir_v02);


	if( cos_v01 > 0 ){
		//require extending 
		//	             (x0-v0)dot(dir_v01)
		float proj_len = ext_vec.dot(dir_v01);

		float v01_len = (v0-v1).norm();

		float ex_scale = (proj_len+v01_len)/v01_len;

		// use v2 as second extending vertex
		mBox_EdgeScale( resParBox , ex_scale , minVIdx , e_v2_idx , FR2WORLD );
	}else{
		//no need extending

	}


	if( cos_v02 > 0 ){
		//require extending  
		//               (x0-v0)dot(dir_v02)
		float proj_len = ext_vec.dot(dir_v02); 
		float v2_len = (v0-v2).norm(); 
		float ex_scale = (proj_len+v2_len)/v2_len;

		// use v2 as second extending vertex
		mBox_EdgeScale( resParBox , ex_scale , minVIdx , e_v1_idx , FR2WORLD );
	}else{
		//no need extending
	}

	return;
}


// index start from 0
// assume use top face --> v_idx = [0~3]
void mBox_enforceParentSupportV2( BoxDataV2 & resParBox , BoxDataV2 const & tarChild , Eigen::Matrix4f const & FR2WORLD  )
{    
	Eigen::MatrixXf par_obb_fr= Eigen::Map<Eigen::MatrixXf>( resParBox.getBox3d_fr() ,3,8);

	par_obb_fr.transposeInPlace(); 
	//=========================================================================================================
	// find a farest points
	Eigen::Vector3f par_cen = par_obb_fr.colwise().mean();

	Eigen::MatrixXf ch_boxfr_n3=tarChild.getBox3d_fr_n3();
	Eigen::Vector3f ch_cen = ch_boxfr_n3.colwise().mean();

	float maxDist=-1;
	int maxVIDX=0;

	for( int iv=0 ; iv < 4 ; iv++ ){
		Eigen::Vector3f a_vex=ch_boxfr_n3.row(iv);
		float dist = (a_vex-par_cen).norm();

		if( dist > maxDist ){
			maxVIDX=iv;
			maxDist=dist;
		} 
	}

	Eigen::Vector3f tarVex = ch_boxfr_n3.row(maxVIDX);

	//=========================================================================================================

	// if the tarVex is already inside the parent
	// skip

	BGPolygon u_fxz; 

	for( int iv=0 ; iv<4 ; iv++ ){
		u_fxz.outer().push_back(BGPoint(par_obb_fr(iv,0),par_obb_fr(iv,2)));  
	}

	BGPolygon u_hxz;
	boost::geometry::convex_hull(u_fxz, u_hxz);


	bool massIsInside = boost::geometry::covered_by(BGPoint(tarVex(0),tarVex(2)) , u_hxz );

	if( massIsInside ){
		return;
	}

	//=========================================================================================================

	// found a closest vertex 

	float minDist=std::numeric_limits<float>::infinity();
	int minVIdx=-1;
	for( int iv=0 ; iv<4 ; iv++ ){
		Eigen::Vector3f vex=par_obb_fr.row(iv);
		float vdist = (vex-tarVex).norm();

		if( vdist < minDist ){
			minDist=vdist;
			minVIdx=iv;
		}
	} 

	// projection direction
	int e_v1_idx = (minVIdx+1)%4;
	int e_v2_idx = (minVIdx-1)%4;

	if( e_v2_idx <0 ){
		e_v2_idx=e_v2_idx+4;
	}

	// e_v1_idx,minVIdx,e_v2_idx
	Eigen::Vector3f v0 = par_obb_fr.row(minVIdx);
	Eigen::Vector3f v1 = par_obb_fr.row(e_v1_idx);
	Eigen::Vector3f v2 = par_obb_fr.row(e_v2_idx);

	Eigen::Vector3f tarCx = tarVex;

	tarCx.y()=0;
	v0.y()=0;
	v1.y()=0;
	v2.y()=0;

	Eigen::Vector3f ext_vec = tarCx-v0;
	Eigen::Vector3f ext_dir = tarCx-v0;
	ext_dir.normalize();

	Eigen::Vector3f dir_v01 = v0-v1;
	Eigen::Vector3f dir_v02 = v0-v2;

	dir_v01.normalize();
	dir_v02.normalize();

	float cos_v01 = ext_dir.dot(dir_v01);
	float cos_v02 = ext_dir.dot(dir_v02);


	if( cos_v01 > 0 ){
		//require extending 
		//	             (x0-v0)dot(dir_v01)
		float proj_len = ext_vec.dot(dir_v01);

		float v01_len = (v0-v1).norm();

		float ex_scale = (proj_len+v01_len)/v01_len;

		// use v2 as second extending vertex
		mBox_EdgeScale( resParBox , ex_scale , minVIdx , e_v2_idx , FR2WORLD );
	}else{
		//no need extending

	}


	if( cos_v02 > 0 ){
		//require extending  
		//               (x0-v0)dot(dir_v02)
		float proj_len = ext_vec.dot(dir_v02); 
		float v2_len = (v0-v2).norm(); 
		float ex_scale = (proj_len+v2_len)/v2_len;

		// use v2 as second extending vertex
		mBox_EdgeScale( resParBox , ex_scale , minVIdx , e_v1_idx , FR2WORLD );
	}else{
		//no need extending
	}

	return;
}


// enforce child support 
// extending bot-Y
// use bottom face of the vertex
void mBox_enforceChildSupTouch( BoxDataV2 const & PARENT_BOX , BoxDataV2 & tarChild ,   Eigen::Matrix4f const & FR2WORLD )
{ 
	// handle X and Z axis extension

	// start from aabb
	Eigen::MatrixXf child_aabb= Eigen::Map<Eigen::MatrixXf>( tarChild.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf child_obb_fr= Eigen::Map<Eigen::MatrixXf>( tarChild.getBox3d_fr() ,3,8);
	Eigen::MatrixXf child_obb_world=Eigen::Map<Eigen::MatrixXf>( tarChild.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( tarChild.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( tarChild.getAABB2FrOBB() ,3,3);

	child_aabb.transposeInPlace();
	child_obb_fr.transposeInPlace();
	child_obb_world.transposeInPlace();
	//==================================================================================

	Eigen::MatrixXf par_frn3 = PARENT_BOX.getBox3d_fr_n3();

	float parHeight = par_frn3.col(2-1).maxCoeff();

	//==================================================================================
 
	float child_lowY = child_obb_fr.col(2-1).minCoeff();

	float signDist = child_lowY-parHeight;
	
	int ch_top_v = 0; 
	int ch_bot_v = 2 + 4;
	 
	// re-extract boxes
	Eigen::RowVector3f newVmax = child_obb_fr.row(ch_top_v);
	Eigen::RowVector3f newVmin = child_obb_fr.row(ch_bot_v);

	newVmin.y() = newVmin.y() - signDist;

	// MaxMin should only work in AABB domain

	Eigen::MatrixXf redOBBfr ;
	redOBBfr.resize(2,3);
	redOBBfr.row(0)=newVmax;
	redOBBfr.row(1)=newVmin;

	Eigen::RowVector3f newFRCen = redOBBfr.colwise().mean();
	// use new fr center
	Eigen::MatrixXf red_AABB = cenTx( newFRCen , obbFr2AABB, redOBBfr);
	Eigen::RowVector3f nmax = red_AABB.colwise().maxCoeff();
	Eigen::RowVector3f nmin = red_AABB.colwise().minCoeff();

	Eigen::MatrixXf newAABB = getBoxByMaxMin_row( nmax , nmin );

	//-------------------------------------------------------------------------  
	// update boxes 
	Eigen::MatrixXf newOBBFr = cenTx( newFRCen , aabb2ObbFr, newAABB);
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBFr);


	tarChild.setBox3d_aabb(newAABB);
	tarChild.setBox3d_fr(newOBBFr);
	tarChild.setBox3d_world(newOBBWorld);

}


float pointPlaneABSDist(Eigen::Vector3f x0 , PlaneData const PLANE )
{

	Eigen::Vector3f nv = PLANE.model.head(3).normalized();

	Eigen::Vector3f x0p0 =PLANE.pts[0]- x0;

	bool sameSide =nv.dot(x0p0.normalized()) < 0;

	float dist =0;

	if( !sameSide ){
		dist = (-1*nv).dot(x0p0);
	}else{
		dist =  nv.dot(x0p0);
	}

	return abs(dist);
}

// return 1xN dist array
Eigen::MatrixXf pointPlaneABSDist_pcd(Eigen::MatrixXf & pcd3N , PlaneData const PLANE )
{
	
	Eigen::Vector3f nv = PLANE.model.head(3).normalized();

	//Eigen::Vector3f x0p0 =PLANE.pts[0]- x0;

	Eigen::MatrixXf pcd3N_x0mp0 = (-1*pcd3N).colwise()+PLANE.pts[0];

	Eigen::MatrixXf dotmat;
	dotmat.resize(1,3);

	dotmat.setZero();
	dotmat(0,0)=nv(0);
	dotmat(0,1)=nv(1);
	dotmat(0,2)=nv(2);
	
	//dist = nv.dot(x0p0);
	// 1xN
	dotmat=dotmat*pcd3N_x0mp0;

	dotmat=dotmat.array().abs();

	return dotmat;
}

// return the dist that
// x0 + DIST*PLANE_NV = x_on_plane
float pointPlaneSignDist(Eigen::Vector3f x0 , PlaneData const PLANE )
{
	Eigen::Vector3f nv = PLANE.model.head(3).normalized();

	Eigen::Vector3f x0p0 =PLANE.pts[0]- x0;

	bool sameSide =nv.dot(x0p0.normalized()) < 0;

	float dist = abs(nv.dot(x0p0));

	if( sameSide ){
		dist = -1 * dist;
	}else{
		//;
	}

	return dist;
}

float box2PlaneDist_fr(BoxDataV2 const & BOX , PlaneData const & PLANE)
{ 
	const float* vtxs = BOX.getBox3d_fr();

	float minDist=std::numeric_limits<float>::infinity();
	float meanDist=0;
	std::vector<float> alldist;
	for (int iv=0 ; iv < BOX.BOXVEX_NUM ; iv++ )
	{
		Eigen::Vector3f p0(vtxs[iv*3],vtxs[iv*3+1],vtxs[iv*3+2]);

		float dist =  pointPlaneABSDist(p0,PLANE);
		meanDist=meanDist+dist;
		alldist.push_back(dist);
		if (dist < minDist )
		{
			minDist=dist;
		}
	}

	meanDist=meanDist/BOX.BOXVEX_NUM;

	std::sort(alldist.begin(),alldist.end());

	return (alldist[0]+alldist[1]+alldist[2]+alldist[3])/4;
}

float topfacePlaneDist_fr(BoxDataV2 const & BOX , PlaneData const & PLANE, int & v1_idx , int & v2_idx )
{ 
	const float* vtxs = BOX.getBox3d_fr();

	std::vector<float> alldist;

	float minDist=std::numeric_limits<float>::infinity();
	int min_idx=-1;

	float secMin=std::numeric_limits<float>::infinity();
	int sec_idx=-1;
	for (int iv=0 ; iv < 4 ; iv++ )
	{
		Eigen::Vector3f p0(vtxs[iv*3],vtxs[iv*3+1],vtxs[iv*3+2]);

		float dist =  pointPlaneABSDist(p0,PLANE);
		 
		alldist.push_back(dist);

		if (dist < minDist )
		{
			minDist=dist;
			min_idx=iv;
		}else if( dist < secMin ){
			secMin=dist;
			sec_idx=iv;
		}
	}

	v1_idx=min_idx;
	v2_idx=sec_idx;

	return (minDist);
}

 
void printQIMGType( QImage & img )
{
	if( !img.isNull() ){

		switch (img.format())
		{
		case QImage::Format_ARGB32:
			std::cout<<" Format_ARGB32 \n";
			break;
		case QImage::Format_Indexed8:
			std::cout<<" Format_Indexed8 \n";
			break;
		case QImage::Format_Mono:
			std::cout<<" Format_Mono \n";
			break;
		case QImage::Format_RGBA8888:
			std::cout<<" Format_RGBA8888 \n";
			break;
		case QImage::Format_Invalid:
			std::cout<<" Format_Invalid \n";
			break;		
		case QImage::Format_RGB888:
			std::cout<<" Format_RGB888 \n";
			break;
		default:
			std::cout<<" Format ="<<img.format()<<" \n";
			break;
		}


	}else{

		std::cout<<"IMG is NULL\n";
	}
}


//
void maniBoxVex(BoxDataV2 & resBox, const int MANI_VEXIDX , Eigen::Vector3f newVex_world , 
				Eigen::Matrix4f const & WORLD2FR,
				Eigen::Matrix4f const & FR2WORLD )
{
	bool verbase=0;

	// handle X and Z axis extension

	// start from aabb
	Eigen::MatrixXf aabb= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( resBox.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( resBox.getAABB2FrOBB() ,3,3);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();

	
	//----------------------------------------------------------------------------
	//[FR_SPACE] calculate point to plane distance 
	// we only test top face vertices
	 
	// get NEW FR vertex
	Eigen::Vector3f extv_fr =  (WORLD2FR*newVex_world.homogeneous()).head(3);
	 
	//-------------------------------------------------------------------------
	// select the diagonal to extending the min/max pair
	// 0,1,2,3
	// 4,5,6,7
	// v2 should be the bottom face vertex

	Eigen::RowVector3f newVmax , newVmin;

	if (MANI_VEXIDX<4)
	{
		int v2Idx = (MANI_VEXIDX+2)%4+4;

		// re-extract boxes
		newVmax = extv_fr;
		newVmin = obb_fr.row(v2Idx);
		
	}else{
		int v1Idx = (MANI_VEXIDX-4+2)%4;

		// re-extract boxes
		newVmax = obb_fr.row(v1Idx);
		newVmin = extv_fr;
	} 

	// MaxMin should only work in AABB domain
	Eigen::MatrixXf simpleOBBfr ;

	simpleOBBfr.resize(2,3);
	simpleOBBfr.row(0)=newVmax;
	simpleOBBfr.row(1)=newVmin;


	Eigen::RowVector3f newFRCen = simpleOBBfr.colwise().mean();
	
	// use new fr center
	// to get correct aabb which lie on new center location.

	Eigen::MatrixXf simple_AABB = cenTx( newFRCen , obbFr2AABB, simpleOBBfr);
	Eigen::RowVector3f nmax = simple_AABB.colwise().maxCoeff();
	Eigen::RowVector3f nmin = simple_AABB.colwise().minCoeff();

	// get the new aabb
	Eigen::MatrixXf newAABB = getBoxByMaxMin_row( nmax , nmin );

	//-------------------------------------------------------------------------  
	// update boxes 
	Eigen::MatrixXf newOBBFr = cenTx( newFRCen , aabb2ObbFr, newAABB);
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBFr);

	resBox.setBox3d_aabb(newAABB);
	resBox.setBox3d_fr(newOBBFr);
	resBox.setBox3d_world(newOBBWorld);

}


// for changing height
void maniBoxVex_height(BoxDataV2 & resBox, const int MANI_VEXIDX ,
				float const Y_DX,
				Eigen::Matrix4f const & WORLD2FR,
				Eigen::Matrix4f const & FR2WORLD )
{
	bool verbase=0;

	// handle X and Z axis extension

	// start from aabb
	Eigen::MatrixXf aabb= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( resBox.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( resBox.getAABB2FrOBB() ,3,3);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();


	//----------------------------------------------------------------------------
	//[FR_SPACE] calculate point to plane distance 
	// we only test top face vertices

	// get NEW FR vertex

	Eigen::Vector3f maniVex_w = obb_world.row(MANI_VEXIDX).transpose();

	Eigen::Vector3f extv_fr =  (WORLD2FR*maniVex_w.homogeneous()).head(3);

	extv_fr.y() = extv_fr.y() + Y_DX;

	//-------------------------------------------------------------------------
	// select the diagonal to extending the min/max pair
	// 0,1,2,3
	// 4,5,6,7
	// v2 should be the bottom face vertex

	Eigen::RowVector3f newVmax , newVmin;

	if (MANI_VEXIDX<4)
	{
		int v2Idx = (MANI_VEXIDX+2)%4+4;

		// re-extract boxes
		newVmax = extv_fr;
		newVmin = obb_fr.row(v2Idx);

	}else{
		int v1Idx = (MANI_VEXIDX-4+2)%4;

		// re-extract boxes
		newVmax = obb_fr.row(v1Idx);
		newVmin = extv_fr;
	} 

	// MaxMin should only work in AABB domain
	Eigen::MatrixXf simpleOBBfr ;

	simpleOBBfr.resize(2,3);
	simpleOBBfr.row(0)=newVmax;
	simpleOBBfr.row(1)=newVmin;


	Eigen::RowVector3f newFRCen = simpleOBBfr.colwise().mean();

	// use new fr center
	// to get correct aabb which lie on new center location.

	Eigen::MatrixXf simple_AABB = cenTx( newFRCen , obbFr2AABB, simpleOBBfr);
	Eigen::RowVector3f nmax = simple_AABB.colwise().maxCoeff();
	Eigen::RowVector3f nmin = simple_AABB.colwise().minCoeff();

	// get the new aabb
	Eigen::MatrixXf newAABB = getBoxByMaxMin_row( nmax , nmin );

	//-------------------------------------------------------------------------  
	// update boxes 
	Eigen::MatrixXf newOBBFr = cenTx( newFRCen , aabb2ObbFr, newAABB);
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBFr);

	resBox.setBox3d_aabb(newAABB);
	resBox.setBox3d_fr(newOBBFr);
	resBox.setBox3d_world(newOBBWorld);

}




void mBoxTranslate( BoxDataV2 & resBox, Eigen::RowVector3f cen_move , Eigen::Matrix4f const & FR2WORLD )
{
	// handle X and Z axis extension

	bool verbase=0;

	// start from aabb
	Eigen::MatrixXf aabb= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( resBox.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( resBox.getAABB2FrOBB() ,3,3);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();

	//=========================================================================================================
	// update boxes   
	
	// obb_fr
	Eigen::MatrixXf newOBBFr = obb_fr;
	newOBBFr = newOBBFr.rowwise() + cen_move;

	Eigen::RowVector3f new_frCen = newOBBFr.colwise().mean(); 

	// THE center of aabb would BE changed
	Eigen::MatrixXf newAABB = cenTx(new_frCen , obbFr2AABB ,newOBBFr);

	// world box would be updated 
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBFr);


	if(verbase)
	{
		Eigen::RowVector3f frCen = obb_fr.colwise().mean(); 

		std::cout<<"obb_fr:"<<obb_fr<<std::endl; 
		std::cout<<"newOBBFr:"<<newOBBFr<<std::endl;
		 

		std::cout<<"frCen:"<<frCen<<std::endl; 
		std::cout<<"new frCen:"<<new_frCen<<std::endl; 

		std::cout<<"cen tx:"<< cen_move <<std::endl; 
		std::cout<<"new cen diff:"<< new_frCen-frCen <<std::endl; 

		//*
		Eigen::MatrixXf de_obb = cenTx( new_frCen , aabb2ObbFr, newAABB );

		std::cout<<"------------------\n"
			     <<"new obbfr:\n\t"<<newOBBFr<<"\n"
				 <<"rt obbfr:\n\t"<<de_obb<<"\n\n"
  				 <<"new-retx OBB sum="<<(newOBBFr-de_obb).array().sum()<<"\n" ;
		 

		Eigen::MatrixXf rt_aabb = cenTx( new_frCen , obbFr2AABB, newOBBFr);
		std::cout<<"------------------\n"
				 <<"new-retx aabb sum="<<(newAABB-rt_aabb).array().sum()<<"\n" ;

		//*/
	}

	//=========================================================================================================  
	// SAVE
	// center of aabb is updated

	resBox.setBox3d_aabb(newAABB);
	resBox.setBox3d_fr(newOBBFr);
	resBox.setBox3d_world(newOBBWorld);
	

}

void checkBox( BoxDataV2 const & ABOX , Eigen::Matrix4f const & FR2WORLD )
{
	// handle X and Z axis extension

	bool verbase=0;

	// start from aabb
	Eigen::MatrixXf aabb= Eigen::Map<const Eigen::MatrixXf>( ABOX.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map<const Eigen::MatrixXf>( ABOX.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<const Eigen::MatrixXf>( ABOX.getBox3d_world() ,3,8);

	Eigen::Map<const Eigen::MatrixXf> obbFr2AABB( ABOX.getFrOBB2AABB() ,3,3);
	Eigen::Map<const Eigen::MatrixXf> aabb2ObbFr( ABOX.getAABB2FrOBB() ,3,3);

	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();

	//=========================================================================================================
	// update boxes   
	
	// obb_fr
	Eigen::MatrixXf newOBBFr = obb_fr;

	Eigen::RowVector3f frCen = obb_fr.colwise().mean(); 

	// THE center of aabb would BE changed
	Eigen::MatrixXf newAABB = cenTx(frCen , obbFr2AABB ,newOBBFr);

	Eigen::MatrixXf newFr = cenTx(frCen , aabb2ObbFr ,aabb);

	// world box would be updated 
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,obb_fr);

	 
	{ 
		//* 

		std::cout<<"------------------\n"
			<<"diff OBB_WORLD sum="<<(newOBBWorld-obb_world).array().abs().sum()<<"\n" 
			<<"diff AABB sum="<<(newAABB-aabb).array().abs().sum()<<"\n" 
			<<"diff FR sum="<<(newFr-obb_fr).array().abs().sum()<<"\n" 
			<<"tx is zero="<<(obbFr2AABB*aabb2ObbFr).array().abs().sum()<<"\n" 
			<<"mx=\n"<<(obbFr2AABB*aabb2ObbFr)<<"\n\n"
			<<"fr2local=\n"<<(obbFr2AABB)<<"\n\n"
			<<"local2fr=\n"<<(aabb2ObbFr)<<"\n";

		cout<<"------------------\n";
			 

		//*/
	}
	 


}




//
void maniBoxVex_Extend(BoxDataV2 & resBox, const int MANI_VEXIDX , Eigen::Vector3f const MOVE_VEC , 
				Eigen::Matrix4f const & WORLD2FR,
				Eigen::Matrix4f const & FR2WORLD )
{
	bool verbase=0;

	// handle X and Z axis extension

	// start from aabb
	Eigen::MatrixXf aabb= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_AABB() ,3,8);
	Eigen::MatrixXf obb_fr= Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_fr() ,3,8);
	Eigen::MatrixXf obb_world=Eigen::Map<Eigen::MatrixXf>( resBox.getBox3d_world() ,3,8);

	Eigen::Map<Eigen::MatrixXf> obbFr2AABB( resBox.getFrOBB2AABB() ,3,3);
	Eigen::Map<Eigen::MatrixXf> aabb2ObbFr( resBox.getAABB2FrOBB() ,3,3);

	// 8x3
	aabb.transposeInPlace();
	obb_fr.transposeInPlace();
	obb_world.transposeInPlace();


	//----------------------------------------------------------------------------
	//[FR_SPACE] calculate point to plane distance 
	// we only test top face vertices

	int v1Idx = MANI_VEXIDX;
	
	Eigen::Vector3f newVex_world= obb_world.row(v1Idx).transpose()+MOVE_VEC;

	// get NEW FR vertex
	Eigen::Vector3f extv_fr =  (WORLD2FR*newVex_world.homogeneous()).head(3);

	//-------------------------------------------------------------------------
	// select the diagonal to extending the min/max pair
	// 0,1,2,3
	// 4,5,6,7
	// v2 should be the bottom face vertex
	int v2Idx = (v1Idx+2)%4+4;

	// re-extract boxes
	Eigen::RowVector3f newVmax = extv_fr;
	Eigen::RowVector3f newVmin = obb_fr.row(v2Idx);


	// MaxMin should only work in AABB domain
	Eigen::MatrixXf simpleOBBfr ;
	simpleOBBfr.resize(2,3);
	simpleOBBfr.row(0)=newVmax;
	simpleOBBfr.row(1)=newVmin;

	Eigen::RowVector3f newFRCen = simpleOBBfr.colwise().mean();


	// use new fr center
	// to get correct aabb which lie on new center location.

	Eigen::MatrixXf simple_AABB = cenTx( newFRCen , obbFr2AABB, simpleOBBfr);
	Eigen::RowVector3f nmax = simple_AABB.colwise().maxCoeff();
	Eigen::RowVector3f nmin = simple_AABB.colwise().minCoeff();

	// get the new aabb
	Eigen::MatrixXf newAABB = getBoxByMaxMin_row( nmax , nmin );

	//-------------------------------------------------------------------------  
	// update boxes 
	Eigen::MatrixXf newOBBFr = cenTx( newFRCen , aabb2ObbFr, newAABB);
	Eigen::MatrixXf newOBBWorld = cenTx44(Eigen::RowVector3f::Zero() , FR2WORLD ,newOBBFr);

	resBox.setBox3d_aabb(newAABB);
	resBox.setBox3d_fr(newOBBFr);
	resBox.setBox3d_world(newOBBWorld);

}







