#pragma once


#include "boxdatav2.h"
#include "common_tool.h"
#include "geoobj.h"

#include <pcl/io/pcd_io.h>
#include <Eigen/Eigen>
#include <QImage>

#include "geo_typedef.h"

#include <pcl/sample_consensus/msac.h>
#include <pcl/sample_consensus/mlesac.h>
#include <pcl/sample_consensus/prosac.h>
#include <pcl/sample_consensus/lmeds.h>

void knnFilter( pcl::PointCloud<pcl::PointXYZ>::ConstPtr PCD_IN , std::vector<int> * pcd_index,
			   std::vector<int> & inlIdx , double medianRatio=3.0,int const NUM_K=30);

void copyViaIndex(std::vector<int> const & VIN , std::vector<int> const& FLAG, std::vector<int> &vout );
  
void exbox(const int TID , cv::Mat const & indImg , 
		   PointCloudXYZ::Ptr f_xyzPcd , PointCloudXYZ::Ptr f_nvPcd , 
		   Eigen::MatrixXf const&world2frMat,
		   BoxDataV2 & res);


//check direction of two vector
bool sameDirC0(Eigen::Vector3f v1 , Eigen::Vector3f v2 );
 

Eigen::MatrixX3f getBoxByMaxMin(Eigen::Vector3f bmax , Eigen::Vector3f bmin);
Eigen::MatrixX3f getBoxByMaxMin_row( Eigen::RowVector3f bmax , Eigen::RowVector3f bmin );
Eigen::MatrixXf cenTx(Eigen::RowVector3f cen ,Eigen::MatrixXf const & TX , Eigen::MatrixXf pts  );

 
float planeRANSAC( const int TID , cv::Mat const & indImg, PointCloudXYZ::Ptr f_xyzPcd , 
				 Eigen::Vector3f const & REF_CAM0 , PlaneData & res  );

Eigen::Vector3f findAP0(Eigen::VectorXf const & PL_MODEL );

Eigen::MatrixXf rotateV2V(Eigen::Vector3f srcVec , Eigen::Vector3f dstVec );

void estimateNE( pcl::PointCloud< pcl::PointXYZ >::Ptr cloud_in, pcl::PointCloud< pcl::PointXYZ >::Ptr cloud_out ,
				float ne_maxDepthChange=0.2, float ne_normal_smoothSize=10 );

template<typename T>
void loadPCDAndCheck(const char* CPATH , pcl::PointCloud< T > & pcd ){

	if( !fileisExisted(CPATH) ){
		std::cout<<"CANNOT FOUND PCD:"<<CPATH<<std::endl;
		throw "pcd not exist";
	}

	pcl::io::loadPCDFile( CPATH , pcd);
	std::cout<<"LOAD PCD:["<<CPATH<<"]\n";
}


void extr_obb( Eigen::MatrixXf const   BASIS , PointCloudXYZ::Ptr tarPcd, Eigen::Matrix4f const & FR2WORLD ,BoxDataV2 & resBox );

void mBoxScale( BoxDataV2 & resBox, Eigen::Matrix4f const & FR2WORLD , Eigen::RowVector3f scaleVec );

float pointPlaneABSDist(Eigen::Vector3f x0 , PlaneData const PLANE );

void mBoxEntend2Plane_Side(BoxDataV2 & resBox, PlaneData const & PLANE_FR , Eigen::Matrix4f const & FR2WORLD );

void mBoxEntend2Plane_BotVex(BoxDataV2 & resBox, PlaneData const & PLANE_FR , Eigen::Matrix4f const & FR2WORLD );

bool ptsIsOnPlane(Eigen::Vector3f const & v0 , Eigen::Vector4f const &  model);

float box2PlaneDist_fr(BoxDataV2 const & BOX , PlaneData const & PLANE);


void printQIMGType(QImage & img);

float pointPlaneSignDist(Eigen::Vector3f x0 , PlaneData const PLANE );

void planeRANSAC_byXZLine( const int TID , cv::Mat const & indImg, PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0, PlaneData & res );

Eigen::MatrixXf pointPlaneABSDist_pcd(Eigen::MatrixXf & pcd3N , PlaneData const PLANE );


void mBoxRotate_Y(BoxDataV2 & resBox, double y_theta , Eigen::Matrix4f const & FR2WORLD);

void mBox_EdgeScale( BoxDataV2 & resBox , float scale , int v1_idx , int v2_idx , Eigen::Matrix4f const & FR2WORLD );

void mBox_EdgeScale( BoxDataV2 const& BOX_IN , float scale , int v1_idx , int v2_idx , Eigen::Matrix4f const & FR2WORLD ,BoxDataV2 & resBox );

void mBoxRotate_mat( BoxDataV2 & resBox, Eigen::Matrix3f const & R , Eigen::Matrix4f const & FR2WORLD );

void mBox_Align2Wall(BoxDataV2 & resBox,PlaneData const & WALL, Eigen::Matrix4f const & FR2WORLD );

void mBox_enforceParentSupport( BoxDataV2 & resParBox , BoxDataV2 const & tarChild , Eigen::Matrix4f const & FR2WORLD );

void mBox_enforceChildSupTouch( BoxDataV2 const & PARENT_BOX , BoxDataV2 & tarChild , Eigen::Matrix4f const & FR2WORLD );

float planeRANSAC_byXZLine_room ( const int TID , cv::Mat const & indImg, PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0, PlaneData & res );

void planeRANSAC_floor( const int TID , cv::Mat const & indImg, PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0, PlaneData & res );

float topfacePlaneDist_fr(BoxDataV2 const & BOX , PlaneData const & PLANE, int & v1_idx , int & v2_idx );

void mBox_enforceParentSupportV2( BoxDataV2 & resParBox , BoxDataV2 const & tarChild , Eigen::Matrix4f const & FR2WORLD );

void mBox_Align2Wall_retake(BoxDataV2 & resBox,PlaneData const & WALL, PointCloudXYZ::Ptr f_xyzPcd , PointCloudXYZ::Ptr f_nvPcd , Eigen::MatrixXf const& FR2WORLD );

void exbox_withnv( Eigen::Vector3f nv1, const int TID , cv::Mat const & indImg , PointCloudXYZ::Ptr f_xyzPcd , PointCloudXYZ::Ptr f_nvPcd , Eigen::MatrixXf const& fr2worldMat, BoxDataV2 & resBox );

void maniBoxVex(BoxDataV2 & resBox, const int MANI_VEXIDX , Eigen::Vector3f newVex , Eigen::Matrix4f const & WORLD2FR ,  Eigen::Matrix4f const & FR2WORLD );

void maniBoxVex_height(BoxDataV2 & resBox, const int MANI_VEXIDX , float const Y_DX, 
					   Eigen::Matrix4f const & WORLD2FR,
					   Eigen::Matrix4f const & FR2WORLD );

void mBoxTranslate( BoxDataV2 & resBox, Eigen::RowVector3f cen_move , Eigen::Matrix4f const & FR2WORLD );

void checkBox(BoxDataV2 const & ABOX , Eigen::Matrix4f const & FR2WORLD );


void maniBoxVex_Extend(BoxDataV2 & resBox, const int MANI_VEXIDX , Eigen::Vector3f const MOVE_VEC , Eigen::Matrix4f const & WORLD2FR, Eigen::Matrix4f const & FR2WORLD );

float planeRANSAC_LSE( const int TID , cv::Mat const & indImg, PointCloudXYZ::Ptr f_xyzPcd , Eigen::Vector3f const & REF_CAM0, PlaneData & res );

