#ifndef MY_Q_GL_PANEL
#define MY_Q_GL_PANEL
 
#include <QObject> 
#include <QImage>
#include <QPixmap>
#include <QPainter> 
#include <QQuickPaintedItem>
#include <QtGui/QOpenGLShaderProgram>

#include <memory>
#include <Eigen/Eigen>
#include <functional>
#include <vector> 

#include "boxdata.h"
#include "statemag.h"

#include "mytrackball.h"
//#include "trackball.h"

#ifndef GL_MULTISAMPLE
#define GL_MULTISAMPLE  0x809D
#endif


namespace Tirth{

	class QGLPanel :public QQuickPaintedItem
	{

		Q_OBJECT

		Q_PROPERTY(int imgID READ getImgID WRITE setImgID )
		Q_PROPERTY(int brushSize READ getBrushSize WRITE setBrushSize )
		Q_PROPERTY( QString stateStr READ getCurStateStr NOTIFY stateStrChange )  
		Q_PROPERTY( QList<QObject*> boxData READ getBoxData NOTIFY boxDataChanged )

	public: 

		//EIGEN_MAKE_ALIGNED_OPERATOR_NEW
		//-------------------------------------------------------------------------	 
		QGLPanel(QQuickItem  * parent=0);
		~QGLPanel();
		 
		virtual void paint(QPainter * painter);

		void setImgID(int imid);
        int getImgID(void);
		
		QList<QObject*> getBoxData();

		inline
		int getBrushSize(){ return ctx.spBrushSize;}
		
		inline
		void setBrushSize(int newSize){ ctx.spBrushSize=newSize;}

		inline
		StateMag * getCtx(){ return &ctx; }
		inline
		QString getCurStateStr(){return QString::fromLocal8Bit(Tirth::MODESTRS[ctx.currentState]);}
		inline
		QPoint getLastMPos(){ return ctx.lastPos ;} 


	// SLOTS ====================================================================
	public slots :
		void openImgSlot();
		void drawLineModeSlot();
		void idleModeSlot();
		void debugTestSlot();

		void roomModeSlot();
		void actionDoneSlot();

		void cutSPModeSlot(); 
		void extrBoxModeSlot();
		void filterSPModeSlot();		

		void predictModeSlot();
		void rePredBtuSlot();

		void delBoxSlot();
		void runBtuSlot();

		void showBoxCHBSlot(bool );
		void showPredCHBSlot(bool );
		void showSegCHBSlot(bool );

		void saveMatStateSlot();
		void loadMatStateSlot();

		void refineBoxSlot();
		void finalSavBtuSlot();
		 
		void confAllBtuSlot();

		// mouse Events
		void mousePressEvent(int x, int y, int btu);
		void mouseReleaseEvent(int x, int y);
		void mouseMoveEvent(int x, int y);
		void wheelEvent(); 
		void mouseEnter();
		void mouseLeave();

		//update UI DATA
		void imidValueChanged(int);

		// binding debug
		void boxDataChangedSlot();

	signals:  
		void tagIsReady();

		void stateStrChange(QString);
		
		void boxDataChanged();

		void requestClearLabel();

    // PRIVATE ====================================================================

	private slots:
		void handleWindowChanged(QQuickWindow *win);
		// 3d rendering state sync
		void stateSYNC_3drendering();
		void paint_ogl_bef();
		void paint_ogl_aft();
		void ogl_cleanup();
		void renderBox();

	private: 
		//------------------------------------------------------------------------- 
		QOpenGLShaderProgram *m_program;

		//-------------------------------------------------------------------------
		// logic
		void dispatch( StateMag::State fromS1 , StateMag::UISignal usignal );

		// update - must return State
		StateMag::State openImage();
		StateMag::State openImageByIMID();
		StateMag::State reopenImageByIMID();

		// cut-sp draw line
		StateMag::State lineTool_mPress();
		StateMag::State lineTool_mMove();
		StateMag::State lineTool_mRelease();
		StateMag::State lineToolActDone();
		StateMag::State lineToolActDone_gt();

		StateMag::State testSave();

		// cut sp filter
		StateMag::State filterSPbyNum(); 
		StateMag::State filterSPbyNum_gt(); 

		// room 
		StateMag::State room_mRelease();
		StateMag::State room_isDone();
		StateMag::State room_isDone_gt();

		// Extract boxes
		StateMag::State exbox_mMove();
		StateMag::State exbox_mRelease();
		StateMag::State exbox_mRelease_gt(); 
		StateMag::State exbox_mRelease_gt_cut();
		StateMag::State exbox_actionDone();

		// Refine box
		StateMag::State refbox_switch2RefBox();
		StateMag::State refbox_doref();
		StateMag::State refbox_mPress();
		StateMag::State refbox_mMove();
		StateMag::State refbox_mRelease();
		
		// SWITCHES
		StateMag::State sp_switch2Line();
		StateMag::State roomMode_switch2Room();		
		StateMag::State cutSP_switch2CutSP();			
		StateMag::State cutSP_switch2CutSP_gt();			
		StateMag::State box_switch2ExBox();			

		// PRED
		StateMag::State predict_switch2PD(); 	
		StateMag::State predict_rePred(); 	
		StateMag::State predict_actionDone(); 	
		StateMag::State predict_confirmAll();

		StateMag::State predict_mPress();
		StateMag::State predict_mMove();
		StateMag::State predict_mRelease();

		// CONFR ALL
		StateMag::State final_actionDone(); 	
		StateMag::State final_saveFinalLabel();

		void getBoxLIDStuff(); 
		void getBox3DVerts();

		// DEFAULT ACTION
		StateMag::State simpleActionDone();

		void inner_predict(Eigen::MatrixXi& bcenter ,Eigen::MatrixXi& preBoxLid , Eigen::MatrixXd& gCost ,Eigen::MatrixXi& lStrs);

		//-------------------------------------------------------------------------
		// [RESET]
		void resetStateCtx(StateMag&); 
		 
		//-------------------------------------------------------------------------
		// misc
	private:

		void iniDebugUIBox();
		void warnDialog(const char* MSG)const; 
		bool YNDialog( const char* MSG)const;
		void debugPrint(const char* msg, const int errlevel=0 )const;
		
		void setAlpha( QImage& img , int rgbFlag ,int alphaValk );

		void drawBrushOnImage( QPixmap& , int x,int y,int rad);
		void drawBrushOnImage( QImage& , int x,int y,int rad);
		
		QImage cvmat2qimage(cv::Mat const& src);		
		cv::Mat qimage2cvmat(QImage const& src);

		void bindCvmat2qimage(cv::Mat & src , QImage & qimg);		
		void bindQImage2cvmat(QImage & src , cv::Mat & cvimg);

		//-------------------------------------------------------------------------
		// Draw Functions
		void drawBackground(QPainter *painter);
		void drawBackground_mixedogl(QPainter *painter);
		
		void drawUI(QPainter&);
		void drawCutLine( QPainter *painter );
		void drawLegend(QPainter *painter); 
		
		double getDepth( Eigen::Vector3d const &   );		
		Eigen::Vector2i getImg2dPts( Eigen::Vector3d const & );

		//void loadMask( cv::Mat& , QImage& qimg , const char* varName  ); 
		void loadByteImg( QImage& qimg , const char* varName , QImage::Format );  
		void loadByteU8Mask( QImage& qimg , const char* varName   ); 
		void sendQImg( QImage const& qimg, const char* ,const char* POST_CMD="" );


	private:
		// DATA member 
		//-------------------------------------------------------------------------
		// State Control
		// next_state func(void) 
		typedef std::function< StateMag::State(QGLPanel&) > UpdateFunc;
		typedef std::function< void( QGLPanel&) > TransFunc;
		//typedef Eigen::Map<Eigen::Matrix<unsigned char,Eigen::Dynamic,Eigen::Dynamic>, 0, Eigen::InnerStride<4> > ESlice;

		Tirth::StateMag ctx;

		// state X signals 2D table
		UpdateFunc updateTable[StateMag::NUM_OF_STATE][StateMag::NUM_OF_USIG];

		// state X state 2D table
		// direct use switching functidebugPrinton
		//TransFunc transTable[StateMag::NUM_OF_STATE][StateMag::NUM_OF_STATE];
		//-------------------------------------------------------------------------
		 
		MyTrackBall tball;
		
		//TrackBall tb1;
		//TrackBall tb2;
		//TrackBall tb3;
		//-------------------------------------------------------------------------
		// debug stuff
		QImage debugImg;

		//-------------------------------------------------------------------------
		//misc		 
		std::unique_ptr<int []> viewportMat;
		std::unique_ptr<double>mv;
		std::unique_ptr<double>proj; 
	};

}//namespace




#endif
