#pragma once

#include <vector>
#include <string>
#include <Eigen/Eigen>
#include "boxdata.h"
#include <map>
#include <opencv2/core/core.hpp>

namespace Tirth{

	// A simple structure to hold the state variables
	// NO METHODS
	struct StateMag
	{
	public:

		EIGEN_MAKE_ALIGNED_OPERATOR_NEW

		//-----------------------------------
		static const int NUM_OF_STATE = 11;
		static const int NUM_OF_USIG = 18;
		//-----------------------------------
		// NOTE that also update STATE string in cpp
		enum State{
			S_INI=0,
			S_IDLE,
			S_CUTS_DRAWLINE,
			S_SELECT,
			S_ROOM,   

			S_CUTSP,
			S_EXBOX,
			S_PREDICT,
			S_REFINE_BOX,

			S_FINAL
		};

		enum UISignal{
			U_MPress=0,
			U_MMove,
			U_MRelease,
			U_BtuOpenImg,
			U_BtuSelectTool, 

			U_BtuLineTool,
			U_BtuRoomMode,
			U_BtuActionDone, 
			U_BtuCutSPMode,
			U_BtuExtrBoxesMode,

			U_BtuPredictMode,
			U_BtuFIlterSPMode,

			U_BtuRePredBtu, 
			U_BtuDelBtu,
			U_BtuRefineBoxMode,

			U_BtuRunAction,
			U_BtuConfAllBtu,

			U_BtuSaveLabelBtu
			 
		};

		struct LineDrawContext{
			bool line_isPressed; 
			QPoint line_LeftStartPos;
			QPoint line_MovePos;
			QPoint line_EndPos;
			std::vector<QPoint> line_allLines;
		};

		//------------------------------
		State currentState;	

		QPoint lastPos;
		Qt::MouseButton lastBtu;
		
		int panelWidth;
		int panelHeight;
		
		//------------------------------
		bool imageIsLoaded;
		bool floorImgIsLoaded;
		bool wallImgIsLoaded;

		int targetImgID;

		//------------------------------
		// SP flat
		bool layoutIsLoaded;
		bool spIsLoaded;
		bool spIsFilted;
		//------------------------------
		// UI vis flag 
		bool ui_showBox;
		bool ui_showSeg;
		bool ui_showPredRes;


		//------------------------------

		bool exBoxLoaded;
		bool exBoxIsDone;
		//------------------------------
		//BOX data
		QList<QObject*> boxDataQList;
		std::vector<BoxData> boxDatas;
		 
		bool predicIsMaded;
		//------------------------------
		 
		std::map<std::string,int>  nyuLabelMap; 
		std::vector<std::string>  nyuLabelList;
		//------------------------------

		int spBrushSize;

		//----------------------------------

		const char * finalSavPath;
		const char * mixmodelPath;
		const char * boxPreSavPath;

		//----------------------------------
		QImage layMaskQimg;
		QImage spEdgeMatQimg;
		QImage objGroupImg;  // filtered , reload as cut-map
		cv::Mat objGroupCvImg;
		//QPixelmap objPixelImg;  // filtered , reload as cut-map
		 
		QImage boxes2dQImg;
		QImage cutSPL3_mid; //obj group Id mask,for record obj id

		QImage allWallQimg;
		QImage floorQimg;
		QImage inputQImg;

		//------------------------------
		// use gt segment mode
		QImage objLabelImg_gt;
		QImage objLabelGidImg_gt;
		QImage objVisBK_gt;

		bool gt_objMapIsLoaded;

		//------------------------------
		LineDrawContext lineDrawCtx;
		 
		//------------------------------
		// 3d data
		// // watch out for the array being reallocated 
		std::vector<std::vector<GLfloat>> boxVertices_aft; 
		std::vector<std::vector<GLfloat>> boxVertices_bef; 

	};

	//for debug print
	extern const char* MODESTRS[StateMag::NUM_OF_STATE];


};