#include "stdafx.h"
#include "svmproxy.h"

svm_node ** makeSparse( std::vector<std::vector<double> > const &features )
{
	int num_of_samples = features.size(); 

	// non-zeros entries + ending term
	//svm_node * f_data = new svm_node[num_of_nzerosData+1*num_of_samples];

	svm_node **f_data_ptr = new svm_node*[num_of_samples];

	for( int ir=0 ; ir < features.size() ; ir++ ){

		int eleNum = features[ir].size();

		f_data_ptr[ir]= new svm_node[eleNum+1];

		int lidx=0;
		for( int ic=0 ; ic < features[ir].size() ; ic++ ){

			if( features[ir][ic] !=0 ){
				// the index is start from 1
				f_data_ptr[ir][lidx].index=ic+1;
				f_data_ptr[ir][lidx].value=features[ir][ic] ;
				lidx=lidx+1;
			}
		}

		//myassert(lidx==eleNum);
		(f_data_ptr[ir]+eleNum)->index=-1;
		(f_data_ptr[ir]+eleNum)->value=0;

	}
	return f_data_ptr;
}

svm_node ** makeSparse( Eigen::MatrixXd features )
{
	int num_of_samples = features.rows();
	int num_of_nzerosData = features.nonZeros();

	// non-zeros entries + ending term
	//svm_node * f_data = new svm_node[num_of_nzerosData+1*num_of_samples];

	svm_node **f_data_ptr = new svm_node*[num_of_samples];

	for( int ir=0 ; ir < features.rows() ; ir++ ){

		int eleNum = features.row(ir).nonZeros();

		f_data_ptr[ir]= new svm_node[eleNum+1];

		int lidx=0;
		for( int ic=0 ; ic < features.cols() ; ic++ ){

			if( features(ir,ic)!=0 ){
				// the index is start from 1
				f_data_ptr[ir][lidx].index=ic+1;
				f_data_ptr[ir][lidx].value=features(ir,ic);
				lidx=lidx+1;
			}
		}

		//myassert(lidx==eleNum);
		(f_data_ptr[ir]+eleNum)->index=-1;
		(f_data_ptr[ir]+eleNum)->value=0;

	}
	return f_data_ptr;
}

svm_parameter makeParm( double feature_size )
{
	// train
	svm_parameter param;
	param.svm_type = C_SVC;
	param.kernel_type = RBF;
	param.degree = 3;
	param.gamma = 1/feature_size;
	param.coef0 = 0;
	param.C = 1;
	param.nu = 0.5;
	param.p = 0.1;
	param.cache_size = 100;
	param.eps = 1e-3;
	param.shrinking = 1;
	param.probability = 1;

	param.nr_weight = 0;
	param.weight_label = NULL;
	param.weight = NULL;

	return param;
}

void deleteSparse( svm_node ** data , int ROW_NUM )
{
	for( int k=0; k < ROW_NUM ; k++ ){

		delete [] data[k];
	}
	delete data;
}



// train the model
// @return svm_model* model_ptr
void svmTrain( std::vector<double> & lids ,  std::vector< std::vector<double> > & fdata,
	svm_model ** model_out, svm_problem ** prob_out   )
{
	
	assert(lids.size()==fdata.size());
	
	// prepare data
	int sizeOfProblem = lids.size(); //number of lines with labels
	int feature_size = fdata[0].size(); //number of features for each data vector
	 

	double * output_label = new double [sizeOfProblem];
	for (int i=0; i < lids.size(); ++i)
	{
		output_label[i] = lids[i]; 
	} 

	svm_problem* prob=new svm_problem;
	prob->l= sizeOfProblem;
	prob->y = output_label; 
	prob->x = makeSparse(fdata);
	 
	svm_parameter param = makeParm(feature_size);

	const char* checkRes = svm_check_parameter(prob,&param);
	if(checkRes) {
		printf("ERROR in validating conversion: %s\n", checkRes);
	} 

	//[Train]
	svm_model* model_ptr = svm_train(prob,&param);


	// param may use WEIGHTING structure
	//svm_destroy_param(&param);
	
	*model_out= model_ptr;
	*prob_out = prob;
}

void svmSave( const char* SAVPATH, const svm_model* model_ptr)
{ 
	svm_save_model(SAVPATH,model_ptr);
	
	//svm_model* reloadModel = svm_load_model("");

	std::cout<<"SAVE at :"<<SAVPATH<<std::endl;
}

void svmPred_track( svm_model* model_ptr,Eigen::MatrixXd const & FVEC_IN,
			 std::vector<int> & pResLids ,
			 std::vector< std::vector< SVMPResElement > > & pResProbs_std 			 )
{

	// assume fvec is Nxk_dim 

	const int TOTAL_N = FVEC_IN.rows() ;

	// sparse representation
	svm_node ** fvec_in =makeSparse(FVEC_IN);

	pResLids.clear();
	pResLids.resize(TOTAL_N);

	pResProbs_std.clear();
	pResProbs_std.resize(TOTAL_N);

	// batch prediction 
	double * predRes = new double[model_ptr->nr_class];

	for( int k=0 ; k<TOTAL_N ; k++ ){

		memset(predRes,0,sizeof(double)*model_ptr->nr_class);

		// [predict]
		double mlabel = svm_predict_probability( model_ptr , fvec_in[k] ,predRes);
		int maxLabel = static_cast<int>(mlabel);
		assert(maxLabel==mlabel);

		pResLids[k]=maxLabel;

		pResProbs_std[k].resize(model_ptr->nr_class);

		for( int i=0 ; i < model_ptr->nr_class ; i++ ){
			// pResProbs(k,i)=predRes[i];

			pResProbs_std[k][i].label=model_ptr->label[i];
			pResProbs_std[k][i].prob=predRes[i]; 
			pResProbs_std[k][i].labelLIdx=i;

		}
	}

	delete [] predRes;
	deleteSparse( fvec_in , TOTAL_N );

}

void svmPred( svm_model* model_ptr,Eigen::MatrixXd const & FVEC_IN,
			 std::vector<double> & pResLids ,
			 Eigen::MatrixXd & pResProbs ,
			 std::vector< std::vector<double> > & pResProbs_std 
			 )
{

	// assume fvec is Nxk_dim 

	const int TOTAL_N = FVEC_IN.rows() ;

	// sparse representation
	svm_node ** fvec_in =makeSparse(FVEC_IN);

	pResLids.clear();
	pResLids.resize(TOTAL_N);
	
	pResProbs.resize(TOTAL_N,model_ptr->nr_class);
	pResProbs.setZero();
	
	pResProbs_std.clear();
	pResProbs_std.resize(TOTAL_N);

	// do prediction 
	double * predRes = new double[model_ptr->nr_class];

	for( int k=0 ; k<TOTAL_N ; k++ ){

		memset(predRes,0,sizeof(double)*model_ptr->nr_class);
		         
		double maxLabel = svm_predict_probability( model_ptr , fvec_in[k] ,predRes);

		pResLids[k]=maxLabel;
		
		pResProbs_std[k].resize(model_ptr->nr_class);

		for( int i=0 ; i < model_ptr->nr_class ; i++ ){
			pResProbs(k,i)=predRes[i];
			pResProbs_std[k][i]=predRes[i];
		}
	}
	
	delete [] predRes;
	deleteSparse( fvec_in , TOTAL_N );
  
}

void svmPred( svm_model* model_ptr,Eigen::MatrixXd const & FVEC_IN,
			 std::vector<double> & pResLids ,
			 Eigen::MatrixXd & pResProbs 
			 )
{

	// assume fvec is Nxk_dim 

	const int TOTAL_N = FVEC_IN.rows() ;

	// sparse representation
	svm_node ** fvec_in =makeSparse(FVEC_IN);

	pResLids.clear();
	pResLids.resize(TOTAL_N);

	pResProbs.resize(TOTAL_N,model_ptr->nr_class);
	pResProbs.setZero();
	 

	// do prediction 
	double * predRes = new double[model_ptr->nr_class];

	for( int k=0 ; k<TOTAL_N ; k++ ){

		memset(predRes,0,sizeof(double)*model_ptr->nr_class);

		double maxLabel = svm_predict_probability( model_ptr , fvec_in[k] ,predRes);

		pResLids[k]=maxLabel;
		 

		for( int i=0 ; i < model_ptr->nr_class ; i++ ){
			pResProbs(k,i)=predRes[i]; 
		}
	}

	delete [] predRes;
	deleteSparse( fvec_in , TOTAL_N );


}

// free the memory
void svmFree(svm_model* model, svm_problem*prob)
{

	svm_free_model_content(model);
	
	// free memory
	deleteSparse(prob->x,prob->l);
	delete [] prob->y; // prob.y 
}
// free the memory
void svmFree(svm_model* model )
{
	if( model == nullptr ){
		return;
	}

	svm_free_model_content(model);

	 
}

bool SVMPResEle_desComp(  SVMPResElement const & L,  SVMPResElement const& R )
{
	return L.prob > R.prob;
}

SVMProxy::SVMProxy()
{
	model=nullptr;
}

void SVMProxy::loadModel( const char* MPATH , std::vector<std::string> const & LID_LIST )
{
	if( model !=nullptr ){
		svmFree(model);
		model=nullptr;
	}
	 
	std::vector<double> lids;
	std::vector<std::vector<double> >fmat;

	//
	// NOTE that the array index start from 0
	readMatBinData(MPATH,lids,fmat);

	model=nullptr;
	svm_problem * prob=nullptr;

	//[ TRAIN ]
	svmTrain(lids,fmat,&model,&prob);
 
	// update Lid,Lnames
	svmLabelName.clear();
	svmLabelList.clear();

	for( int i=0; i<model->nr_class ; i++ ){

		svmLabelList.push_back(model->label[i]);
		
		std::string lname=LID_LIST[model->label[i]];

		svmLabelName.push_back( lname );
	}

}
  

void SVMProxy::pred( Eigen::MatrixXd const & FVEC )
{ 
	svmPred_track(model,FVEC,lastRes,lastResProb);
	lastResProb_sorted=lastResProb;

	//Sorts the elements in the range [first,last) into Desscending order.
	for( int i=0 ; i < lastResProb_sorted.size() ; i++ ){
		std::sort(lastResProb_sorted[i].begin(),lastResProb_sorted[i].end(),SVMPResEle_desComp);
	}
}

SVMProxy::~SVMProxy()
{
	if( model ){
		svmFree(model);
		model=nullptr;
	}
}

