#pragma once


#include <Eigen/Eigen> 
#include <vector>
#include <fstream>
#include <cstdio>
#include <iostream>
#include <map>
#include <svm.h>
#include <iomanip>
#include <string>
#include <algorithm>
#include "common_tool.h"

struct SVMPResElement
{
public:
	int label;
	int labelLIdx;
	double prob;
};
 

bool SVMPResEle_desComp (  SVMPResElement const &,  SVMPResElement const &);

svm_node ** makeSparse(std::vector<std::vector<double> > const &features);

svm_node ** makeSparse(Eigen::MatrixXd features);

void deleteSparse(svm_node ** data , int ROW_NUM);

svm_parameter makeParm( double feature_size );
 
void svmTrain( std::vector<double> & lids , std::vector< std::vector<double> > & fdata, svm_model ** model_out, svm_problem ** prob_out );

void svmSave( const char* SAVPATH, const svm_model* model_ptr);
 
void svmPred( svm_model* model_ptr, Eigen::MatrixXd const & FVEC_IN, std::vector<double> & pResLids , Eigen::MatrixXd & pResProbs );

void svmPred( svm_model* model_ptr,Eigen::MatrixXd const & FVEC_IN, std::vector<double> & pResLids , 
			  Eigen::MatrixXd & pResProbs , std::vector<std::vector<double> > & pResProbs_std );

void svmPred_track( svm_model* model_ptr,Eigen::MatrixXd const & FVEC_IN, std::vector<int> & pResLids , 
				   std::vector< std::vector< SVMPResElement > > & pResProbs_std );

void svmFree(svm_model* model, svm_problem*prob);

void svmFree(svm_model* model );

int getFSize(const char* BPATH);

class SVMProxy
{
public:

	typedef std::vector< std::vector<SVMPResElement> > PLogVector;
	typedef std::vector<SVMPResElement> PLog;

	SVMProxy();

	// non trivial destr.
	~SVMProxy();

	void loadModel( const char* MPATH , std::vector<std::string> const & LID_LIST );
	//void pred(Eigen::MatrixXd const & FVEC,std::vector<double> & resLids , Eigen::MatrixXd & resProbs);

	void pred(Eigen::MatrixXd const & FVEC);
	 
	inline 
		std::vector<int> const& getLastResLabels()const{
		return lastRes;
	}
	 
	inline  
	PLogVector const &  getLastResProb(){
			return lastResProb;
	}
	inline  
		PLogVector const &  getLastResProbSorted(){
			return lastResProb_sorted;
	}
	inline
		std::vector<int> const& getSVMClassLIDList()const{
			return svmLabelList;
	}
	inline
		std::vector<std::string> const& getSVMClassNameList()const{
			return svmLabelName;
	}

private:

	//-------------------------------------------------------------------------
	
	// initial at model ini stage  
	std::vector<int> svmLabelList;

	// initial at model ini stage 
	std::vector<std::string> svmLabelName;
	 

	std::vector<int> lastRes;
	 
	PLogVector lastResProb;
	PLogVector lastResProb_sorted;

	svm_model* model;

};




