#include "stdafx.h"
#include "texturenode.h"

 
TextureNode::TextureNode( QQuickWindow *window ) : 
	m_fbo(0)
	, m_texture(0)
	, m_window(window)
	, m_program(0)
{
	connect(m_window, SIGNAL(beforeRendering()), this, SLOT(renderFBO()));
}
 

TextureNode::~TextureNode()
{
	delete m_texture;
	delete m_fbo;
	delete m_program; 
	m_program=NULL;
}

void TextureNode::renderFBO()
{ 

	if (!m_fbo) { 
		QOpenGLFramebufferObjectFormat format;
		format.setAttachment(QOpenGLFramebufferObject::CombinedDepthStencil);
		m_fbo = new QOpenGLFramebufferObject(fbo_size, format);
		m_texture = m_window->createTextureFromId(m_fbo->texture(), fbo_size); 
		setTexture(m_texture);
	}

	m_fbo->bind();

	glViewport(0, 0, fbo_size.width(), fbo_size.height());

	//do real rendering
	render();

	//set back
	m_fbo->bindDefault();

	m_window->update();
}

void TextureNode::render()
{
	setupShader(); 

	//! [4] //! [5]
	m_program->bind();
	glViewport(0, 0, fbo_size.width(),fbo_size.height());
	glClearColor(0, 1, 0, 1);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	m_program->disableAttributeArray(0);
	m_program->release();
}

void TextureNode::setSize( int w,int h )
{
	this->setRect(0,0,w,h);
	fbo_size = rect().size().toSize();
}

void TextureNode::initial()
{
	this->setFiltering(QSGTexture::Linear);
	 
}

void TextureNode::setupShader()
{
	if (!m_program) {
		m_program = new QOpenGLShaderProgram();
		m_program->addShaderFromSourceCode(QOpenGLShader::Vertex,
			"attribute highp vec4 vertices;"
			"varying highp vec2 coords;"
			"void main() {"
			"    gl_Position = vertices;"
			"    coords = vertices.xy;"
			"}");
		m_program->addShaderFromSourceCode(QOpenGLShader::Fragment,
			"uniform lowp float t;"
			"varying highp vec2 coords;"
			"void main() {"
			"    lowp float i = 1. - (pow(abs(coords.x), 4.) + pow(abs(coords.y), 4.));"
			"    i = smoothstep(t - 0.8, t + 0.8, i);"
			"    i = floor(i * 20.) / 20.;"
			"    gl_FragColor = vec4(coords * .5 + .5, i, i);"
			"}");

		m_program->bindAttributeLocation("vertices", 0);
		m_program->link();

		m_program->enableAttributeArray(0);

		float values[] = {
			-1, -1,
			1, -1,
			-1, 1,
			1, 1
		};
		m_program->setAttributeArray(0, GL_FLOAT, values, 2);
		m_program->setUniformValue("t", (float) 1);

		//connect(window()->openglContext(), SIGNAL(aboutToBeDestroyed()),
		//	this, SLOT(cleanup()), Qt::DirectConnection);
	}
}
