#ifndef TRACKBALL_H
#define TRACKBALL_H

#include <QtGui>

#include <QtGui/qvector3d.h>
#include <QtGui/qquaternion.h>

class TrackBall
{
public:
	enum TrackMode
	{
		Plane,
		Sphere,
	};
	TrackBall(TrackMode mode = Sphere);
	TrackBall(float angularVelocity, const QVector3D& axis, TrackMode mode = Sphere);
	// coordinates in [-1,1]x[-1,1]

	void push(const QPointF& p);
	void move(const QPointF& p);
	void release(const QPointF& p);

	void push(const QPointF& p, const QQuaternion &transformation);
	void move(const QPointF& p, const QQuaternion &transformation);
	void release(const QPointF& p, const QQuaternion &transformation);
	void start(); // starts clock
	void stop(); // stops clock

	QQuaternion rotation() const;

	inline
	QPointF pixelPosToViewPos(const QPointF& p ,float w , float h)
	{
		return QPointF(2.0 * float(p.x()) / w - 1.0,
			1.0 - 2.0 * float(p.y()) / h);
	}


private:
	QQuaternion m_rotation;
	QVector3D m_axis;
	float m_angularVelocity;

	QPointF m_lastPos;
	QTime m_lastTime;
	bool m_paused;
	bool m_pressed;
	TrackMode m_mode;
};

#endif