import QtQuick 2.1
import QtQuick.Controls 1.2
import QtQuick.Layouts 1.0
import QtQuick.Dialogs 1.0
import QtQuick.Window 2.1

//
// labelTextInput - focus
//
//
//

Item{


    property var labelNames:[];

    property var actionDoneCallBack:function(newLabel){}
    property var actionCancelCallBack:function(){}

    property string iniLabel:'';

    //-------------------------------------------------------
    property var __actionDoneHandler:function(newLabel){
        actionDoneCallBack(newLabel);
        diaRoot.destroy();
    }
    property var __actionCancelHandler:function(){
        actionCancelCallBack();
        diaRoot.destroy();
    }

    property var updateSuggestList: function( newText ){
        if( sugglistModel.count > 0 ){
            sugglistModel.clear();
        }

        if(newText.length >0){

            for(var i=0 ; i < labelNames.length ; i++){
                if(labelNames[i].substr(0,newText.length) === newText ){
                    sugglistModel.append({labelName:labelNames[i]});
                }

            }
        }

    }

    property var applySuggest: function( newLabel ){
        if(newLabel!==undefined){
            labelTextInput.text=newLabel;
            updateSuggestList(newLabel);
        }
    }

    //------------------------------------------------------------------

    id:diaRoot
    anchors.fill: parent

    PropertyAnimation { target: diaRoot; property: "opacity";
        duration: 400; from: 0; to: 1;
        easing.type: Easing.InOutQuad ; running: true }

    Component.onCompleted: {

        labelTextInput.text=diaRoot.iniLabel;

        // initial
        labelNames.push('bathtub');
        labelNames.push('bed');
        labelNames.push('bookshelf');
        labelNames.push('blinds');

        labelNames.push('cabinet');
        labelNames.push('chair');
        labelNames.push('counter');

        labelNames.push('dishwasher');
        labelNames.push('dresser');

        labelNames.push('shelves');
        labelNames.push('sink');
        labelNames.push('sofa');

        labelNames.push('night stand');
        labelNames.push('lamp');

        labelNames.push('refridgerator');

        labelNames.push('television');
        labelNames.push('toilet');
        labelNames.push('printer');

        labelNames.push('microwave');
        labelNames.push('monitor');
        labelNames.push('ottoman');

        labelNames.push('pillow');
        labelNames.push('picture');

        labelNames.push('table');

        console.log(labelNames);

    }

    // modal warpper
    Rectangle {
        id: overlayRect

        anchors.fill: parent
        opacity: 0.6
        color: "#000000"

        // add a mouse area so that clicks outside
        // the dialog window will not do anything
        //*
        MouseArea {
           anchors.fill: parent
           propagateComposedEvents: false
           onClicked: {
               //console.log('modal area')
           }

        }
        //*/
    }

        // Actual Dialog
        Rectangle {
            id: dialogWindow
            width: 268
            height: 230
            radius: 10
            anchors.centerIn: parent
            z : overlayRect.z+2
            // Y direction order
            Column{
                id: column1
                anchors.horizontalCenter: parent.horizontalCenter

                //title str
                Text {
                    id: titleText
                    width: 109
                    height: 40
                    text: "Type the name of label"
                    font.bold: true
                    font.family: "Comic Sans MS"
                    font.pointSize: 13
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter

                    anchors.topMargin: 0
                    anchors.horizontalCenterOffset: 0
                    anchors.horizontalCenter: parent.horizontalCenter

                }

                // text input Field
                Row{
                    anchors.horizontalCenter: parent.horizontalCenter
                    TextInput{
                        id: labelTextInput
                        width:150
                        height:30
                        text:'initial'
                        horizontalAlignment: TextInput.AlignHCenter
                        font.family: "Comic Sans MS"
                        font.pixelSize: 16

                        focus:true

                        onTextChanged: {
                            //console.log(text);
                            diaRoot.updateSuggestList(text);
                        }

                        Keys.onPressed: {
                            if (event.key === Qt.Key_Down ) {
                                sugView.incrementCurrentIndex();
                                //.log('Key_Down :' + sugView.currentIndex );

                                event.accepted = true;
                            }else if (event.key === Qt.Key_Up ) {

                                sugView.decrementCurrentIndex()
                                //console.log('Key_Up'  + sugView.currentIndex );

                                event.accepted = true;
                            }else if(event.key === Qt.Key_Enter || event.key === 16777220 ){
                                //console.log('Key_Enter ' + sugView.currentItem);

                                if(sugglistModel.get(sugView.currentIndex)!==undefined){
                                  diaRoot.applySuggest(sugglistModel.get(sugView.currentIndex).labelName);
                                }

                                event.accepted = true;
                                diaRoot.__actionDoneHandler(labelTextInput.text);
                            }else{
                               // console.log('key:'+event.key);
                            }
                        }
                    }
                }

                // suggestion
                Rectangle {
                    id: sugWrapperRect

                    width: 180; height: 200
                    anchors.centerIn: parent.Center
                    color:'transparent'
                    //color:'red'
                    anchors.horizontalCenter: parent.horizontalCenter

                    ListView{
                        anchors.fill: parent
                        id:sugView
                        model:sugglistModel
                        delegate: sugDelegeter
                        highlight: highlight

                        highlightFollowsCurrentItem: false
                        focus: true
                    }

                }

                // button
                Row{
                    spacing: 20
                    MyButton{
                        text:'OK'
                        onClicked: {
                            diaRoot.__actionDoneHandler(labelTextInput.text);
                        }
                    }
                    MyButton{

                        text:'Cancel'
                        onClicked: {
                            diaRoot.__actionCancelHandler();
                        }

                    }
                }//button row
            }

        }//Actual Dialog RECT


    ListModel {
        id: sugglistModel
        /*
        ListElement { labelName: "Banana" }
        ListElement { labelName: "Orange" }
        ListElement { labelName: "Apple" }
        ListElement { labelName: "Coconut" }
        */
    }

    Component {
        id:sugDelegeter

        Rectangle{
            width:100
            height:35
            color: 'transparent'
            anchors.horizontalCenter: parent.horizontalCenter

            //opacity:0.5
            Text{
                color:Qt.rgba(0.5,0.5,0.5,0.8)
                text:labelName
                font.family: "Comic Sans MS"
                font.italic: true
                font.pixelSize: 20

                anchors.horizontalCenter: parent.horizontalCenter

                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
            }
        }
    }

    Component {
         id: highlight
         Rectangle {
             width: sugWrapperRect.width
             height: 25

             color: "lightsteelblue"; radius: 3
             //anchors.horizontalCenter: sugWrapperRect.horizontalCenter

             y: sugView.currentItem.y === undefined ? 0 : sugView.currentItem.y
             visible: sugView.currentItem.y !== undefined

         }
     }

}














