#include "myqtgl.h"

MyQtGL::MyQtGL(QWidget *parent)
	: QGLWidget(parent)
{
	ui.setupUi(this);

	sp1_test=nullptr;
	sp2_nyu=nullptr;
}

MyQtGL::~MyQtGL()
{

	if( sp1_test != nullptr ){
		delete sp1_test;
	}

}
void MyQtGL::initializeGL()
{
	QGLWidget::initializeGL();
	
	qglEntery = context()->functions();
	 
	qglEntery->initializeGLFunctions();

	glClearColor(0.0, 0.0, 0.0, 0.0);
	glEnable(GL_DEPTH_TEST);


	ini_nyu();
}

void MyQtGL::paintGL()
{ 

	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	
	//paintSimpleTest();

	paintNYU();
	

}

void MyQtGL::resizeGL( int width, int height )
{
	glViewport(0, 0, (GLint)width, (GLint)height);

	panelWidth=width;
	panelHeight=height;

	autoConstruct();
}


void MyQtGL::ini_sp2_nyu()
{

	if( sp2_nyu != nullptr ){ 
		throw "wired state"; 
		delete sp2_nyu;
	}

	sp2_nyu=new QGLShaderProgram;
	sp2_nyu->addShaderFromSourceCode(QGLShader::Vertex,
		//"#version 330\n"
		"uniform highp mat4 mvp_matrix; \n"
		"attribute highp vec4 a_position; \n" 
		"uniform lowp float point_size; \n" 
		"\n"
		"\n"
		"void main() { \n"
		"	 gl_PointSize = point_size; \n"
		"    gl_Position = mvp_matrix * a_position; \n" 
		"}\n");

	sp2_nyu->addShaderFromSourceCode(QGLShader::Fragment,
		//"#version 330\n" 
		"uniform highp vec4 lineColor;\n" //RGBA 
		"void main() { \n" 
			"\n"
		"  gl_FragColor = lineColor;   \n"
		"}"); 

	bool res=sp2_nyu->link();

	if( !res ){
		std::cout<<"link fail :"<<sp2_nyu->log().toStdString()<<std::endl;
		throw "link fail "; 
	}
}

void MyQtGL::paintNYU()
{
	if( sp2_nyu == nullptr ){
		ini_sp2_nyu();
	} 
	glPushAttrib(GL_DEPTH_BUFFER_BIT);
	//-------------------------------------------------------------------------

	glViewport(0,0, panelWidth , panelHeight);  	
	
	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST); 
	glEnable(GL_PROGRAM_POINT_SIZE);

	glDepthMask(true); 

	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);  
	//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	//-------------------------------------------------------------------------
	// indexes and vertices 
	static GLubyte boxIdx[] = { 
		1-1,2-1,   2-1,3-1,  3-1,4-1,  4-1,1-1,  
		1-1,5-1,   2-1,6-1,  3-1,7-1,  4-1,8-1, 
		5-1,6-1,   6-1,7-1,  7-1,8-1,  8-1,5-1 
	}; 
	static GLubyte boxVexIdx[] = { 
		0,1,2,3,4,5,6,7
	}; 
	static GLubyte boxTopFace[] = { 
		0,1,2,3
	}; 
	static GLfloat test_vertices[]={
		-0.789352  , 0.228232   , 3.51174,
		0.00600265 , -0.150193  ,  1.79513,
		1.66237    , -0.0659316 ,   2.544,
		0.867015   , 0.312494   , 4.26061,
		-0.817103  , -0.459625  ,  3.65052,
		-0.0217491 , -0.838051  ,  1.93391,
		1.63462    , -0.753789  ,  2.68278,
		0.839263   , -0.375364  ,  4.39939
	};

	//-------------------------------------------------------------------------
	// MVP
	static QMatrix4x4 mvp=oglMVP; 

	//-------------------------------------------------------------------------
	QGLShaderProgram  * curShader=sp2_nyu;
	curShader->bind();

	static int mvpLoc = curShader->uniformLocation("mvp_matrix");
	static int lineColorLoc = curShader->uniformLocation("lineColor");
	static int pointSizeLoc = curShader->uniformLocation("point_size");

	int vexAttrSlot = curShader->attributeLocation("a_position");   

	curShader->setUniformValue(lineColorLoc,1,1,1,1);
	curShader->setUniformValue(mvpLoc, mvp); 

	curShader->enableAttributeArray(vexAttrSlot);   
	 
	glLineWidth(1);

		//draw call

	// box vertices 
	/*
	GLubyte tmpIdx=0;		 
	curShader->setAttributeArray(vexAttrSlot, GL_FLOAT , test_vertices, 3 ); 
	curShader->setUniformValue(lineColorLoc,0.0f,1.0f,0.0f,1.0f); 		
	curShader->setUniformValue(pointSizeLoc, 3.0f); // must use float
	glDrawElements(GL_POINTS , sizeof(boxVexIdx)/sizeof(GLubyte) , GL_UNSIGNED_BYTE ,boxVexIdx);    
	*/	 
	// first vex
	curShader->setAttributeArray(vexAttrSlot, GL_FLOAT , test_vertices, 3 ); 
	curShader->setUniformValue(pointSizeLoc, 3.0f); // must use float
	curShader->setUniformValue(lineColorLoc,1.0f,0.0f,0.0f,1.0f); 		
	glDrawElements(GL_POINTS , 1 , GL_UNSIGNED_BYTE ,boxVexIdx);    


	// original box outline
	curShader->setUniformValue(pointSizeLoc, 1.0f); 
	curShader->setUniformValue(lineColorLoc,0,1,0,1); 
	curShader->setAttributeArray(vexAttrSlot, GL_FLOAT ,  test_vertices,3 ); 
	glDrawElements(GL_LINES , sizeof(boxIdx)/sizeof(GLubyte) , GL_UNSIGNED_BYTE ,boxIdx);    
	
	// top face
	curShader->setUniformValue(pointSizeLoc, 1.0f); 
	curShader->setUniformValue(lineColorLoc,0,1,0,1); 
	curShader->setAttributeArray(vexAttrSlot, GL_FLOAT ,  test_vertices,3 ); 
	//glDrawElements(GL_TRIANGLE_FAN , sizeof(boxTopFace)/sizeof(GLubyte) , GL_UNSIGNED_BYTE ,boxTopFace);    


	//-------------------------------------------------------------------------
	// re-construct

	int vptsNum = vpts.size()/3;

	if( vptsNum ){
		curShader->setUniformValue(pointSizeLoc, 3.0f); 
		curShader->setUniformValue(lineColorLoc,1,0.5,0.2,1); 

		curShader->setAttributeArray(vexAttrSlot, GL_FLOAT , vpts.data(), 3 ); 

		GLuint *idx= new GLuint[vptsNum];

		for( int i=0 ; i < vptsNum ; i++ ){
			idx[i]=i;
		}

		glDrawElements(GL_POINTS ,vptsNum  , GL_UNSIGNED_INT ,idx);    

		delete idx;
	} 
	//-------------------------------------------------------------------------
	//plane grid

	curShader->setUniformValue(pointSizeLoc, 1.0f); 
	curShader->setUniformValue(lineColorLoc,0,0.0,1,1); 

	curShader->setAttributeArray(vexAttrSlot, GL_FLOAT , plane_grid.data(), 3 ); 
	
	unsigned int gnum = plane_grid.size()/3;
	GLuint *idx= new GLuint[gnum];
	for( unsigned int i=0 ; i < gnum ; i++ ){
		idx[i]=i;
	}

	glDrawElements(GL_LINES ,gnum  , GL_UNSIGNED_INT ,idx);    
	delete idx;


	//-------------------------------------------------------------------------

	glPopAttrib();
	glDisable(GL_PROGRAM_POINT_SIZE);

	curShader->disableAttributeArray(vexAttrSlot);  
	curShader->release();
	 
}

void MyQtGL::ini_nyu()
{
	// Matlab data is column-major
	float mvp[16] = {
		//[640x480] 1.6214309411703185,0,0,0,0,2.1644567130053121,0,0,0.017445154409969854,-0.057234026391685955,1.2222222222222223,1,0,0,-2.2222222222222223,0
		// 1.8497607885008978,0,0,0,0,2.4331129326523411,0,0,0.00029393729479607056,0.0035776752505636278,1.2222222222222223,1,0,0,-2.2222222222222223,0
		1.8497607885008978,0,0,0,0,2.4331129326523411,0,0,0.00029393729479607056,0.0035776752505636278,1.002002002002002,1,0,0,-2.0020020020020022,0
	};

	// this is ..top-left center
	float world2img[16]={
		518.85790117450188,0,0,0,0,-519.46961112127485,0,0,280.58244941119028,212.73616633400468,1.002002002002002,1,0,0,-2.0020020020020022,0
	};

	//ctx.TX_world2img=Eigen::Map<Eigen::Matrix4f >(world2img); 

	oglMVP=QMatrix4x4(mvp);
	// Qmat use row-major order.
	oglMVP=oglMVP.transposed();

	//ctx.panelHeight= 471-45+1;
	//ctx.panelWidth = 601-41+1;

}


void MyQtGL::ini_sp1()
{ 

	if( sp1_test != nullptr ){

		throw "wired state";

		delete sp1_test;
	}

	sp1_test=new QGLShaderProgram;
	sp1_test->addShaderFromSourceCode(QGLShader::Vertex,
		//"#version 130\n"
		"attribute highp vec4 vertex;\n"
		"uniform mediump mat4 matrix;\n"
		""
		"void main(void)\n"
		"{\n"
		"   gl_Position = matrix * vertex;\n"
		"}");
	sp1_test->addShaderFromSourceCode(QGLShader::Fragment,
		//"#version 130\n"
		"uniform mediump vec4 color;\n"
		//"out mediump vec4 out_color;\n"
		""
		"void main(void)\n"
		"{\n"
		"   gl_FragColor = color;\n"
		"}");
	sp1_test->link();
	sp1_test->bind();
}


void MyQtGL::paintSimpleTest()
{
	if( sp1_test == nullptr ){
		ini_sp1();
	}

	QGLShaderProgram & curShader = *sp1_test;

	curShader.bind();

	int vertexLocation = curShader.attributeLocation("vertex");
	int matrixLocation = curShader.uniformLocation("matrix");
	int colorLocation = curShader.uniformLocation("color");

	QVector3D triangleVertices[] = {
		QVector3D(60.0f,  10.0f,  0.0f),
		QVector3D(110.0f, 110.0f, 0.0f),
		QVector3D(10.0f,  110.0f, 0.0f)
	};

	QMatrix4x4 pmvMatrix;
	pmvMatrix.ortho(rect());

	curShader.enableAttributeArray(vertexLocation);

	curShader.setAttributeArray(vertexLocation, triangleVertices);

	curShader.setUniformValue(matrixLocation, pmvMatrix);
	curShader.setUniformValue(colorLocation, QColor(0, 255, 0, 255));

	glDrawArrays(GL_TRIANGLES, 0, 3);

	curShader.disableAttributeArray(vertexLocation); 

	//sp1_vex->disableAttributeArray(vexAttrSlot); 
	//sp1_vex->disableAttributeArray(texCoordSlot); 
	curShader.release();
}

void MyQtGL::mouseMoveEvent( QMouseEvent * event )
{

}

void MyQtGL::mousePressEvent( QMouseEvent * event )
{

	int mx=event->x();
	int my=panelHeight-event->y()-1;

	std::cout<<"pressed at:"<<mx<<','<<my<<"("<<event->y()<<")"<<std::endl;
	constrDepthSensorV2(mx,my);

	update();
}

void MyQtGL::mouseReleaseEvent( QMouseEvent * event )
{

}

void MyQtGL::constrDepthSensor(int mx , int my)
{


	Eigen::Vector3f v1,v2,v3;
	v1<<-0.789352  , 0.228232   , 3.51174;
	v2<<0.00600265 , -0.150193  ,  1.79513;
	v3<<1.66237    , -0.0659316 ,   2.544;

	Eigen::Vector3f nv = (v2-v1).cross(v3-v1);
	nv.normalize();
	//*
	float pl_d = -1*nv.dot(v1);
	float pl_a = nv.x();
	float pl_b = nv.y();
	float pl_c = nv.z();


	//std::cout<<"pl:"<<nv.dot(v2)+pl_d<<std::endl;

	//*/
	Eigen::Matrix4d A; 
	Eigen::Vector4d B;
	
	//float pl_a=0.0395175, pl_b=0.979483, pl_c=-0.197617, pl_d=1.19472;

	double term_x,term_y;

	term_x = (2.0*mx/panelWidth  - 1);
	term_y = (2.0*my/panelHeight - 1);

	A << oglMVP(3-1,1-1),oglMVP(3-1,2-1),oglMVP(3-1,3-1),-1,
 		 oglMVP(2-1,1-1),oglMVP(2-1,2-1),oglMVP(2-1,3-1),-term_y,
		 oglMVP(1-1,1-1),oglMVP(1-1,2-1),oglMVP(1-1,3-1),-term_x,
		 pl_a,pl_b,pl_c,0;
		 

	B << -1*oglMVP(3-1,4-1),oglMVP(2-1,4-1),oglMVP(1-1,4-1),-pl_d;

	Eigen::Vector4d sol = A.colPivHouseholderQr().solve(B);

	//std::cout << "Here is the matrix A:\n" << A << std::endl;
	//std::cout << "Here is the vector b:\n" << B << std::endl;
	
	std::cout << "The solution is:\n" <<  sol.transpose() <<  "\n\n";

	vpts.push_back(sol.x());
	vpts.push_back(sol.y());
	vpts.push_back(sol.z());


	//-------------------------------------------------------------------------
	float mvp[16] = {
		//[640x480] 1.6214309411703185,0,0,0,0,2.1644567130053121,0,0,0.017445154409969854,-0.057234026391685955,1.2222222222222223,1,0,0,-2.2222222222222223,0
		// 1.8497607885008978,0,0,0,0,2.4331129326523411,0,0,0.00029393729479607056,0.0035776752505636278,1.2222222222222223,1,0,0,-2.2222222222222223,0
		1.8497607885008978,0,0,0,0,2.4331129326523411,0,0,0.00029393729479607056,0.0035776752505636278,1.002002002002002,1,0,0,-2.0020020020020022,0
	};
	Eigen::Matrix4f emvp=Eigen::Map<Eigen::Matrix4f >(mvp); 
	Eigen::Vector4f ee;
	ee<< -0.789352,0.228232,3.51174,1;

	Eigen::Vector4f eNDC_pre = emvp*ee;
	Eigen::Vector4f eNDC = eNDC_pre/eNDC_pre(3);

	float img_X = (eNDC.x()   +1 )*panelWidth/2.0;
	float img_Y = (eNDC.y()  +1 )*panelHeight/2.0;

	std::cout<<"img_X= "<<img_X<<"\n";
	std::cout<<"img_Y= "<<img_Y<<"\n";
	std::cout<<"eNDC= "<<(eNDC_pre).transpose()<<"\n\n";
	std::cout<<"eNDC h= "<<(eNDC).transpose()<<"\n\n";
	std::cout<<"mvp: \n"<<emvp<<"\n";

}

void MyQtGL::constrDepthSensorV2(int mx , int my)
{


	float pl_a=0.0395175, pl_b=0.979483, pl_c=-0.197617, pl_d=1.19472;

	Eigen::Matrix3d A; 
	Eigen::Vector3d B;
	
	double term_x,term_y;

	term_x = (2.0*mx/panelWidth  - 1);
	term_y = (2.0*my/panelHeight - 1);

	A << oglMVP(1-1,1-1),oglMVP(1-1,2-1),oglMVP(1-1,3-1)-term_x,
		 oglMVP(2-1,1-1),oglMVP(2-1,2-1),oglMVP(2-1,3-1)-term_y,
		 pl_a,pl_b,pl_c;
		 

	B << -1*oglMVP(1-1,4-1),-1*oglMVP(2-1,4-1),-1*pl_d;

	Eigen::Vector3d sol = A.colPivHouseholderQr().solve(B);

	//std::cout << "Here is the matrix A:\n" << A << std::endl;
	//std::cout << "Here is the vector b:\n" << B << std::endl;
	
	std::cout << "The solution is:\n" <<  sol.transpose() <<  "\n\n";

	vpts.push_back(sol.x());
	vpts.push_back(sol.y());
	vpts.push_back(sol.z());

	 

}

Eigen::Vector3d MyQtGL::constrDepthSensorV3(int mx , int my)
{
	//floor
	float pl_a=0.0395175, pl_b=0.979483, pl_c=-0.197617, pl_d=1.19472;

	Eigen::Matrix3d A; 
	Eigen::Vector3d B;
	
	double term_x,term_y;

	term_x = (2.0*mx/(1.0*panelWidth)  - 1);
	term_y = (2.0*my/(1.0*panelHeight) - 1);

	A << oglMVP(1-1,1-1),oglMVP(1-1,2-1),oglMVP(1-1,3-1)-term_x,
		 oglMVP(2-1,1-1),oglMVP(2-1,2-1),oglMVP(2-1,3-1)-term_y,
		 pl_a,pl_b,pl_c;
		 

	B << -1*oglMVP(1-1,4-1),-1*oglMVP(2-1,4-1),-1*pl_d;

	Eigen::Vector3d sol = A.fullPivHouseholderQr().solve(B);

	return sol;
}

void MyQtGL::autoConstruct()
{

	plane_grid.clear();

	const int dx=20;
	const int dy=20;

	Eigen::Vector3d pts;
	/*
	plane_grid.push_back( -0.53161);
	plane_grid.push_back(-0.703989);
	plane_grid.push_back(2.45003);

	plane_grid.push_back( -0.53161);
	plane_grid.push_back(-0.703989);
	plane_grid.push_back(150);
	*/

	///*
	for( int i=10; i<panelWidth-10 ; i=i+dx ){

		pts=constrDepthSensorV3(i,10);

		plane_grid.push_back(pts.x());
		plane_grid.push_back(pts.y());
		plane_grid.push_back(pts.z());

		pts=constrDepthSensorV3(i, panelHeight-10 );
		
		if( pts.z() < 0 ){
			plane_grid.push_back(pts.x());
			plane_grid.push_back(pts.y());
			plane_grid.push_back(-1*pts.z());
		}else{
			plane_grid.push_back(pts.x());
			plane_grid.push_back(pts.y());
			plane_grid.push_back(pts.z());

		}

	} 
	//*/
	/*
	for( int j=0; j<panelHeight ; j=j+dy ){
		pts=constrDepthSensorV3(0,j);

		plane_grid.push_back(pts.x());
		plane_grid.push_back(pts.y());
		plane_grid.push_back(pts.z());

		pts=constrDepthSensorV3(panelWidth-1,j);

		plane_grid.push_back(pts.x());
		plane_grid.push_back(pts.y());
		plane_grid.push_back(pts.z());
	}


	pts=constrDepthSensorV3(0,0);
	plane_grid.push_back(pts.x());
	plane_grid.push_back(pts.y());
	plane_grid.push_back(pts.z());

	pts=constrDepthSensorV3(0,panelHeight-1);
	plane_grid.push_back(pts.x());
	plane_grid.push_back(pts.y());
	plane_grid.push_back(pts.z());


	pts=constrDepthSensorV3(panelWidth-1,0);
	plane_grid.push_back(pts.x());
	plane_grid.push_back(pts.y());
	plane_grid.push_back(pts.z());

	pts=constrDepthSensorV3(panelWidth-1,panelHeight-1);
	plane_grid.push_back(pts.x());
	plane_grid.push_back(pts.y());
	plane_grid.push_back(pts.z());
	*/
}

